@@ -2,7 +2,7 @@
 # Script:  checkdigest
 # Usage:  checkdigest
 #
-# This file may be copied without restriction.
+# This file may be copied and modified without restriction.
 #
 # ===============================================================
 # Package Requirements for this script to work:
@@ -36,7 +36,7 @@
 #            To import:  gpg --import "signers_public_key"
 
 premg="checkdigest: "
-
+CMD_NOT_FOUND=/
 # ===============================================================
 #  4.1.6.1 Control Script Execution Environment
 # ===============================================================
@@ -93,7 +93,7 @@ missing_which() {
 			;;
 		*)
 			echo "Missing $pgm" 1>&2
-			echo "/"
+			echo $CMD_NOT_FOUND
 			return 1
 			;;
 	esac
@@ -159,48 +159,196 @@ normalize_openssl_SHA1() {
 }
 
 INFO_to_digests() {
-	(
-	export awk_dig_name="$1";
-	sed    -e 's/^[[:space:][:space:]]*file.*/\
-	file/' -e 's/^[[:space:][:space:]]*control_file.*/\
-	control_file/' |
+	E_DIGTYPE="$1"	
+	export E_DIGTYPE
 	$VAWK -- '
-	BEGIN { RS = ""; FS="\n"; dig_name=ENVIRON["awk_dig_name"]; }
-	{
-	#print $0
-	if (/[ \t]*file[# \t]*/) { 
-		#printf ("HERE %s\n", $0);
-		path = field("path"); 
-		dig = field(dig_name);
-		type = field("type");
-		if (type == "f") {
-			if (dig == "") {
-				dig = "X"
+	BEGIN {
+        	# Global Variables
+        	#
+		PROGNAME="INFO_to_digests()";
+		e_digtype=ENVIRON["E_DIGTYPE"];		
+		g_exitval=0;
+		g_arridx=0;
+		g_mode_single_line=1;
+		Lines[0]="";
+		Keywords[0]="";
+		Values[0]="";
+		in_file = 0; in_control_file = 0;
+		filename="catalog/INFO";
+		while (getline g_x <filename > 0) {
+			parse_info(g_x);
+		}; process_info_object();
+	}
+	function parse_keyword_value(S, K, V,      l_v, l_k) {
+		aa_i=0;
+		while(length(S[aa_i]) > 0) {
+			l_k = get_keyword(S[aa_i]);
+			K[aa_i] = l_k;
+			check_errno();
+			l_v = get_value(S[aa_i]);
+			V[aa_i] = l_v;
+			check_errno();
+			aa_i++;
+        	}
+	}
+	function get_keyword(line) {
+                # Remove the value part and leading whitespace
+                # before the keyword
+                sub(/^[ \t]*/,"", line);
+                sub(/[ \t]+.*/,"", line);
+                return line;
+	}
+	function get_value(line,     junk) {
+                # Remove the keyword
+                g_errno=0;
+                sub(/^[ \t]*[^ \t]+[ \t]+/,"", line);
+
+                if ( line ~ /^[^"]+/) {
+                        # Unquoted Value
+                        # Remove the comment after the value
+                        sub(/#.*/,"", line);
+                        sub(/[ \t]*$/,"", line);
+                } else if ( line ~ /^".*/) {
+                        # Quoted Value
+                        # Remove the First Quote
+                        sub(/[ \t]*"/,"", line);
+                        # Remove the last Quote
+                        sub(/".*/,"", line);
+                } else {
+                        errmsg("error in get_value for [" line "]");
+                        line="";
+                        g_errno=2;
+                }
+                return line;
+	}
+	function find_value(O, keyword,        idx, retval, l_k) {
+		idx=0;
+		retval=0;
+		g_found=0;
+		while(length(O[idx]) > 0) {
+			l_k = get_keyword(O[idx])
+			if (l_k == keyword "") {
+				g_found=1;
+				return "" get_value(O[idx]);
 			}
-			printf("%s  %s\n", dig, path); 
+			idx++;
 		}
-		next;
+		g_found=0;
+		return "";
 	}
+	function print_sum(O,             v, p, mtime, att, retval) {
+		retval=0;
+		if (length(O[0]) == 0) return 0;
+		att=e_digtype;
+		v = find_value(O, att); check_errno();
+		if (g_found != 0) { 
+			att="path";
+			p = find_value(O, att); check_errno();
+			if (g_found == 0) { errmsg(att " attribute not found"); debug_print_object(O, Keywords, Values); return 1; }
+			print v "  " p
+		}
 	}
-	END {  }
-	function field(name,	i,f) {
-		#printf("NF = %d  NR = %d\n", NF, NR);
-		for (i = 1; i <= NF; i++) {
-			split($i, f, " ");
-			if (match($i, "^ *" name " ")) {
-			     sub("^ *" name " ", "", $i);
-			     return $i;
-			}
-       	 }
-		printf("error: no field %s in record\n", name)  | "cat 1>&2"
+	function debug_print_object(O, K, V) {
+		parse_keyword_value(O, K, V);
+		g_arridx=0;
+		printf("Object=[%s]\n",  K[g_arridx++]) | "cat 1>&2";
+		while(length(O[g_arridx]) > 0) {
+			printf("keyword=[%s] value=[%s]\n",  K[g_arridx], V[g_arridx]) | "cat 1>&2";
+			g_arridx++;
+		}
+	}
+	function obcopy(D, S) {
+		D[0] = S[0];
+		for(i=1; i<g_arridx; i++) {
+			D[i] = S[i];
+		}
+		D[g_arridx]=""; # This add a terminating empty string
+	}
+	function process_info_object (       i, ret) {
+		if (in_file == 1) {
+			obcopy(O_tmp_file, Lines);
+			ret = print_sum(O_tmp_file);
+			if (ret != 0) g_exitval=1;
+		} else if (in_control_file == 1) {
+			obcopy(O_tmp_file, Lines);
+			ret = print_sum(O_tmp_file);
+			if (ret != 0) g_exitval=1;
+		} else {
+			;
+		}
+		return;
+	}
+	function init_info_object (line,      i) {
+		#
+		# Got new object, Store the one that just was completed.
+		process_info_object();
+
+		#
+		# Now determine the object type from the keyword name and set the context
+		# variables.
+
+		if (line ~ /file/) {
+			# reset_context(); in_file = 1;
+			in_file = 1;
+		} else if (line ~ /control_file/) {
+			# reset_context(); in_control_file = 1;
+			in_control_file = 1;
+		} else {
+			errmsg("error: unrecognized object: " line);
+		}
+		g_arridx=0;
+		Lines[g_arridx++]=line;
+	}
+	function parse_info(rc) {
+		if (g_mode_single_line == 1 &&  rc ~ /(^[ \t]*file|^[ \t]*control_file)/) { init_info_object(rc); return;}
+		if (g_mode_single_line == 1 && rc ~ /^[\t ]*#/  ) { return; }
+		###  Quoted string all on one line
+		if (g_mode_single_line == 1 && rc ~ /^[ \t]*[a-zA-Z_]+[ \t]+"[^"]*"/ ) { process_keyword(rc); return; }
+		###  Quoted string all on more than one line
+		if (g_mode_single_line == 1 && rc ~ /^[ \t]*[a-zA-Z_]+[ \t]+"[^"]*/ ) { errmsg("INFO file: invalid input");  return; }
+		# process the intermediate line of a quoted multi-line value
+		if (g_mode_single_line == 0 && rc ~ /^[^"]*$/ ) { errmsg("INFO file: invalid input"); return; }
+		# process the last line of a quoted multi-line value
+		if (g_mode_single_line == 0 && rc ~ /^[^"]*"($|[ \t]*$)/ ) { errmsg("INFO file: invalid input"); return; }
+		###  Any line, including single line attributes
+		if (rc ~ /^[ \t]*[a-zA-Z_]/ ) { process_keyword(rc);  return; }
+	}
+	function process_keyword(line) {
+		g_mode_single_line=1;
+		Lines[g_arridx]=line;
+		g_arridx++;
+	}
+	function check_errno() {
+		if (g_errno != 0) {
+			errmsg("error: code=" g_errno);
+		}
+	}
+	function dmsg(msg) {
+		printf("%s: <debug>: %s\n", PROGNAME, msg) | "cat 1>&2"
+	}
+	function errmsg(msg) {
+		printf("%s: error: %s\n", PROGNAME, msg) | "cat 1>&2"
 	}
 	'
-	) 2>/dev/null 
 }
 
 checkINFOdigests() {
 	dig_type="$1"
 	dig_utility="$2"
+
+	case "$dig_utility" in
+		/)
+		case "$dig_type" in
+			md5sum|sha1sum)
+				return 1
+				;;
+			*)
+			return 98
+			;;
+		esac
+		;;
+	esac
+
 	(
 	cd ${package_dir}
 	case "$?" in
@@ -222,10 +370,11 @@ checkINFOdigests() {
 			;;
 	esac
 	tmpfile="${tmpdir}/checkINFOdigests"
-	"${CAT}" catalog/INFO | INFO_to_digests "$dig_type" | "${SORT}"  >"$tmpfile"
-	grep ^X $tmpfile 1>/dev/null
+	"${CAT}" catalog/INFO | INFO_to_digests "$dig_type" | "${SORT}"  | tee /tmp/unscreme_$dig_type >"$tmpfile"
+	grep ^. $tmpfile 1>/dev/null
 	case "$?" in
-		0)
+		0) ;;
+		*)
 			#
 			# digests were missing
 			#
@@ -236,9 +385,15 @@ checkINFOdigests() {
 					;;
 				sha1sum)
 					# For now, make sha1sums in the INFO file optional.
+					echo " SKIPPED (not tested)"
 					echo "checkdigest: Warning: sha1 digests not present in the INFO file" 1>&2
 					return 199
 					;;
+				sha512sum)
+					# For now, make sha512sums in the INFO file optional.
+					echo " SKIPPED (not tested)"
+					return 199
+					;;
 			esac
 			;;
 		*)
@@ -264,7 +419,7 @@ checkINFOdigests() {
 		esac
 
 	) | "${SORT}" |
-	diff $tmpfile - 1>&2
+	"${DIFF}" $tmpfile - 1>&2
 	ret=$?
 	"${RM}" -fr "$tmpdir"
 	return $ret
@@ -314,13 +469,12 @@ info2symlinks() {
 }
 
 echo_caveat() {
-		echo  -e -n "\
+		printf "\
             CAVEAT: Obtaining the signers public key from an untrusted locatation 
 	    and subsequently verifying the package does not prove authenticity,\n\
 	    it only proves the the package may be an authentic fake.  Likewise\n\
             it is always possible an imposter stole the alleged signers private key\n\
-	    and made this package.\n\
-		  " | sed -e 's/^[[:space:]][[:space:]]*//'
+	    and made this package.\n" | sed -e 's/^[[:space:]][[:space:]]*//'
 }
 
 get_dfiles() {
@@ -328,19 +482,19 @@ get_dfiles() {
 	# Get dfiles value.
 	#
 	prefixdir="$1"
-	DFILES=`${EGREP} "^[[:space:]]*dfiles[[:space:]]" "$prefixdir"/catalog/INDEX | \
+	DFILES=`${EXPAND} "$prefixdir"/catalog/INDEX | ${EGREP} "^ *dfiles " | \
 	"${TAIL}" -1 | \
-	"${SED}" -e 's/[[:space:]]*dfiles[[:space:]][[:space:]]*//'`
+	"${SED}" -e 's/ *dfiles  *//'`
 	echo "$DFILES"
 }
 
 get_option() {
 	prefixdir="$1"
 	arg="$2"
-	DFILES=`${EGREP} '^[[:space:]]*'"${arg}"'[[:space:]]' "$prefixdir"/catalog/INDEX | \
+	aOPT=`${EXPAND} "$prefixdir"/catalog/INDEX | ${EGREP} '^ *'"${arg}"' ' | \
 	"${TAIL}" -1 | \
-	"${SED}" -e 's/[[:space:]]*'"${arg}"'[[:space:]][[:space:]]*//'`
-	echo "$DFILES"
+	"${SED}" -e 's/ *'"${arg}"'  *//'`
+	echo "$aOPT"
 }
 
 get_tar_creating_utility() {
@@ -392,9 +546,45 @@ get_tar_options() {
 	echo "$xx1"
 }
 
-check_files() {
-	csum=`${CAT} ${package_dir}/catalog/dfiles/files | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
-	fsum=`${TAR} cf - ${package_dir} | ${TAR} tf - | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
+check_files2() {
+	tmpdir_f=/tmp/checkdigest.files.$$
+	"${MKDIR}" -m u+rw,o-rwx,g-rwx $tmpdir_f
+	case $? in
+		0)
+			;;
+		*)
+			echo "Error making tmpdir : $tmpdir_f"
+			not_authentic
+			exit 1
+			;;
+	esac
+	tmpfifo="$tmpdir_f"/fifo
+	"${MKFIFO}" "$tmpfifo"
+	case $? in 0) ;; *) echo "error mkfifo" 1>&2; exit 1 ;; esac
+
+	(${CAT} "${package_dir}"/catalog/dfiles/files | ${SED} -e 's@/$@@' | ${SORT} | dd of="$tmpfifo" 2>/dev/null) &
+	${TAR} cvf /dev/null "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${DIFF} -h - "$tmpfifo"
+	ret_f=$?
+	wait
+	"${RM}" -f "$tmpfifo"
+	"${RMDIR}" "$tmpdir_f"
+	return $ret_f
+}
+
+check_files_by_md5() {
+	csum=`${CAT} "${package_dir}"/catalog/dfiles/files | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
+	fsum=`${TAR} cvf /dev/null "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
+	# OLD slow way:  fsum=`${TAR} cf - ${package_dir} | ${TAR} tf - | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
+	case "$csum" in
+		"") return 1
+		;;
+	esac
+
+	case "$fsum" in
+		"") return 1
+		;;
+	esac
+
 	if [ "$csum" = "$fsum" ]; then
 		return 0
 	else
@@ -402,6 +592,25 @@ check_files() {
 	fi
 }
 
+check_files() {
+	check_files_by_md5
+	ret_aa=$?
+	check_files2
+	ret_bb=$?
+	case "$ret_aa" in
+		0) ;;
+		*) return 1
+		;;
+	esac
+
+	case "$ret_bb" in
+		0) ;;
+		*) return 1
+		;;
+	esac
+	return 0
+}
+
 create_archive_stream() {
 	tarcreateutil="$1"
 	case "$tarcreateutil" in
@@ -487,6 +696,7 @@ check_symlinks() {
 # ===============================================================
 
 GTAR=${GTAR:-"tar"}
+
 SH=/bin/sh
 AWK=`missing_which awk 2>/dev/null`
 GAWK=`missing_which gawk 2>/dev/null`
@@ -500,6 +710,7 @@ SED=`missing_which sed` || exit 1
 CAT=`missing_which cat` || exit 1
 MD5SUM=`missing_which md5sum 2>/dev/null`
 SHA1SUM=`missing_which sha1sum 2>/dev/null`
+SHA512SUM=`missing_which sha512sum 2>/dev/null`
 TAR=`missing_which $GTAR` || exit 1
 PAX=`missing_which pax 2>/dev/null`
 SORT=`missing_which sort` || exit 1
@@ -507,6 +718,9 @@ FIND=`missing_which find` || exit 1
 RM=`missing_which rm` || exit 1
 CMP=`missing_which cmp` || exit 1
 MKDIR=`missing_which mkdir` || exit 1
+MKFIFO=`missing_which mkfifo` || exit 1
+RMDIR=`missing_which rmdir` || exit 1
+DIFF=`missing_which diff` || exit 1
 OPENSSL=`missing_which openssl 2>/dev/null`
 
 if [ ! -f "$MD5SUM" ]; then
@@ -571,10 +785,6 @@ fi
 if [ "$EXECDISP" = "no_set_" ]; then
 	echo "The shell variable SW_CONTROL_TAG is not set." 1>&2
 	echo "This indicates execution was not as expected." 1>&2
-	echo "If you override this mechanism, you alone take" 1>&2
-	echo "responsibility that this script itself is authentic." 1>&2
-	echo "If you see this message unexpectedly perhaps" 1>&2
-	echo "swverify is not setting this environment variable." 1>&2
 	echo "To run this script manually set the environment variable:" 1>&2
 	echo "SW_CONTROL_TAG=checkdigest # or" 1>&2
 	echo "SW_CONTROL_TAG=checkfile" 1>&2
@@ -720,12 +930,14 @@ archive_digest_failure=""
 
 md5sumfile="${package_dir}/catalog/${DFILES}/md5sum"
 sha1sumfile="${package_dir}/catalog/${DFILES}/sha1sum"
+sha512sumfile="${package_dir}/catalog/${DFILES}/sha512sum"
 
 case "$SW_CONTROL_TAG" in
 checkdigest|checksig)
 	if [ -f "${md5sumfile}" ]; then
 		md5sum_did_check=1
 		sha1sum_ret=0
+		sha512sum_ret=1
 		printf "${premg}Checking Archive md5 "
 
 		check_digest_from_catalog_files "$MD5SUM" "${md5sumfile}" "" "$tarcreateutil"
@@ -746,11 +958,39 @@ checkdigest|checksig)
 			not_authentic
 			exit 1
 		fi
+
 		if [ "$sha1sum_ret" -eq 0 ]; then
 			echo  "OK (Good)"
 		else
 			echo  "FAIL (deferred)"
 		fi
+		
+		if [ -f "${sha512sumfile}" ]; then
+			printf "${premg}Checking Archive sha512 "
+			check_digest_from_catalog_files "$SHA512SUM" "${sha512sumfile}" "" "$tarcreateutil"
+			sha512sum_ret=$?
+		else
+			sha512sum_ret=99
+		fi
+	
+		case "$sha512sum_ret" in
+			99)
+				;;
+			*)
+			if [ "$md5sum_ret" -eq 0 -a "$sha1sum_ret" -eq 0 -a "$sha512sum_ret" -ne 0 -a "${SHA512SUM}" != $CMD_NOT_FOUND ]; then
+				echo  "FAIL (Bad)"
+				echo "Archive sha512 digest invalid".
+				not_authentic
+				exit 1
+			fi
+
+			if [ "$sha512sum_ret" -eq 0 ]; then
+				echo  "OK (Good)"
+			else
+				echo  "FAIL (deferred)"
+			fi
+				;;
+		esac
 	else
 		echo "Archive md5sum file missing.  No way to verify package."
 		not_authentic
@@ -783,6 +1023,14 @@ checkdigest|checksig)
 	checkINFOdigests "md5sum" "$MD5SUM"
 	infodigestret=$?
 	case $infodigestret in
+		98)
+			echo \
+			" FAIL (Bad)"
+			echo \
+			"${premg}The md5sum utility was not found" 1>&2
+			not_authentic
+			exit 1
+			;;
 		99)
 			# No md5 in info file.
 			echo "checkdigest: Warning INFO file does not have md5sum attributes" 1>&2
@@ -810,6 +1058,14 @@ checkdigest|checksig)
 			# No sha1 in info file.
 			echo "checkdigest: Warning INFO file does not have sha1sum attributes" 1>&2
 			;;
+		98)
+			echo \
+			" FAIL (Bad)"
+			echo \
+			"${premg}The sha1sum utility was not found" 1>&2
+			not_authentic
+			exit 1
+			;;
 		0)
 			echo \
 			" OK (Good)"
@@ -824,6 +1080,34 @@ checkdigest|checksig)
 			;;
 	esac
 
+	printf "${premg}Checking INFO file sha512 digests "
+	checkINFOdigests "sha512sum" "$SHA512SUM"
+	infodigestret=$?
+	case $infodigestret in
+		98)
+			echo \
+			" FAIL (Bad)"
+			echo \
+			"${premg}Warning: the sha512sum utility was not found" 1>&2
+			;;
+		199)
+			# No sha512 in info file.
+			echo "checkdigest: Warning INFO file does not have sha512sum attributes" 1>&2
+			;;
+		0)
+			echo \
+			" OK (Good)"
+			;;
+		*)
+			echo \
+			" FAIL (Bad)"
+			echo \
+			"${premg}The INFO file sha512sum attributes did not all match the package files." 1>&2
+			not_authentic
+			exit 1
+			;;
+	esac
+
 	md5sumfile="${package_dir}/catalog/${DFILES}/adjunct_md5sum"
 	if [ -f "${md5sumfile}" ]; then
 		adjunct_md5sum_did_check=1
@@ -840,21 +1124,19 @@ checkdigest|checksig)
 			archive_digest_failure="yes"
 			echo  " FAIL (Bad)"
 
-				echo  -e -n  "\
+				printf "\
 		The adjunct_md5sum failed. The package should be considered\n\
-		non-authentic and should be rejected.\n\
-				  " | sed -e 's/^[[:space:]][[:space:]]*//' 1>&2
+		non-authentic and should be rejected.\n" | sed -e 's/^[[:space:]][[:space:]]*//' 1>&2
 	
 		echo  1>&2
-				echo  -e -n  "\
+				printf "\
 		The ability to check the archive message digest from the\n\
 		installed directory form of a package is fragile.\n\
 		A possible cause is a file timestamp or permission is wrong,\n\
 		Try unpacking from scratch using GNU tar with --preserve-permissions.\n\
-		Other causes are GNU tar and swpackage version incompatibilities\n\
-		and file names longer than 99 chars with various versions of GNU tar\n\
-		and swpackage.\n\
-				  " | sed -e 's/^[[:space:]][[:space:]]*//' 1>&2
+		Other causes are GNU tar and swpackage version incompatibilities,\n\
+		and, file names longer than 99 chars with various versions of GNU tar\n\
+		and swpackage.\n" | sed -e 's/^[[:space:]][[:space:]]*//' 1>&2
 		echo  1>&2
 
 		not_authentic
@@ -936,6 +1218,34 @@ checkfile)
 			exit 1
 			;;
 	esac
+
+	printf "${premg}Checking INFO file sha512 digests "
+	checkINFOdigests "sha512sum" "$SHA512SUM"
+	infodigestret=$?
+	case $infodigestret in
+		199)
+			# No sha512 in info file.
+			echo "checkdigest: Warning INFO file does not have sha512sum attributes" 1>&2
+			;;
+		98)
+			echo \
+			" FAIL (Bad)"
+			echo \
+			"${premg}Warning: the sha512sum utility was not found" 1>&2
+			;;
+		0)
+			echo \
+			" OK (Good)"
+			;;
+		*)
+			echo \
+			" FAIL (Bad)"
+			echo \
+			"${premg}The INFO file sha512sum attributes did not all match the package files." 1>&2
+			not_authentic
+			exit 1
+			;;
+	esac
 ;;
 *)
 	echo "${premg}unsupported SW_CONTROL_TAG value" 1>&2
