@@ -18,7 +18,7 @@ case "$version" in
 		#echo "It looks like you have a suitable version of GNU tar [$version]."
 		echo 1>/dev/null
 		;;
-	1.14.*|1.14|1.15.*|1.1[3-9].[1-9][0-9])
+	1.14.*|1.14|1.15.*|1.1[3-9].[1-9][0-9]|1.1[6-9]|1.1[6-9].*|2.0)
 		echo 1>/dev/null
 		#echo "It looks like you have a suitable version of GNU tar [$version]."
 		;;
