@@ -1,3 +1,9 @@
+case "$1" in
+	swbis_swbis)
+	sed \
+		-e 's/swbis.html#/swbis_swbis.html#/g'
+		;;
+	*)
 sed \
 	-e 's/#_0028swbis_005f\(.[^_]*\)_0029/\1.html#/g' \
 	-e 's/_003a//g' |
@@ -9,4 +15,5 @@ sed \
 	-e 's/sw.html#"/sw_5.html"/' \
 	-e 's/swign.html#"/swign_1.html"/' \
 	-e 's/lxpsf.html#"/lxpsf_1.html"/'
-
+;;
+esac
