@@ -1,4 +1,4 @@
-/*  globalblob.h -- All the main routine globals in a struct */
+/*  globalblob.h -- The main routine globals in a struct */
 /*
    Copyright (C) 2006 Jim Lowe
    All Rights Reserved.
@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,11 +14,14 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+*/
+
 
 #ifndef globalblob_h_200604
 #define globalblob_h_200604
 
+#include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -40,11 +43,13 @@
 #include "swcommon0.h"
 #include "progressmeter.h"
 #include "swevents.h"
+#include "swi.h"
 #include "swicol.h"
 #include "swutillib.h"
 #include "xformat.h"
 
 typedef struct {
+	struct extendedOptions * optaM;
 	int 	g_stdout_testfd;
 	int 	g_fail_loudly; 
 	int 	g_verbose_threshold;  /* threshold for printing a message */ 
@@ -68,11 +73,13 @@ typedef struct {
 	int 	g_stderr_fd;
 	int * 	g_target_fdar;
 	int * 	g_source_fdar;
+	int * 	g_save_fdar;
 	char * g_fork_pty;
 	char * g_fork_pty2;
 	char * g_fork_pty_none;
 	sigset_t g_fork_defaultmask;
 	sigset_t g_fork_blockmask;
+	sigset_t g_ssh_fork_blockmask;
 	sigset_t g_currentmask;
 	int 	g_do_progressmeter;
 	struct sw_logspec g_logspec;
@@ -80,19 +87,58 @@ typedef struct {
 	int 	g_meter_fd;
 	XFORMAT * g_xformat;
 	int 	g_swi_event_fd;
-	SWUTIL * g_swutil;
+	SWLOG * g_swlog;
 	struct g_pax_write_command * g_pax_write_commands;
 	struct g_pax_read_command * g_pax_read_commands;
+	struct g_pax_remove_command * g_pax_remove_commands;
 	char * g_source_script_name;
 	char * g_target_script_name;
+	char * g_swi_debug_name;
   	void (*g_main_sig_handler)(int signum);
   	void (*g_safe_sig_handler)(int signum);
 	int g_is_seekable;
 	int g_do_debug_events;
+	char * g_target_terminal_host;
+	char * g_source_terminal_host;
+	char * g_sh_dash_s;  /* i.e. `sh -s'   or  `bash -s' */ 
+	int g_opt_alt_catalog_root;  /* utility  -r option */
+	int g_do_task_shell_debug;
+	int g_do_distribution;
+	int g_do_installed_software;
+	int g_master_alarm;
+	int g_noscripts;
+	SWI * g_swi;
+	int e_in_control_scriptM;
+	int devel_verboseM;
+	int g_to_stdout;
+	int g_force;
+	int g_force_locks;
+	int in_shls_looperM;
+	SWICOL * g_swicolM;
+	int g_psignalM;
+	int g_running_signalsetM;
+	int g_running_signalusr1M;
+	int g_target_did_abortM;
+	int g_save_stderr_fdM;
+	int g_opt_previewM;
+	int g_do_cleanshM;
+	char g_gpufieldM[30];
+	int g_gpuflagM;
 } GB;
-	
+
+#define G_opta (G->optaM)
+#define GFP_GET(K) gb_fparams_get(G, K)
+#define GFP_ADD(K, V) gb_fparams_add(G, K, V)
+
+#define G_is_in_control_script(G) (G->e_in_control_scriptM != 0)
+#define G_set_is_in_control_script(G, a) G->e_in_control_scriptM = a
+
+void gb_fparams_init(GB * G);
+void gb_fparams_delete(GB * G);
 GB * gb_create(void);
-void gb_delete(GB*);
-void gb_init(GB*);
+void gb_delete(GB* G);
+void gb_init(GB * G);
+int gb_fparams_add(GB * G, char * kw, char * val);
+char * gb_fparams_get(GB * G, char * kw);
 
 #endif
