@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,8 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+*/
 
 
 #include "swcommon0.h"
@@ -25,13 +26,13 @@ g_pax_write_command g_pax_write_commands[] = {
 		"pax -x ustar -w"
 	},
 	{"star", 
-		"star cf - -H ustar"
+		"star cbf 1 - -H ustar"
 	},
 	{"tar",  
-		"tar cf -"
+		"tar cbf 1 -"
 	},
 	{"gtar", 
-		"gtar cf -"
+		"gtar cbf 1 -"
 	},
 	{"swbistar",
 		SWBISLIBEXECDIR "/swbis/swbistar"
@@ -53,8 +54,8 @@ g_pax_read_command g_pax_read_commands[] = {
 	},
 	{"tar", 
 		"tar xpf -",
-		"tar xpvf - 1>&2", /*  use of stdout breaks swcopy piping. */
-		"tar xpkf - 1>&2" /* use of stdout breaks swcopy piping. */
+		"tar xpvf -  1>&2", /*  use of stdout breaks swcopy piping. */
+		"tar xpkf -  1>&2" /* use of stdout breaks swcopy piping. */
 	},
 	{"gtar",
 		"gtar xpf - --overwrite",
@@ -64,3 +65,25 @@ g_pax_read_command g_pax_read_commands[] = {
 		{(char*)NULL, (char*)NULL, (char*)NULL, (char*)NULL}
 	};
 
+struct 
+g_pax_remove_command g_pax_remove_commands[] = {
+	{"pax", 
+		NULL,
+		NULL
+	},
+	{"star",
+		NULL,
+		NULL
+	},
+	{"tar", 
+		"tar zcf - --files-from=- --no-recursion --remove-files",
+		"tar zvcf - --files-from=- --no-recursion --remove-files 1>&2"
+	},
+	{"gtar",
+		"gtar zcf - --files-from=- --no-recursion --remove-files",
+		"gtar zvcf - --files-from=- --no-recursion --remove-files 1>&2"
+	},
+		{(char*)NULL, (char*)NULL, (char*)NULL}
+	};
+
+
