@@ -5,7 +5,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,8 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+*/
 
 #ifndef swcommon_200212j_h
 #define swcommon_200212j_h
@@ -30,8 +31,9 @@
 #include "swutillib.h"
 #include "swverid.h"
 #include "globalblob.h"
+#include "swi.h"
 
-#define KILL_PID		SWSSH_TRACK_PID
+#define KILL_PID		"$$"
 
    void swc_copyright_info(FILE * fp);
 
@@ -42,12 +44,12 @@
  
    char * initialize_options_files_list(char * usethis);
 
-   int swc_do_preview_cmd(char * prefix, FILE * fver, char * targetpath,
+   int swc_do_preview_cmd(GB * G, char * prefix, FILE * fver, char * targetpath,
 			SHCMD * sshcmd, SHCMD * kill_sshcmd, char * cl_target, 
 			STRAR * cmdlist, int nhops, int count,
 			int opt_no_getconf, char * shellname);
 
-   int swc_form_command_args(char * context, char * targetpath,
+   int swc_form_command_args(GB * G, char * context, char * targetpath,
 			STRAR * target_cmdlist, int opt_preview, int nhops,
 			int opt_no_getconf, char * shellname);
 
@@ -60,7 +62,7 @@
 
    int swc_pump_line(int ofd, int ifd);
 
-   pid_t swc_run_ssh_command( 
+   pid_t swc_run_ssh_command(GB * G,
 		SHCMD ** sshcmd, 
 		STRAR * cmdlist, 
 		char * path, 
@@ -91,15 +93,13 @@
 
    void swc_record_pid(pid_t pid, pid_t * p_pid_array, int * p_pid_array_len,
 							int verbose_level);
-
-   void swc_handle_blocked_signals(int sig_set[][2], SHCMD * killcmd,
-					int verbose_level, void (*)(int));
+   void swc_set_sig_ign_by_mask(sigset_t * ignoremask);
 
    char * swc_get_next_target(char ** argvector, int uargc, int * poptind,
 				int * targetsfd, char * default_target, int * rem);
-
+/*
    mode_t swc_get_umask(void);
-
+*/
    char * swc_print_umask(char * buf, size_t len);
 
    int swc_construct_newkill_vector(SHCMD * kmd, int nhops, STRAR * tramp_list,
@@ -121,11 +121,14 @@
 
    char * swc_get_pax_write_command(struct g_pax_write_command g_pax_write_commands[], char * keyid,
 				 int verbose_level, char * paxdefault);
+   
+   char * swc_get_pax_remove_command(struct g_pax_remove_command g_pax_remove_commands[], char * keyid,
+				 int verbose_level, char * paxdefault);
 
    int swc_open_logfile(char * logfile);
    void swc_initialize_logspec(struct sw_logspec * logspec, char * logfile, int log_level);
    
-   int swc_process_swoperand_file(SWUTIL * swutil, char * type_string, char * filename,
+   int swc_process_swoperand_file(SWLOG * swutil, char * type_string, char * filename,
 			int * p_stdin_in_use, int * p_array_index, int * fd_array);
 
 void
@@ -147,7 +150,7 @@ void swc_gf_xformat_close(GB * G, XFORMAT * xformat);
 
 int swc_get_stderr_fd(GB * G);
 
-void swc_shutdown_logger(GB * G, int signum);
+int swc_shutdown_logger(GB * G, int signum);
 
 void swc_set_stderr_fd(GB * G, int fd);
 void swc_lc_raise(GB * G, char * file, int line, int signo);
@@ -155,6 +158,8 @@ void swc_lc_exit(GB * G, char * file, int line, int status);
 int swc_is_option_true(char * s);
 int sw_exitval(GB * G, int count, int suc);
 void swc_check_for_current_signals(GB * G, int lineno, char * no_remote_kill);
+
+
 pid_t swc_run_source_script(GB * G,
 		int swutil_x_mode, 
 		int local_stdin, 
@@ -166,14 +171,30 @@ pid_t swc_run_source_script(GB * G,
 		sigset_t *fork_defaultmask,
 		char * hostname,
 		char * blocksize,
-		int verbose_threshold); 
+		int verbose_threshold,
+		int (*func_write_source_copy_script)
+			(
+				GB * G,
+				int ofd,
+				char * sourcepath,
+				int do_get_file_type,
+				int vlv,
+				int delaytime,
+				int nhops,
+				char * pax_write_command_key,
+				char * hostname,
+				char * blocksize
+			)
+		); 
 void swc_helptext_target_syntax(FILE * fp);
-int swc_get_catalog_path(STROB * control_message, char * installed_software_catalog);
-int swc_read_catalog_ctl_message(GB * G, int fd,
-	STROB * control_message, 
-	int vlevel, 
-	char * loc,
-	char * installed_software_catalog);
 int swc_wait_on_script(GB * G, pid_t write_pid, char * location);
-
+void swc_set_shell_dash_s_command(GB * G, char * wopt_shell_command);
+int swc_write_swi_debug(SWI * swi, char * filename);
+char * swc_convert_multi_host_target_syntax(GB * G, char * target_spec);
+int sw_e_msg(GB * G, char * format, ...);
+int sw_l_msg(GB * G, int write_at_level, char * format, ...);
+int sw_d_msg(GB * G, char * format, ...);
+int sw_msg(GB * G, int fd, char * format, ...);
+void swc_stderr_fd2_set(GB * G, int fd);
+void swc_stderr_fd2_restore(GB * G);
 #endif
