@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,14 +17,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef swexstruct_i_h
 #define swexstruct_i_h
 
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <limits.h>
 #include <sys/stat.h>
 #include <assert.h>
 #include "swmetadata.h"
@@ -65,6 +71,7 @@ class swExStruct_i: public swExStruct
 	static swPackageDir * dirM;	// Object to write out directory.
 	static STROB * leadingPathM;	// The distribution leading path.
 	static time_t current_timeM;	// The current time (create_time).
+	static time_t dir_mtimeM;	// The mtime of the leading path prefix 
 	static char * sigFileM;		// ascii armor sigfile.
 	static int filesFdM;		// catalog/dfiles/files file list.
 	static char * filesM;		// The pointer to the list of files.
@@ -133,6 +140,11 @@ class swExStruct_i: public swExStruct
 		if (status > 0) exit(status);
 	}
 
+
+	time_t getDirMtime(void) { return dir_mtimeM; }
+	
+	void setDirMtime(time_t tm) { dir_mtimeM = tm; }
+	
 	time_t getCreateTime(void) { return current_timeM; }
 	
 	void setCreateTime(time_t tm) { current_timeM = tm; }
@@ -175,14 +187,14 @@ class swExStruct_i: public swExStruct
 	virtual int containedListAdd(swExStruct * o) {
 		assert(0); return 0;
 	}
-
+/*
 	virtual int write (){
 			if (mediaTypeM)
 				return write_serial();
 			else
 				return write_directory();
 	}
-
+*/
 	virtual swPtrList<swPackageFile> * getFileListObject(void) {
 		return NULL;
 	}
@@ -664,6 +676,18 @@ class swExStruct_i: public swExStruct
 			cksumflagsM &= ~(SWFILE_DO_FILE_DIGESTS);
 		}
 	}
+	
+	void
+	set_digests2_creation(int s) {
+		if (s) {
+			 // On
+			 cksumflagsM |= SWFILE_DO_FILE_DIGESTS_SHA2;
+		} else {
+			// Off
+			cksumflagsM &= ~(SWFILE_DO_FILE_DIGESTS_SHA2);
+		}
+	}
+
 
 	swPackageFile * getArchiver(void) { return archiverM; }
 
@@ -829,6 +853,10 @@ class swExStruct_i: public swExStruct
 		if (current_timeM == 0) {
 			current_timeM = time(NULL);
 		}
+		
+		if (dir_mtimeM == 0) {
+			dir_mtimeM = current_timeM;
+		}
 
 		if (nullfdM == 0) {
 			nullfdM = open("/dev/null", O_RDWR, 0);
