@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,19 +17,25 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef swobjfiles_i_hxx
 #define swobjfiles_i_hxx
 
-#include "swuser_assert_config.h"
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <assert.h>
+
+#include "swuser_assert_config.h"
 #include "swmetadata.h"
 #include "swdefinition.h"
 #include "swexstruct_i.h"
@@ -762,6 +768,7 @@ class swObjFiles_i:  public swExStruct_i
 				if (strstr(noesc_source, "\\x")) {
 					dupsource = strdup(noesc_source);
 					swlib_process_hex_escapes(dupsource);
+					swlib_unexpand_escapes(NULL, dupsource);
 					did_dup = 1;
 				} else {
 					dupsource = noesc_source;
