@@ -6,7 +6,7 @@
 //  COPYING TERMS AND CONDITIONS.
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2, or (at your option)
+//  the Free Software Foundation; either version 3, or (at your option)
 //  any later version.
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
 //  GNU General Public License for more details.
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 
 #include "swuser_config.h"
 #include "swmetadata.h"
@@ -25,13 +25,15 @@ int swMetaData::use_count_;
 int swMetaData::logical_modeM;
 swstructdef * swMetaData::defaults_;
 
+		// To add a element you must increment the namesM array length in swmetadata.h
 char * swMetaData::namesM[] = {"type", "mode", "mtime", "ctime", "owner", 
 				"uid", "group", "gid", "umask", "size", 
 				"is_volatile", "compression_state", "cksum", 
-				"md5sum", "sha1sum", "link_source", "minor", "major",
+				"md5sum", "sha1sum", "sha512sum", "link_source", "minor", "major",
 				"_ino", "_nlink", "_complete", ""};
 
 
+
 swMetaData::swMetaData (void) {
     init();
     set_p_offset(uxfio_lseek(data_p_->get_mem_fd(), 0, SEEK_CUR));
@@ -323,11 +325,13 @@ void swMetaData::init(void)
     next_ = NULL;
 }
 
-int swMetaData::swmetadata_local_open(SWVARFS * swvarfs, char * name) {
+int swMetaData::swmetadata_local_open(SWVARFS * swvarfs, char * name, int * p_seek_set) {
 	int u_fd;
 	if (swvarfs && swvarfs_get_format(swvarfs) != UINFILE_FILESYSTEM) {
+		*p_seek_set = UXFIO_SEEK_VSET;
 		u_fd = swvarfs_u_open(swvarfs, name);
 	} else {
+		*p_seek_set = SEEK_SET;	
 		u_fd = open(name, O_RDONLY, 0);
 	}
 	return u_fd;
@@ -343,28 +347,42 @@ int swMetaData::swmetadata_local_close(SWVARFS * swvarfs, int u_fd) {
 	return ret;
 }
 
-char swMetaData::getFileTypeFromMode(mode_t mode) {
-	int ch = taru_get_tar_filetype(mode);
+char
+swMetaData::getFileTypeFromMode(mode_t mode) {
+	int ch;
+	ch = taru_get_tar_filetype(mode);
 	switch (ch) {
 		case REGTYPE:
 		case CONTTYPE:
 			return SW_ITYPE_f;
+			break;
 		case DIRTYPE:
 			return SW_ITYPE_d;
+			break;
 		case CHRTYPE:
 			return SW_ITYPE_c;
+			break;
 		case BLKTYPE:
 			return SW_ITYPE_b;
+			break;
 		case FIFOTYPE:
 			return SW_ITYPE_p;
+			break;
 		case SYMTYPE:
 			return SW_ITYPE_s;
+			break;
 		case LNKTYPE:
 			return SW_ITYPE_h;
+			break;
 		case AREGTYPE:
-		default:
-			fprintf(stderr, "warning:  typeflag not supported, returning regular file. \n");
 			return SW_ITYPE_f;
+			break;
+		case NOTDUMPEDTYPE:
+			return SW_ITYPE_y;
+			break;
+		case -1:
+		default:
+			return '\0';
 	}
 }
 
@@ -415,12 +433,15 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		char * p_ctime;
 		char * p_swbi;
 		char * p_linksource;
+		char * p_tmp;
 		unsigned long filetime;
 		mode_t mode;
 		int intval;
+		int uxfio_seek_set;
 		int u_fd = -1;
 		int do_create_cksum;
 		int do_create_file_digests;
+		int do_create_file_digests2;
 		int ret = 0;
 		int devnum;
 		HLLIST * hllist = swvarfs_get_hllist(swvarfs);
@@ -431,7 +452,9 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		SWMETADATA_DEBUG("");
 		do_create_cksum = cksumflags & SWFILE_DO_CKSUM;
 		do_create_file_digests = cksumflags & SWFILE_DO_FILE_DIGESTS;
-		
+		do_create_file_digests2 = cksumflags & SWFILE_DO_FILE_DIGESTS_SHA2;
+	
+		uxfio_seek_set = SEEK_SET;	
 		//
 		// File Type
 		//
@@ -439,7 +462,14 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		if (*p_type == '\0') {
 			SWMETADATA_DEBUG("");
 			*p_type = swMetaData::getFileTypeFromMode(st->st_mode);
+			if (*p_type == '\0') {
+				fprintf(stderr, "swpackage: unrecognized file type for file: %s\n", source?source:"<null>");
+			}
 			*(p_type + 1) = '\0';
+			if (*p_type == SW_ITYPE_y) {
+				// fprintf(stderr, "swpackage: %s: socket ignored\n", source?source:"<null>");
+				ret = 1;
+			}
 		}
 		if (did_sense_hard_link(swvarfs, st) == 1) {
 			SWMETADATA_DEBUG("did sense hard link");
@@ -458,48 +488,75 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		array_is_set[typeE] = 1;
 		array_is_set[cksumE] = 0;
 
-		if ((do_create_cksum || do_create_file_digests) && *p_type == SW_ITYPE_f ) {
+		if ((do_create_cksum || do_create_file_digests || do_create_file_digests2) && *p_type == SW_ITYPE_f ) {
 			unsigned long ck;
-			u_fd = swMetaData::swmetadata_local_open(swvarfs, source);
+			u_fd = swMetaData::swmetadata_local_open(swvarfs, source, &uxfio_seek_set);
 			if (u_fd < 0) {
 				fprintf(stderr, "swMetaData::swmetadata_local_open [%s] failed.\n", source);
 				return -1;
 			} else {
+				int uxret;
 				char bufc[60];
 				char bufc_sha1[60];
+				char bufc_size[40];
+				char bufc_sha2[132];
+				char * p_bufc_sha2;
 				bufc[0] = '\0';
 				bufc_sha1[0] = '\0';
+				bufc_sha2[0] = '\0';
+				bufc_size[0] = '\0';
+
 
 				if (do_create_cksum) {
-					uxfio_lseek(u_fd, 0, SEEK_SET);
+					uxret = uxfio_lseek(u_fd, 0, uxfio_seek_set);
+					if (uxret < 0) {
+						fprintf(stderr, "Internal software error. location=a.\n");
+						exit(52);
+					}
 					ck = ::swlib_cksum(u_fd);
 					snprintf(bufc, sizeof(bufc), "%lu", ck);
 					bufc[sizeof(bufc) - 1] = '\0';
 					::swlib_strncpy(array[cksumE], bufc, file_permsLength - 1);
 					array_is_set[cksumE] = 1;
 				}
-				if (do_create_file_digests) {
+				if (do_create_file_digests || do_create_file_digests2) {
 					if (bufc[0] != '\0') {  // Optimization.
 						//
 						// Need to rewind the file.
 						//
-						if (uxfio_lseek(u_fd, 0, SEEK_SET) != 0) {
-							fprintf(stderr, 
-							"Internal software error. location=b.\n");
+						uxret = uxfio_lseek(u_fd, 0, uxfio_seek_set);
+						if (uxret < 0) {
+							fprintf(stderr, "Internal software error. location=b.\n");
 							exit(52);
 						}
 					}
-					// swlib_md5(u_fd, (unsigned char *)bufc, 1);
-					if (swlib_digests(u_fd, bufc, bufc_sha1)) {
+					if (do_create_file_digests2) {
+						p_bufc_sha2 = bufc_sha2;			
+					} else {
+						p_bufc_sha2 = NULL;
+					}
+					if (swlib_digests(u_fd, bufc, bufc_sha1, bufc_size, p_bufc_sha2)) {
 						// Fatal error
 						fprintf(stderr, 
 						"Internal software error in swlib_digests\n");
 						exit(52);
 					}
+					
+					// Sanity check
+					//
+					if (st->st_size != atoi(bufc_size)) {
+						fprintf(stderr, "Internal software fatal error. (digests) wrong size\n");
+						exit(55);
+					}
+
 					::swlib_strncpy(array[md5sumE], bufc, file_permsLength - 1);
 					array_is_set[md5sumE] = 1;
 					::swlib_strncpy(array[sha1sumE], bufc_sha1, file_permsLength - 1);
 					array_is_set[sha1sumE] = 1;
+					if (do_create_file_digests2) {
+						::swlib_strncpy(array[sha512sumE], bufc_sha2, file_permsLength - 1);
+						array_is_set[sha512sumE] = 1;
+					}
 				}
 				swMetaData::swmetadata_local_close(swvarfs, u_fd);
 			}
@@ -582,7 +639,9 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		//
 		if (*p_type == SW_ITYPE_f) {
 			SWMETADATA_DEBUG("");
-			snprintf(p_size, file_permsLength,  "%d", (int)(st->st_size));
+			// snprintf(p_size, file_permsLength,  "%d", (int)(st->st_size));
+			p_tmp = ::swlib_imaxtostr((intmax_t)(st->st_size), NULL); 
+			strncpy(p_size, p_tmp, file_permsLength);
 			p_size[file_permsLength-1] = '\0';
 			array_is_set[sizeE] = 1;
 		} else if (*p_type == SW_ITYPE_s) {
@@ -721,7 +780,7 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 			SWMETADATA_DEBUG2("p_owner Doing numeric: [%s]", p_owner);
 			snprintf(p_uid,  file_permsLength, "%d",(int)uid);
 			p_uid[file_permsLength-1] = '\0';
-			*p_owner == '\0';
+			*p_owner = '\0';
 			array_is_set[ownerE] = 0;
 			array_is_set[uidE] = 1;
 		} else {
@@ -746,7 +805,7 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 			SWMETADATA_DEBUG2("p_group Doing numeric: [%s]", p_group);
 			snprintf(p_gid,  file_permsLength, "%d",(int)gid);
 			p_gid[file_permsLength-1] = '\0';
-			*p_group == '\0';
+			*p_group = '\0';
 			array_is_set[groupE] = 0;
 			array_is_set[gidE] = 1;
 		} else {
