@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,17 +17,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
 #ifndef swmetadata_19980601jhl_h
 #define swmetadata_19980601jhl_h
 
+extern "C" {
+#include "swuser_config.h"
+}
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <typeinfo>
 #include "swstructdef.h"
 #include "swattributemem.h"
@@ -57,6 +61,7 @@ extern "C" {
 
 #define SWFILE_DO_CKSUM   (1 << 0)	// Do create Posix cksum
 #define SWFILE_DO_FILE_DIGESTS  (1 << 1)	// Do create md5 digest.
+#define SWFILE_DO_FILE_DIGESTS_SHA2  (1 << 2)	// Do create md5 digest.
 #define SWBIS_NO_STAT_INDICATOR "\x1C"    // FS source names that begin with this are not lstat'ed., Internal use only.
 #define SWBIS_NO_STAT_INDICATOR_C '\x1C'    // FS source names that begin with this are not lstat'ed., Internal use only.
 
@@ -80,11 +85,11 @@ class swMetaData
 	static int logical_modeM;	// 0 no censoring, 1 means get_value() returns NULL if status_ is non-zero. 
   public:
 
-	enum swExtEnum {file_permsLength = 101};
+	enum swExtEnum {file_permsLength = 131};  // a sha512string is 128 bytes plus a NUL
 	enum swFileOpts {typeE, modeE, mtimeE, ctimeE, ownerE, uidE, 
 			groupE, gidE, umaskE, sizeE, volaE, compE, 
-			cksumE, md5sumE, sha1sumE, linksourceE, minorE, majorE, inoE, nlinkE, swbiE, lastE};
-	static char * namesM[22];
+			cksumE, md5sumE, sha1sumE, sha512sumE, linksourceE, minorE, majorE, inoE, nlinkE, swbiE, lastE};
+	static char * namesM[23];
      
 	swMetaData(char * keyword, char * value);
 	swMetaData (void);
@@ -220,7 +225,7 @@ class swMetaData
 	int 	return_entry_group 	(int index);
 
 	static int did_sense_hard_link(SWVARFS * swvarfs, struct stat * st);
-	static int swmetadata_local_open(SWVARFS * swvarfs, char * name);
+	static int swmetadata_local_open(SWVARFS * swvarfs, char * name, int * virt_whence_set);
 	static int swmetadata_local_close(SWVARFS * swvarfs, int u_fd);
 	static int swmetadata_decode_file_stats(SWVARFS * swvarfs, 
 			char *source, char * hl_path,
