@@ -8,7 +8,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,15 +18,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #ifndef swpackagefile_19980601jhl_h
 #define swpackagefile_19980601jhl_h
 
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <tar.h>
 #include <time.h>
 #include "swattribute.h"
@@ -215,7 +220,7 @@ class swPackageFile :  public swAttribute
 		return swfile_write_pkg(static_cast<char*>(NULL));
 	}
 	
-	virtual int swfile_write_pkg(char * name) {
+	virtual intmax_t swfile_write_pkg(char * name) {
 		int 	source_fd = swfile_open_public_image_fd(name);
 		int 	len;
 		char * 	val;
@@ -356,7 +361,7 @@ class swPackageFile :  public swAttribute
 		}
 	}
 
-	int write_swdirectory  (char * name) {
+	intmax_t write_swdirectory  (char * name) {
 		char * pathname = swp_make_name(name, NULL);
 		xFormat_set_mtime(time(NULL));
 		xFormat_set_mode(default_tar_mode_dir);
@@ -364,7 +369,7 @@ class swPackageFile :  public swAttribute
 		return xFormat_write_file(NULL, pathname, (int)(-1));
 	}
 
-	int swfile_write(char * name, int source_fd) {
+	intmax_t swfile_write(char * name, int source_fd) {
 		if (! name ) {
 			char * pathname = swp_make_name (NULL, NULL);
 			return  xFormat_write_file(NULL, pathname, source_fd);
@@ -375,7 +380,6 @@ class swPackageFile :  public swAttribute
 
 	int swfile_read_archive_header (char * buf) {
 		int ret=xFormat_read_header();
-		// fprintf(stderr, "ret = %d %s\n", ret, xFormat_get_name(NULL));
 		if (ret <= 0) return ret; 
 		swfile_get_name(NULL);
 		if (is_ieee_layoutM && !xFormat_is_end_of_archive()) {
@@ -607,7 +611,7 @@ class swPackageFile :  public swAttribute
 	    uxformatM->xFormat_set_mtime(mtime);
 	}
 	
-	void xFormat_set_filesize(int filesize) {
+	void xFormat_set_filesize(intmax_t filesize) {
 	      uxformatM->xFormat_set_filesize(filesize);
 	}
 	
@@ -691,12 +695,12 @@ class swPackageFile :  public swAttribute
 	   return uxformatM->xFormat_get_ofd();
 	}
 	
-	virtual int xFormat_write_file(void) {
+	virtual intmax_t xFormat_write_file(void) {
 		return xFormat_write_file((struct stat *)NULL, swfile_get_package_filename());
 	}
 	
-	virtual int xFormat_write_file(swDefinition * swdef, char * package_path) {
-		int ret;
+	virtual intmax_t xFormat_write_file(swDefinition * swdef, char * package_path) {
+		intmax_t ret;
 		char * source;
 		char * path;
 		char * pstart;
@@ -732,7 +736,11 @@ class swPackageFile :  public swAttribute
 				// This can be optimized. xFormat_u_open_file does unneccessary
 				// fstat's.
 				//
-				source_fd = xFormat_u_open_file(source);  // This sets the global stat/header buf.
+				if ((ret=swlib_unexpand_escapes(tmpM, source))) {
+					fprintf(stderr, "%s: error expanding escapes for source name ( %s ) ret=%s\n",
+						swlib_utilname_get(), source, swlib_imaxtostr(ret, NULL));
+				}
+				source_fd = xFormat_u_open_file(strob_str(tmpM));  // This sets the global stat/header buf.
 				if (source_fd < 0)  {
 					return -4;
 				}
@@ -772,59 +780,68 @@ class swPackageFile :  public swAttribute
 				tmp_control_path);
 			
 			swfile_set_package_filename(package_path);
-			//if (source != swdef->find("source")) {
-			//	fprintf(stderr, "GOTTTTTITTT\n");
-			//}
 			source = swdef->find("source"); // This line fixed a core dump.
 			swfile_set_source_filename(source);
+			if ((ret=swlib_unexpand_escapes2(package_path))) {
+				fprintf(stderr, "%s: error expanding escapes for package_path ( %s ) ret=%s\n",
+					swlib_utilname_get(), package_path, swlib_imaxtostr(ret, NULL));
+			}
 			ret = swPackageFile::xFormat_write_file(static_cast<struct stat *>(NULL), package_path, source_fd);
 			if (source_fd >= 0) xFormat_u_close_file(source_fd);
 		} else {
 			//
 			// Stat the source.
 			//
-			ret = swPackageFile::xFormat_write_file(static_cast<struct stat*>(NULL), package_path, source); 
+			if ((ret=swlib_unexpand_escapes(tmpM, source))) {
+				fprintf(stderr, "%s: error expanding escapes for source name ( %s ) ret=%s\n",
+					swlib_utilname_get(), source, swlib_imaxtostr(ret, NULL));
+			}
+			if ((ret=swlib_unexpand_escapes2(package_path))) {
+				fprintf(stderr, "%s: error expanding escapes for package_path ( %s ) ret=%s\n",
+					swlib_utilname_get(), package_path, swlib_imaxtostr(ret, NULL));
+			}
+			ret = swPackageFile::xFormat_write_file(static_cast<struct stat*>(NULL), package_path, strob_str(tmpM)); 
 		}
 		return ret;
 	}
 	
-	virtual int xFormat_write_file(char * name) {
+	virtual intmax_t xFormat_write_file(char * name) {
 		return uxformatM->xFormat_write_file(name);
 	}
 	
-	virtual int xFormat_write_file(int fd) {
+	virtual intmax_t xFormat_write_file(int fd) {
 		return uxformatM->xFormat_write_file  (fd);
 	}
 	
-	virtual int xFormat_write_file(int (*fout)(int)) {
+	virtual intmax_t xFormat_write_file(int (*fout)(int)) {
 		return uxformatM->xFormat_write_file(fout);
 	}
 	
-	virtual int xFormat_write_file(struct stat *st, char * name) {
+	virtual intmax_t xFormat_write_file(struct stat *st, char * name) {
 		return uxformatM->xFormat_write_file(st, name);
 	}
 	
-	virtual int xFormat_write_file(struct stat *st, char * name, char * source) {
+	virtual intmax_t xFormat_write_file(struct stat *st, char * name, char * source) {
 		return  uxformatM->xFormat_write_file(st, name, source); 
 	}
 	
-	virtual int xFormat_write_file(char * name, char * source) {
-		int ret;
+	virtual intmax_t xFormat_write_file(char * name, char * source) {
+		intmax_t ret;
 		//struct stat st;
 		//uxformatM->xFormat_set_to_statbuf(&st);
 		ret = xFormat_write_file(static_cast<struct stat*>(NULL), name, source); 
 		return ret;	
 	}
 	
-	virtual int xFormat_write_file(struct stat *st, char * name, int source_fd) {
+	virtual intmax_t xFormat_write_file(struct stat *st, char * name, int source_fd) {
 		return  uxformatM->xFormat_write_file(st, name, source_fd); 
 	}
 	
-	virtual int xFormat_write_file(struct stat *st, char * name, int (*fout)(int)) {
+	virtual intmax_t xFormat_write_file(struct stat *st, char * name, int (*fout)(int)) {
 	     return  uxformatM->xFormat_write_file(st, name, fout); 
 	}
 	
-	virtual int xFormat_write_file_data(int source_fd){
+	virtual intmax_t xFormat_write_file_data(int source_fd){
 	    return uxformatM->xFormat_write_file_data(source_fd);
 	}
 	
@@ -836,7 +853,7 @@ class swPackageFile :  public swAttribute
 	   return uxformatM->xFormat_get_virtual_eof();
 	}
 	
-	int xFormat_read_file_data(int dst_fd) {
+	intmax_t xFormat_read_file_data(int dst_fd) {
 	    return  uxformatM->xFormat_read_file_data(dst_fd);
 	}
 	
@@ -860,15 +877,15 @@ class swPackageFile :  public swAttribute
 	     return uxformatM->xFormat_write_header(name); 
 	}
 	
-	int xFormat_copy_pass(int discharge_fd) {
+	intmax_t xFormat_copy_pass(int discharge_fd) {
 	     return  uxformatM->xFormat_copy_pass(discharge_fd); 
 	}
 	
-	int xFormat_copy_pass(void) {
+	intmax_t xFormat_copy_pass(void) {
 	     return  uxformatM->xFormat_copy_pass();
 	}
 	
-	int xFormat_copy_pass(int discharge_fd, int source_fd) {
+	intmax_t xFormat_copy_pass(int discharge_fd, int source_fd) {
 	     return  uxformatM->xFormat_copy_pass(discharge_fd, source_fd); 
 	}
 	
