@@ -10,7 +10,7 @@
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2, or (at your option)
+//  the Free Software Foundation; either version 3, or (at your option)
 //  any later version.
 
 //  This program is distributed in the hope that it will be useful,
@@ -20,7 +20,7 @@
 
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
 
 
@@ -37,7 +37,7 @@ class swsCollection{
    public:
 	swsCollection(swDefinitionFile * swdeffile);
 	virtual ~swsCollection(void);
-	int write(int fd);
+	int iwrite(int fd);
 	int write_debug(int fd);
 	swStruct * get_swstruct (void);
 	swDefinitionFile * get_swdeffile(void);
