@@ -7,7 +7,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -17,9 +17,11 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
+#include "swuser_config.h"
+#include "stdlib.h"
 #include "strar.h"
 
 #define swbis_free free
@@ -37,6 +39,7 @@ strar_open(void)
 	strb->nsM = 0;
 	strb->storageM = strob_open(132);
 	strb->listM = cplob_open(10);
+	cplob_add_nta(strb->listM, NULL);
 	return strb;
 }
 
@@ -59,7 +62,7 @@ strar_close(STRAR * strar)
 }
 
 int
-strar_max_index(STRAR * strar)
+strar_num_elements(STRAR * strar)
 {
 	return strar->nsM;
 }
@@ -73,15 +76,15 @@ strar_get(STRAR * strar, int index)
 	return s;
 }
 
-int
-strar_add(STRAR * strar, char * src)
+char *
+strar_return_store(STRAR * strar, int len)
 {
 	char * oldbase;
 	char * base;
 	char * last;
 	int nused;
 
-	strar->lenM = strar->lenM + strlen(src) + 1;
+	strar->lenM = strar->lenM + len + 1;
 
 	oldbase = strob_str(strar->storageM);
 	strob_set_memlength(strar->storageM, strar->lenM);
@@ -117,6 +120,24 @@ strar_add(STRAR * strar, char * src)
 		last = base;
 	}
 	cplob_add_nta(strar->listM, last);
+	memset(last, (int)'_', len);
+	*(last+len) = '\0';
+	return last;
+}
+
+int
+strar_add(STRAR * strar, char * src)
+{
+	char * last;
+	last = strar_return_store(strar, strlen(src));
 	strcpy(last, src);
 	return 0;
 }
+
+void
+strar_qsort(STRAR * strar, int (*f_comp)(const void*, const void*))
+{
+	size_t nused;
+	nused = (size_t)cplob_get_nused(strar->listM) - 1;
+	std_quicksort(strar->listM->list, (size_t)nused, sizeof(char*), f_comp);
+}
