@@ -6,7 +6,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef strar_19990124_h
@@ -41,7 +41,9 @@ void		strar_close	(STRAR * strar);
 void		strar_reset	(STRAR * strar); 
 int 		strar_add	(STRAR * strar, char * s);
 char * 		strar_get	(STRAR * strar, int index);
-int 		strar_max_index(STRAR * strar);
-/*D char * 		strar_dump_string_s(STRAR * strar, char * prefix); */
+int 		strar_num_elements(STRAR * strar);
+char *		strar_dump_string_s(STRAR * strar, char * prefix);
+char *		strar_return_store(STRAR * strar, int len);
+void		strar_qsort(STRAR * strar, int (*)(const void*, const void*));
 
 #endif
