@@ -6,7 +6,7 @@
  COPYING TERMS AND CONDITIONS:
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,11 +14,12 @@
  GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 #define FILENEEDDEBUG 1
 #undef FILENEEDDEBUG
+
 #include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,6 +44,8 @@
 extern struct extendedOptions * optionsArray;
 extern char * CHARTRUE;
 
+static int swextopt_statusG = 0;
+
 static char * utilnames[] = {
 	"swask",       /* SWC_U_A  (1 << 0) */
 	"swcopy",      /* SWC_U_C  (1 << 1) */
@@ -81,6 +84,7 @@ check_applicability(struct extendedOptions * opta, enum eOpts nopt)
 	char ** names = utilnames;
 	int ap_mask;	
 
+	E_DEBUG("case");
 	ap_mask = opta[nopt].app_flags;
 
 	cu = swlib_utilname_get();
@@ -103,10 +107,12 @@ combine1(STROB * result, char * directory, char * soc_spec)
 {
 	char * x;
 	char * t;
+	E_DEBUG3("directory=%s soc_spec=%s", directory, soc_spec);
 	if (*directory == '@') directory++;
 	if (*soc_spec == '@') directory++;
 	strob_strcpy(result, "@"); /* FIXME: the utilities prepend a '@' */
 	strob_strcat(result, soc_spec);
+	E_DEBUG2("result=%s", strob_str(result));
 	x = directory;
 	if (*directory == ':') x++;
 	if (*directory == '@') x++;  /* FIXME: the utilities prepend a '@' */
@@ -117,12 +123,15 @@ combine1(STROB * result, char * directory, char * soc_spec)
 		1
 	) {
 		strob_strcat(result, ":");
+		E_DEBUG2("result=%s", strob_str(result));
 	}
 	if (*directory == ':') directory ++;
 	if ((t=strrchr(strob_str(result), ':')) && *(t+1) == '\0') {
 		strob_strcat(result, directory);
+		E_DEBUG2("result=%s", strob_str(result));
 	} else {
 		swlib_unix_dircat(result, directory);
+		E_DEBUG2("result=%s", strob_str(result));
 	}
 }
 
@@ -216,11 +225,43 @@ swextopt_write_session_options(STROB * buf, struct extendedOptions * opta, int S
 	}
 }
 
+int
+swextopt_is_value_false(char * s)
+{
+	if (
+		s == NULL ||
+		(
+			strcasecmp(s, "false") == 0 ||
+			strcasecmp(s, "no") == 0 ||
+			strcasecmp(s, "n") == 0 ||
+			0
+		)
+	) return 1;
+	return 0;
+}
+
+
 int
 swextopt_is_value_true(char * s)
 {
-	if (!s || strcasecmp(s, "true")) return 0;
-	return 1;  
+	if (
+		s != NULL &&
+		(
+			strcasecmp(s, "true") == 0 ||
+			strcasecmp(s, "yes") == 0 ||
+			strcasecmp(s, "y") == 0 ||
+			0
+		)
+	) return 1;
+	return 0;
+}
+
+int
+swextopt_is_option_false(enum eOpts nopt, struct extendedOptions * options)
+{
+	char * val;
+	val = get_opta(options, nopt);
+	return swextopt_is_value_false(val);
 }
 
 int
@@ -497,10 +538,38 @@ getExtendedOption(char * optionname, struct extendedOptions * peop,
 	return NULL; /* error, option not found. */
 }
 
+void
+set_opta_boolean(struct extendedOptions * opta, enum eOpts nopt, char * value)
+{
+	if (opta[nopt].is_boolM) {
+		if (
+			value != NULL &&
+			strcasecmp(value, SW_E_TRUE) &&
+			strcasecmp(value, SW_E_FALSE) &&
+			strcasecmp(value, "no") &&
+			strcasecmp(value, "yes") &&
+			strcasecmp(value, "n") &&
+			strcasecmp(value, "y") &&
+			strcmp(value, "0") &&
+			strcmp(value, "1")
+		) {
+			swextopt_statusG = 1;
+		}
+	}
+	if (value) {
+		if (*value == '0') value = SW_E_FALSE;
+		if (*value == '1') value = SW_E_TRUE;
+	}
+	i_set_opta(1, opta, nopt, value);
+}
+
 void
 set_opta(struct extendedOptions * opta, enum eOpts nopt, char * value)
 {
-	i_set_opta(1, opta, nopt, value);
+	if (opta[nopt].is_boolM)
+		set_opta_boolean(opta, nopt, value);
+	else
+		i_set_opta(1, opta, nopt, value);
 }
 
 void
@@ -514,8 +583,9 @@ swbisoption_get_opta(struct extendedOptions * opta, enum eOpts nopt)
 {
 	char * value;
 	value = opta[nopt].valueM;
-	if (value == (char*)NULL || strcasecmp(value, "false") == 0) 
-		return (char*)NULL;
+	if (value == NULL) {
+		fprintf(stderr, "internal warning: swbis option %d [%s] is null\n", (int)nopt, opta[nopt].optionNameM);
+	}
 	if (!value) value = opta[nopt].defaultValueM;
 	return value;
 }
@@ -525,6 +595,9 @@ get_opta(struct extendedOptions * opta, enum eOpts nopt)
 {
 	char * value;
 	value = opta[nopt].valueM;
+	if (value == NULL) {
+		fprintf(stderr, "internal warning: sw option %d [%s] is null\n", (int)nopt, opta[nopt].optionNameM);
+	}
 	if (!value) value = opta[nopt].defaultValueM;
 	return value;
 }
@@ -593,6 +666,7 @@ parse_options_files(struct extendedOptions * opta, char * option_files,
 	while (file) {
 		if (do_check_access) {
 			if (access(file, R_OK) != 0) {
+				E_DEBUG("case");
 				if (reqd) {
 					fprintf(stderr, "%s : %s\n", 
 						file, strerror(errno));
@@ -605,6 +679,7 @@ parse_options_files(struct extendedOptions * opta, char * option_files,
 				}
 			} else {
 				skip = 0;
+				E_DEBUG("case");
 				if (show_only) {
 					fprintf(stdout, "%s\n", file);
 					skip = 1;
@@ -613,17 +688,21 @@ parse_options_files(struct extendedOptions * opta, char * option_files,
 		}
 
 		if (show_only) {
+			E_DEBUG("case");
 			skip = 1;
 		}
 
 		if (!skip) {
+			E_DEBUG("case");
 			ret = parse_options_file(opta, file, util_name);
 			if (ret) {
+				E_DEBUG("case");
 				fprintf(stderr, 
 				"error processing option file : %s\n", file);
 				return 1;
 			}
 		}
+		E_DEBUG("case");
 		skip = 0;
 		file = strob_strtok(ktmp, NULL, " ");
 	}
@@ -657,19 +736,37 @@ is_host(char * soc)
 	return 0;
 }
 
-/*
- * merge the exteneded option directory with the source or target
- * given on the the command line input to form a complete source or
- * target.  If the command line arg has a directory component then
- * do nothing.
+is_fq(char * soc_spec)
+{
+	char * trailing_colon;
+
+	if (*soc_spec == '/') return 1;
+	trailing_colon = strrchr(soc_spec, ':');
+	if (trailing_colon && *(trailing_colon+1) == '/') return 1;
+	if (
+		trailing_colon && 
+		*(trailing_colon+1) == '.' &&
+		*(trailing_colon+2) == '\0'
+	) return 1;
+	return 0;
+}
+
+/**
+ *  swextopt_combine_directory  - merge target specs
+ *
+ *  merge the exteneded option directory with the source or target
+ *  given on the the command line input to form a complete source or
+ *  target.  If the command line arg has a directory component then
+ *  do nothing.
  */
 
 int
 swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 {
-
 	int retval;
+	E_DEBUG("ENTERING");
 
+	E_DEBUG3("soc_spec=%s directory=%s", soc_spec, directory);
 	if (soc_spec) {
 		strob_strcpy(result, soc_spec);
 	}
@@ -680,16 +777,20 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 		strlen(directory) == 0 ||
 		strcmp(directory, "-") == 0 ||
 		strcmp(directory, ":") == 0 ||  /* not used */
+		(*soc_spec == '.' && *directory == '.') ||
+		(*soc_spec == '-') ||
+		is_fq(soc_spec) ||
 		0
 	) {
 		/*
 		 * don't merge these special values
 		 */
 		E_DEBUG("returning doing nothing");
+		E_DEBUG("LEAVING retval=0");
 		return 0;
 	} 
 
-	E_DEBUG3("soc_spec=%s   directory=%s", soc_spec, directory);
+	E_DEBUG3("soc_spec=%s directory=%s", soc_spec, directory);
 	
 	/*
 	 * Now determine if soc_spec has a directory component
@@ -703,7 +804,9 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 	) {
 		;
 		/* AMBIG error */
-		E_DEBUG("case");
+		E_DEBUG3("directory: is_host(%s)=%d", directory, is_host(directory));
+		E_DEBUG3("soc_spec:  is_host(%s)=%d", soc_spec, is_host(soc_spec));
+		E_DEBUG("AMBIG retval=-1");
 		retval = -1;
 	}
 	else if (
@@ -734,6 +837,7 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 			 * directory = "HOST"
 			 * soc_spec = ":PATHNAME"
 			 */
+			E_DEBUG("case");
 			strob_strcpy(result, directory);
 			strob_strcat(result, soc_spec);
 		}
@@ -741,6 +845,8 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 			/* 
 			 * Sanity check
 			 */
+			E_DEBUG("case");
+			E_DEBUG("LEAVING retval=-1");
 			return -1;
 		}
 	}
@@ -806,5 +912,14 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 
 	E_DEBUG2("result=%s", strob_str(result));
 	E_DEBUG2("returning with value %d", retval);
+	E_DEBUG2("LEAVING retval=%d", retval);
 	return retval;
 }
+
+int
+swextopt_get_status(void)
+{
+	E_DEBUG("");
+	return swextopt_statusG;
+}
+
