@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
   
    This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
   
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "swuser_config.h"
 
@@ -61,31 +61,35 @@ fdgo(   int select_ret,
 	if (select_ret > 0 && read_ret <= 0) {
 		if (read_ret < 0) {
 			/* 
-			* Error 
-			*/
-			if (Gbe_silent < 1) fprintf(stderr, 
-				"Uh-oh [read] read_ret at %s.\n", location);
+			 * Error 
+			 */
+			if (Gbe_silent < 1)
+				fprintf(stderr, "%s: fdgo(): read error, loc=%s, errno=%d: %s\n",
+					swlib_utilname_get(), location, errno, strerror(errno));
+			return -1;
+		} else {
+			/* 
+			 * EOF 
+			 */
+			return 0;
 		}
-		/* 
-		* EOF 
-		*/
-		return 0;
 	} else if (select_ret < 0) {
 		/* 
-		* error 
-		*/
-		if (Gbe_silent < 1) fprintf(stderr, 
-				"Uh-oh [select] ret at %s.\n", location);
-		return 0;
+		 * error 
+		 */
+		if (Gbe_silent < 1)
+			fprintf(stderr, "%s: fdgo(): select error, loc=%s, errno=%d: %s\n",
+				swlib_utilname_get(), location, errno, strerror(errno));
+		return -1;
 	} else if (select_ret > 0 && read_ret > 0) {
 		/* 
-		* Good. Got data. 
-		*/
+		 * Good. Got data. 
+		 */
 		return 1;
 	} else if (select_ret == 0 && read_ret == 0) {
 		/* 
-		* Nothing happening,  keep checking 
-		*/
+		 * Nothing happening,  keep checking 
+		 */
 		return 1;
 	} else {
 		return 1;
@@ -96,13 +100,13 @@ static
 ssize_t
 read_line(int fd, void * buf, size_t len)
 {
-	return swgp_read_line(fd, (STROB*)buf);
+	return swgp_read_line(fd, (STROB*)buf, DO_NOT_APPEND);
 }
 
 static
 void
 read_pipe(int ofd, int ifd, void * buf, int *readReturn,
-		char * location, unsigned long int * statbytes,
+		char * location, uintmax_t * statbytes,
 		ssize_t (*wf)(int, void *, size_t),
 		ssize_t (*rf)(int, void *, size_t)
 		)
@@ -216,7 +220,7 @@ doPumpCycle(
 	int sec, 
 	int usec, 
 	int * readReturn, 
-	unsigned long int * statbytes)
+	uintmax_t * statbytes)
 {
 	fd_set rfds;
 	struct timeval tv;
@@ -237,8 +241,8 @@ doPumpCycle(
 		if (FD_ISSET(ifd, &rfds)) {
 			read_pipe(ofd, ifd, (void*)buf,
 				readReturn, location, statbytes,
-				(ssize_t(*)(int, void *, size_t))(write),
-				(ssize_t(*)(int, void *, size_t))(read)
+				(ssize_t(*)(int, void *, size_t))(uxfio_unix_safe_write),
+				(ssize_t(*)(int, void *, size_t))(uxfio_unix_safe_read)
 				);
 		} 
 	} else if (retval < 0 && errno != EINTR) {
@@ -267,7 +271,7 @@ sumpPump(
 	int sec, 
 	int usec, 
 	int loopcount, 
-	unsigned long int * statbytes)
+	uintmax_t * statbytes)
 {
 	int count = 0;
 	int rret, selret, retval = 1;
@@ -284,7 +288,8 @@ sumpPump(
 				selret, rret, location, retval);
 		}
 		if (retval < 0) {
-			fprintf(stderr, "sumpPump internal error: loc=1\n");
+			if (Gbe_silent < 1)
+				fprintf(stderr, "sumpPump internal error: loc=1\n");
 			break;
 		} else if (retval == 0) {
 			break;
@@ -314,7 +319,7 @@ swgp_stdioPump(int stdoutfd,      /* local stdout */
 		int * waitret, 
 		int * statusp, 
 		int * cancel,
-		unsigned long int * statbytes)
+		uintmax_t * statbytes)
 {
 	int ret0, ret1;
 	int retval = 0;
@@ -456,7 +461,8 @@ swgp_stdioPump(int stdoutfd,      /* local stdout */
 	}
 
 	if (ret1 < 0) {
-		fprintf(stderr, "swssh: stdioPump ERROR on ret1\n");
+		if (Gbe_silent < 1)
+			fprintf(stderr, "swssh: stdioPump ERROR on ret1\n");
 	}
 
 	if (opt_verbose) {
@@ -555,20 +561,40 @@ void swgp_sig_handler_exit(int signo)
 }
 
 int
-swgp_read_line(int fd, STROB * buf)
+swgp_write_as_echo_line(int fd, char * buf)
+{
+	int ret;
+	int len;
+	STROB * btmp;
+
+	btmp = strob_open(90);
+	strob_sprintf(btmp, 0, "%s\n", buf);
+	len = (int)strob_strlen(btmp);
+	ret = atomicio((ssize_t (*)(int, void *, size_t))uxfio_write,
+			fd, strob_str(btmp), len);
+	strob_close(btmp);
+	if (ret != len) return -1;
+	return 0;
+}
+
+int
+swgp_read_line(int fd, STROB * buf, int do_append)
 {
 	int n;
 	char c[2];
 	int count = 0;
 
 	c[1] = '\0';
-	strob_strcpy((STROB*)buf, "");
-	
-	n = read(fd, (void*)&c, 1);
+	if (do_append == 0) {
+		strob_strcpy((STROB*)buf, "");
+	} else {
+		;
+	}	
+	n = uxfio_unix_safe_read(fd, (void*)&c, 1);
 	while (n == 1 && c[0] != '\n') {
 		strob_strcat((STROB*)buf, c);
 		count ++;
-		n = read(fd, (void*)&c, 1);
+		n = uxfio_unix_safe_read(fd, (void*)&c, 1);
 	}
 	if (n == 1 && c[0] == '\n') {
 		count ++;
@@ -698,7 +724,7 @@ swgpReadFdNonblock(char * pipe_buf, int fd_from, int * len)
 	*pipe_buf = '\0';
 	ret = doExhaustFd_i((void*)pipe_buf, fd_from,
 		"ret2", 0, 1000, len,
-		(ssize_t(*)(int, void *, size_t))(read));
+		(ssize_t(*)(int, void *, size_t))(uxfio_unix_safe_read));
 	return ret;
 }
 
