@@ -8,7 +8,7 @@
    COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
@@ -41,6 +41,7 @@
 #include "uinfile.h"
 #include "cpiohdr.h"
 #include "swverid.h"
+#include "swutilname.h"
 #include "swparse.h"
 
 #include "debug_config.h"
@@ -66,10 +67,13 @@ static char *
 swheader_goto_c_next_line (void * vheader, int * output_line_offset,
 					int peek_only)
 {
-	SWHEADER * swheader=(SWHEADER*)vheader;
-	char * ih=swheader->image_head_;
-	char *keyw, *p;
-	int n, type;
+	SWHEADER * swheader;
+	char * ih;
+	char *keyw;
+	char *p;
+	char * line;
+	int n;
+	int type;
 	
 	/* Interface:
 	==============
@@ -81,6 +85,9 @@ swheader_goto_c_next_line (void * vheader, int * output_line_offset,
 							-(*output_line_offset)
 	*/
 
+	swheader = (SWHEADER*)vheader;
+	ih = swheader->image_head_;
+
 	if (!output_line_offset){ 
 		swheader__reset(swheader);
 		return (char*)(NULL);
@@ -113,11 +120,15 @@ swheader_goto_c_next_line (void * vheader, int * output_line_offset,
 
 	/* Must be a attribute or object keyword. */
 
-	keyw = swheaderline_get_keyword(ih+(*output_line_offset));
-	if (!keyw)
+	keyw = swheaderline_get_keyword(ih + (*output_line_offset));
+	if (keyw == NULL) {
 		return NULL;
-	n = swheaderline_get_value_length(ih+(*output_line_offset));
-	if (n < 0) return NULL;
+	}
+
+	n = swheaderline_get_value_length(ih + (*output_line_offset));
+	if (n < 0) {
+		return NULL;
+	}
         if (type == 'A') n++; /* add the space between the value and keyword*/
 
 	/* detecting the last line requires that there be a NULL 
@@ -125,22 +136,22 @@ swheader_goto_c_next_line (void * vheader, int * output_line_offset,
 
 	/* assumes newline is 1 char long */
 
-	p = keyw + strlen(keyw) + n + NEWLINE_LEN;
+	line = keyw + (strlen(keyw) + n + NEWLINE_LEN);
 
 	/* next char will be a '0' or '-' or NUL.
 	   if NUL then we're at the last line.
 	  '0' and '-' ar possible first chars of a header line.  */
 
-	if (*p == '\0' && peek_only == 0) {
+	if (*line == '\0' && peek_only == 0) {
 		*output_line_offset=-1;
 	}	
-	if (*p == '\0') {
+	if (*line == '\0') {
 		return (char *) (NULL);
 	}	
 	if (peek_only == 0) {
-		*output_line_offset=p-swheader->image_head_;
+		*output_line_offset = (int)(line - swheader->image_head_);
 	}
-	return p;
+	return line;
 }
 
 static void
@@ -202,26 +213,45 @@ get_object(SWHEADER * swheader, SWVERID * swverid,
 	return (char*)NULL;
 }
 
+void
+restore_state(SWHEADER * header, SWHEADER_STATE * state)
+{
+	header->current_offset_p_ = state->save_current_offset_pM;
+	header->current_offset_ = state->save_current_offsetM;
+	if (header->current_offset_p_)
+		*(header->current_offset_p_) = state->save_current_offset_vM;
+}
+
 
 /* -------------------------------------------------------------------*/
 /* ------------------ Public Functions ------------------------------*/
 
 void
-swheader_store_state(SWHEADER * header)
+swheader_store_state(SWHEADER * header, SWHEADER_STATE * fp_state)
 {
-	header->save_current_offset_p_ = header->current_offset_p_;
-	header->save_current_offset_ = header->current_offset_;
-	if (header->current_offset_p_)
-		header->save_current_offset_v_ = *(header->current_offset_p_);
+	SWHEADER_STATE * state;
+
+	if (fp_state)
+		state = fp_state;
+	else
+		state = &(header->saved_);	
+
+	state->save_current_offset_pM = header->current_offset_p_;
+	state->save_current_offsetM = header->current_offset_;
+	if (header->current_offset_p_) {
+		state->save_current_offset_vM = *(header->current_offset_p_);
+	} else {
+		state->save_current_offset_vM = 0;
+	}
 }
 
 void
-swheader_restore_state(SWHEADER * header)
+swheader_restore_state(SWHEADER * header, SWHEADER_STATE * state)
 {
-	header->current_offset_p_ = header->save_current_offset_p_;
-	header->current_offset_ = header->save_current_offset_;
-	if (header->current_offset_p_)
-		*(header->current_offset_p_) = header->save_current_offset_v_;
+	if (state)
+		restore_state(header, state);
+	else
+		restore_state(header, &(header->saved_));
 }
 
 SWHEADER *
@@ -231,8 +261,6 @@ swheader_open(char* (*f_goto_next)(void * , int* , int), void * image_object)
 	SWHEADER *swheader=(SWHEADER *)malloc(sizeof(SWHEADER));
 	if (!swheader)
 		return swheader;
-	
-	/* swheader->length_ = 0; */
 
 	for (i=0; i<=SWHEADER_IMAGE_STACK_LEN; i++)
 			swheader->image_object_stack_[i]=(void*)(NULL);
@@ -252,10 +280,10 @@ swheader_open(char* (*f_goto_next)(void * , int* , int), void * image_object)
         if (!swheader->f_goto_next_)
 		swheader_set_iter_function(swheader, 
 				swheader_goto_c_next_line);
+	swheader_store_state(swheader, &(swheader->saved_));
 	return swheader;
 }
 
-o__inline__
 void
 swheader_close(SWHEADER * swheader)
 {
@@ -417,23 +445,6 @@ swheader_get_iter_function(SWHEADER * swheader)
 	return (void*)(swheader->f_goto_next_);
 }
 
-/*
-o__inline__
-void
-swheader_set_image_length(SWHEADER * swheader, int len)
-{
-	swheader->length_ = len;
-}
-
-
-o__inline__
-int
-swheader_get_image_length(SWHEADER * swheader)
-{
-	return swheader->length_;
-}
-*/
-
 o__inline__
 int *
 swheader_get_current_offset_p(SWHEADER * swheader)
@@ -457,7 +468,6 @@ swheader_get_image_head(SWHEADER * swheader)
 						&i, SWHEADER_GET_NEXT);
 }
 
-
 /*
  *  swheader_get_next_object():
  *
@@ -564,11 +574,15 @@ swheader_get_single_attribute_value(SWHEADER * swheader, char *keyword)
         char * line;
         char * value;
 	int current_file_offset;
-	current_file_offset = swheader_get_current_offset(swheader);
+	SWHEADER_STATE state;
+	swheader_store_state(swheader, &state);
 	line = swheader_get_attribute_in_object(swheader, keyword, NULL, NULL);
-	if (!line) return line;
+	if (line == NULL) {
+		swheader_restore_state(swheader, &state);
+		return NULL;
+	}
         value = swheaderline_get_value(line, NULL);
-	swheader_set_current_offset(swheader, current_file_offset);
+	swheader_restore_state(swheader, &state);
         return value;
 }
 
@@ -603,12 +617,6 @@ swheader_get_attribute(SWHEADER *swheader,
 	return ret;
 }
 
-/*
- *  swheader_get_attribute_list
- *     returns a null terminated array of strings.  Each string
- *     is new storage and may be freed with free().  The pointer
- *     list vector may be freed with free() as well.
- */
 char **
 swheader_get_attribute_list(SWHEADER *swheader,
 			char *attribute_keyword, int *multi_line)
@@ -858,12 +866,15 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 	char did_type = 0;
 
 	taru_init_header(file_hdr);
+	taru_init_header_digs(file_hdr);
 
 /*JL fprintf(stderr, ">> object start\n"); */
+
 	while((attr=swheader_get_next_attribute(fileheader))) {
+
 		keyword = swheaderline_get_keyword(attr);
 		value = swheaderline_get_value(attr, &len);
-/*JL fprintf(stderr, "%s = [%s]\n", keyword, value); */
+		/*JL fprintf(stderr, "%s = [%s]\n", keyword, value); */
 
 		if (strcmp(keyword, "mode") == 0) {
 			file_hdr->c_mode = 0;
@@ -872,6 +883,10 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 			(file_hdr->usage_maskM) |= TARU_UM_MODE;
 		} else if (strcmp(keyword, "size") == 0) {
 			file_hdr->c_filesize = (unsigned long)atol(value);		
+			if (file_hdr->digsM) {
+				swlib_strncpy(file_hdr->digsM->size, value, sizeof(file_hdr->digsM->size));
+				file_hdr->digsM->do_size = DIGS_ENABLE_ON;
+			}
 			did_size = 1;
 		} else if (strcmp(keyword, "path") == 0) {
 			ahsStaticSetTarFilename(file_hdr, value);
@@ -899,20 +914,38 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 			did_mtime = 1;
 			(file_hdr->usage_maskM) |= TARU_UM_MTIME;
 		} else if (strcmp(keyword, "major") == 0) {
-			file_hdr->c_dev_maj = swlib_atoi(value, NULL);	
+			file_hdr->c_rdev_maj = swlib_atoi(value, NULL);	
 			did_major = 1;
 		} else if (strcmp(keyword, "minor") == 0) {
-			file_hdr->c_dev_min = swlib_atoi(value, NULL);	
+			file_hdr->c_rdev_min = swlib_atoi(value, NULL);	
 			did_minor = 1;
+		} else if (strcmp(keyword, "md5sum") == 0) {
+			if (file_hdr->digsM) {
+				swlib_strncpy(file_hdr->digsM->md5, value, sizeof(file_hdr->digsM->md5));
+				file_hdr->digsM->do_md5 = DIGS_ENABLE_ON;
+			}
+		} else if (strcmp(keyword, "sha1sum") == 0) {
+			if (file_hdr->digsM) {
+				swlib_strncpy(file_hdr->digsM->sha1, value, sizeof(file_hdr->digsM->sha1));
+				file_hdr->digsM->do_sha1 = DIGS_ENABLE_ON;
+			}
+		} else if (strcmp(keyword, "sha512sum") == 0) {
+			if (file_hdr->digsM) {
+				swlib_strncpy(file_hdr->digsM->sha512, value, sizeof(file_hdr->digsM->sha512));
+				file_hdr->digsM->do_sha512 = DIGS_ENABLE_ON;
+			}
 		} else if (strcmp(keyword, "type") == 0) {
 			info_filetype = (int)(*value);	
 			did_type = 1;
+		} else if (strcmp(keyword, "is_volatile") == 0) {
+			(file_hdr->usage_maskM) |= TARU_UM_IS_VOLATILE;
 		}
-	}
+	} /* while */
 
 	/*
-	* Now add the type bits to the mode field
-	*/
+	 * Now add the type bits to the mode field
+	 */
+
 	tartype = swheader_getTarTypeFromTypeAttribute(info_filetype);
 	if (tartype == LNKTYPE)
 		file_hdr->c_is_tar_lnktype = 1;
@@ -923,9 +956,9 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 	file_hdr->c_mode = (unsigned long)modet;
 
 	/*
-	* Now set the user/group lookup policy based on what was
-	* present in the INFO file "file" object.
-	*/
+	 * Now set the user/group lookup policy based on what was
+	 * present in the INFO file "file" object.
+	 */
 
 	if (did_uname && did_uid) {
 		file_hdr->c_cu = TARU_C_BY_UNONE;
@@ -942,8 +975,9 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 	}
 
 	/*
-	* do a sanity check
-	*/
+	 * do a sanity check
+	 */
+
 	if (
 		(!did_gname && !did_gid) ||
 		(!did_uname && !did_uid)
@@ -966,8 +1000,8 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 	}
 
 	/*
-	* FIXME, need to impose required attributes per spec.
-	*/
+	 * FIXME, need to impose required attributes per spec.
+	 */
 
 	return 0;
 }
@@ -989,8 +1023,10 @@ swheader_getTarTypeFromTypeAttribute(char ch) {
 			return SYMTYPE;
 		case 'h':
 			return LNKTYPE;
+		case SW_ITYPE_y:
+			return NOTDUMPEDTYPE;
 		default:
-			fprintf(stderr, "warning:  ISO 15068-2 file type [%c]  not found, returning regular file.\n", ch);
+			fprintf(stderr, "%s: invalid C701 file type [%c]  not found, ignoring.\n", swlib_utilname_get(), ch);
 			return REGTYPE;
 	}
 }
