@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 
@@ -26,6 +26,37 @@
 
 #define NEWLINE_LEN 1
 
+/*
+   The routines in this file parse or scan the parser output line 
+   produced by the parser.  The stand-alone parser can make this
+   output by:
+
+	swprogs/swbisparse -n {--psf|--info|--index} <INPUT_FILE
+
+An example of INFO file output is:
+
+0    O01 file
+12   A02 path ./autogen.sh
+1    A02 type f
+3    A02 mode 755
+
+A line has the form:
+
+SSSS X0N keyword value
+	where
+		SSSS is a decimal number which is the length of the value in octets.
+		X    is the attribute type 'A' or 'O' or 'E' for
+			Attribute, Object or Extended.
+		0    is the first digit of the containment level, always 0 since
+			levels >9 do not exist in the grammar.
+
+		N   is ones digit of the containment level.
+
+		keyword is the attribute keyword
+	
+		value is the value
+*/
+
 static
 char *
 get_value(char *output_line, int *value_len, int do_terminate)
@@ -54,8 +85,47 @@ get_value(char *output_line, int *value_len, int do_terminate)
 char * 
 swheaderline_get_type_pointer(char *output_line)
 {
+	char * ret;
 	if (isdigit((int) (*output_line))) {
-		return output_line + SWPARSE_MKUP_LEN_WIDTH + 1;
+		if (*(output_line + SWPARSE_MKUP_LEN_WIDTH) == ' ') {
+			ret = output_line + SWPARSE_MKUP_LEN_WIDTH + 1;
+		} else {
+			/* long value, the line image image has shifted 1 column. */
+			char *t;
+
+			t = output_line;
+	
+			/* Step over the size, which we hope is > than  SWPARSE_MKUP_LEN_WIDTH
+				digits long, otherwise there are other problems */
+
+			while (isdigit((int)(*t))) {
+				t++;
+			}
+
+			if (*t == '\0' || *t != ' ') {
+				fprintf(stderr, "internal error in swheaderline_get_type_pointer\n");
+				return NULL;
+			}
+
+			/* step over the space */
+			t++;
+	
+			if (
+				t[0] == SWPARSE_MD_TYPE_ATT || 
+				t[0] == SWPARSE_MD_TYPE_OBJ ||
+				t[0] == SWPARSE_MD_TYPE_EXT || 
+				t[0] == SWPARSE_MD_TYPE_FILEREF || 
+				0
+			)  {
+				;
+				/* good */
+			} else {
+				fprintf(stderr, "internal error in swheaderline_get_keyword loc=2\n");
+				return NULL;
+			}
+			ret = t;
+		}
+		return ret;
 	} else {
 		return output_line;
 	}
@@ -81,11 +151,12 @@ int
 swheaderline_get_level(char *outputline)
 {
 	char * typep;
-	if (isdigit((int) (*outputline))) {
-		typep = outputline  + SWPARSE_MKUP_LEN_WIDTH + 1;
-	} else {
-		typep = outputline;
+	typep = swheaderline_get_type_pointer(outputline);
+	if (!typep) { 
+		fprintf(stderr, "internal error in swheaderline_get_level\n");
+		exit(2);
 	}
+
 	typep += 2;
 		/*
 		* NOTE : MAX level is 9 since only the one's digit is scanned.
@@ -103,11 +174,14 @@ swheaderline_get_type(char *output_line)
 char *
 swheaderline_get_keyword(char *output_line)
 {
-	char *t, *ret;
+	char *t;
+	char *ret;
 
 	if (!output_line) return (char*)(NULL);
 
 	if (isdigit((int) (*output_line))) {
+		/* this step over the length field which is, which
+		   is a fixed length unless the attribute value is very long */
 		t = output_line + SWPARSE_MKUP_LEN_WIDTH + 1;
 	} else {
 		t = output_line;
@@ -132,8 +206,9 @@ swheaderline_get_keyword(char *output_line)
 		/* good */
 	} else {
 		/* bad */
-		fprintf(stderr, "internal error in swheaderline_get_keyword\n");
-		return NULL;
+		/* Ok, now start from scratch, it must be a long attribute value */
+		t = swheaderline_get_type_pointer(output_line);
+		if (t == NULL) return NULL;
 	}
 
 	t+=3;
@@ -203,3 +278,21 @@ swheaderline_get_value(char *output_line, int *value_len)
 {
 	return get_value(output_line, value_len, 1);
 }
+
+	/* Flag1 is only valid for object lines */
+void
+swheaderline_set_flag1(char * output_line) {
+	*(output_line + 1) = '0';
+}
+
+	/* Flag1 is only valid for object lines */
+void
+swheaderline_clear_flag1(char * output_line) {
+	*(output_line + 1) = ' ';  /* original value from parser */
+}
+
+	/* Flag1 is only valid for object lines */
+int
+swheaderline_get_flag1(char * output_line) {
+	return *(output_line + 1) == ' ' ? 0 : 1;
+}
