@@ -2,13 +2,13 @@
  */
 
 /*
-  Copyright (C) 2004, 2005, 2006 Jim Lowe
+  Copyright (C) 2004,2005,2006,2007 Jim Lowe
   All Rights Reserved.
   
   COPYING TERMS AND CONDITIONS:
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
+  the Free Software Foundation; either version 3, or (at your option)
   any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,11 +16,12 @@
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 #define FILENEEDDEBUG 1 /* controls the E_DEBUG macros */
 #undef FILENEEDDEBUG
+
 #include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,6 +56,248 @@
 #define SWICOL_E_DEBUG3(arg, arg1, arg2)
 #endif 
 
+#define SWICOL_STOP_STRING "305:"
+#define SWICOL_START_STRING "304:"
+#define SWICOL_CTS_STRING "318:"
+
+static
+char * 
+subshell_marks(STROB * subsh, char * type, int wh, 
+			int nhops, int verbose_level, char * group_delim)
+{
+	int is_subshell;
+
+	is_subshell = (*group_delim == '(');
+
+	strob_strcpy(subsh, "");
+	if (wh == (int)'L') {
+		/*
+		 * Left
+		 */
+		strob_sprintf(subsh, STROB_DO_APPEND, "%s\n", group_delim);
+	} else if (wh == (int)'R') {
+		/*
+		 * Right
+		 */
+		strob_sprintf(subsh, STROB_DO_APPEND, "exit $sw_retval;\n%s", group_delim+2);
+		if (is_subshell && strcmp(type, "source") == 0) {
+			/*
+			* Source
+			*/
+			if (nhops >= 1) {
+				if (verbose_level == 0) {
+					strob_strcat(subsh, 
+						"0</dev/null 2>/dev/null");
+				} else {
+					strob_strcat(subsh, "0</dev/null");
+				}
+			} else {
+				if (verbose_level == 0) {
+					strob_strcat(subsh, "2>/dev/null");
+				} else {
+					strob_strcat(subsh, "0</dev/null");
+				}
+			}
+		} else if (is_subshell && strcmp(type, "target") == 0) {
+			/*
+			* Target
+			*/
+			if (nhops >= 1) {
+				if (verbose_level == 0) {
+					strob_strcat(subsh, 
+						"1>/dev/null 2>/dev/null");
+				} else {
+					strob_strcat(subsh, "1>/dev/null");
+				}
+			} else {
+				if (verbose_level == 0) {
+					strob_strcat(subsh, 
+						"1>/dev/null 2>/dev/null");
+				} else {
+					strob_strcat(subsh, "1>/dev/null");
+				}
+			}
+		} else if (is_subshell && strcmp(type, "install_target") == 0) {
+			if (nhops >= 1) {
+				if (verbose_level == 0) {
+					strob_strcat(subsh, "");
+				} else {
+					strob_strcat(subsh, "");
+				}
+			} else {
+				if (verbose_level == 0) {
+					strob_strcat(subsh, 
+						"2>/dev/null");
+				} else {
+					strob_strcat(subsh, "");
+				}
+			}
+		} else {
+			if (is_subshell) {
+				fprintf(stderr, "internal error in swicol_task_wait at line %d\n", __LINE__);
+				exit(1);
+			}
+		}
+		if (is_subshell) {
+			strob_strcat(subsh, "; exit $?");
+		} else {
+			strob_strcat(subsh, "");
+		}
+	} else {
+		/* error */
+		;
+	}
+	return strob_str(subsh);
+}
+
+static
+void
+init_magic_header(SWICOL * swicol)
+{
+	int i;
+	STROB * tmp;
+	/* This makes a string 513 bytes long */
+	tmp = strob_open(10);
+	for (i=0; i<9; i++)
+		strob_sprintf(tmp, 1, "%s\n", "#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_##");
+	swicol->magic_headerM = strob_release(tmp);
+	if (strlen(swicol->magic_headerM) < 513) exit(42);
+}
+
+
+/**
+ * task_wait_for - wait for task
+ *
+ * Returns 0 on success, Non-zero on error
+ */
+
+static
+int
+task_wait_for(SWICOL * swicol, STROB * retbuf, 
+			int event_fd, int timelimit, char * stop_string)
+{
+	int retval = -1;
+	int ret = 0;
+	int cret;
+	int do_stop;
+	int readReturn;
+	STROB * buf = strob_open(180);
+	char * s;
+	time_t start = time(NULL);
+	time_t now = start;
+	struct timespec req;
+              
+	req.tv_sec = 0;  	/* seconds */
+	req.tv_nsec = 297000;  /* 1/30th second in nanoseconds */
+
+	E_DEBUG2("Entering: %s", stop_string);	
+	if (retbuf) strob_strcpy(retbuf, "");
+
+	cret = 0;
+	do_stop = 0;
+	E_DEBUG("");	
+	while(
+		(cret == 0) || 
+		(do_stop == 0 && ((int)(now - start) < timelimit))
+	     )
+	{
+		nanosleep(&req, (struct timespec *)(NULL));
+		now = time(NULL);
+		ret = swgpReadLine(buf, event_fd, &readReturn);
+		if (cret == 0) {
+			cret = ret;
+		}
+	
+		E_DEBUG2("read buf: [%s]", strob_str(buf));	
+		s = strstr(strob_str(buf), stop_string);
+		if (s) {
+			/*
+			 * Got the terminating event.
+			 * The terminating event is 305 "SWI_TASK_ENDS"
+			 */
+
+			if (
+				strlen(s) == 6 /* e.g. "305:0\n" */ &&
+				*(s + 5) == '\n' 
+			) {
+				/*
+				 * Normal stop
+				 */
+				E_DEBUG("Normal stop");
+				retval = 0;
+				do_stop = 1;
+			} else if (
+				strstr(s, SWICOL_START_STRING) &&
+				*(s + strlen(s) - 1) == '\n'
+			) {
+				/*
+				 * Normal stop
+				 */
+				E_DEBUG("Got Start");
+				retval = 0;
+				do_stop = 1;
+			} else if (strlen(s) > 8 && strstr(s, SWICOL_STOP_STRING)) {
+				/*
+				 * error
+				 */
+				E_DEBUG("");	
+				retval = 1;
+				do_stop = 1;
+				fprintf(stderr, "internal error in swicol_task_wait at line %d\n", __LINE__);
+				fprintf(stderr, "expect string [%s]\n", s);
+			} else {
+				/*
+				* OK
+				* partial read on stop event
+				*/
+				E_DEBUG("partial read");	
+				;	
+			}
+		} else {
+			/* fprintf(stderr, "JLXX [%s]\n", strob_str(buf)); */
+			/*
+			 * OK
+			 */
+			E_DEBUG("nothing");	
+			;
+		}
+		if (ret && retbuf) 
+			strob_strcat(retbuf, strob_str(buf));
+	}
+	if (do_stop == 0) {
+		/*
+		 * time limit exceeded.
+		 */
+		E_DEBUG("time limit exceeded");	
+		retval = 2;
+		swutil_doif_writef(swicol->verbose_levelM, 1, swicol->logspecM,
+				STDERR_FILENO,
+				"SW_RESOURCE_ERROR: time limit of %d seconds exceeded\n",
+							timelimit);
+
+	}
+	strob_close(buf);
+	E_DEBUG2("Leaving: %d", retval);	
+	return retval;
+}
+
+static
+void
+form_debug_task_filename(char * buf)
+{
+	char * s;
+	s = buf;
+
+	while (*s) {
+		if (*s == ' ') *s = '_';
+		if (*s == ':') {
+			*s = '\0';
+			break;
+		}
+		s++;
+	}
+}
+
 static
 int
 set_selected_index(SWICOL * swicol, int first_current_event)
@@ -83,12 +326,14 @@ convert_result_to_event_list(char * event_string, STRAR * event_list)
 	int ret;
 
 	/* fprintf(stderr, "JLX event string = [%s]\n", event_string);  */
+	E_DEBUG2("event string: %s", event_string);	
 
-	ret = strar_max_index(event_list);
+	ret = strar_num_elements(event_list);
 	s = strchr(event_string, '\n');
 	current = event_string;
 	while(s && *s) {
 		*s = '\0';
+		E_DEBUG2("adding event to event list: %s", current);	
 		strar_add(event_list, current);	
 		s++;
 		current = s;
@@ -101,81 +346,25 @@ convert_result_to_event_list(char * event_string, STRAR * event_list)
 	strar_add(event_list, "\n");	
 
 	/*
-	 * return the position of the start of these current events
+	 * return the index of these current events
 	 */
-	return ret /* JLx + 1 */;
+	return ret;
 }
 
 static
 int
-print_task_group(SWICOL * swicol, STROB * group_script,
-		char * script_name, int num_shells)
-{
-	STROB * subsh;
-	STROB * event_msg;
-	int i;
-
-	subsh = strob_open(32);
-	event_msg = strob_open(32);
-
-	strob_sprintf(group_script, STROB_NO_APPEND,
-	"	%s\n" /* Task Group Begins */
-	"	%s"
-	"	sw_retval=0\n"
-	"	%s"
-	"	umask %s\n"
-	"	blocksize=\"%s\"\n"
-	"	xtarget=\"%s\"\n"
-	"	wcwd=`pwd`\n"
-	"	swexec_status=0\n"
-	,
-	TEVENT(2, swicol->verbose_levelM, SWI_GROUP_BEGINS, script_name),
-	swicol_subshell_marks(subsh, "target", 'L', swicol->nhopsM, swicol->verbose_levelM),
-	swicol->setvxM,
-	swicol->umaskM,
-	swicol->blocksizeM,
-	swicol->targetpathM
-	);
-
-	/*
-	 * now write the working shells that are in this group
-	 */
-	for (i=0; i<num_shells; i++) {
-		strob_sprintf(group_script, STROB_DO_APPEND,
-		"	case $swexec_status in	0) %s ;; *) false_ ;; esac\n" 
-		"	sw_retval=$?\n"
-		"	swexec_status=$sw_retval\n"
-		);
-	}
-
-	/*
-	 * Now write the final pieces of this group
-	 */
-	strob_sprintf(event_msg, STROB_NO_APPEND, "%s: status=$sw_retval\n", script_name);
-	strob_sprintf(group_script, STROB_DO_APPEND,
-	"	sleep %d\n"
-	"	%s\n" 
-	"%s\n",
-	swicol->delaytimeM,
-	TEVENT(2, swicol->verbose_levelM, SWI_GROUP_ENDS, strob_str(event_msg)),
-	swicol_subshell_marks(subsh, "install_target", 'R', swicol->nhopsM, swicol->verbose_levelM)
-	);
-
-	strob_close(event_msg);
-	strob_close(subsh);
-	return 0;
-}
-
-static
-int
-print_task_script(SWICOL * swicol, STROB * command, unsigned long int size, char * dir, char * task_script, char * task_desc)
+print_task_script(SWICOL * swicol, STROB * command, uintmax_t size, char * dir, char * task_script, char * task_desc)
 {
 	char * verbosestring;
 	char * tonullstring;
-	STROB * enddesc = strob_open(32);
-
-	int blocks = (int)(size / 512);
+	char * ablocks;
+	STROB * enddesc;
+	STROB * ubuf;
+	uintmax_t blocks;
 	
+	ubuf = strob_open(32);
+	enddesc = strob_open(32);
+	blocks = (int)(size / 512);
 	if (size % 512) {
 		/*
 		 * size must be a whole number of 512 blocks.
@@ -195,51 +384,56 @@ print_task_script(SWICOL * swicol, STROB * command, unsigned long int size, char
 	if (swicol->verbose_levelM > SWC_VERBOSE_8) {
 		verbosestring = "set -vx";
 	} else {
-		verbosestring = "";
+		verbosestring = "#set -vx";
 	}
 	
-	if (swicol->verbose_levelM >= SWC_VERBOSE_7) {
-		tonullstring = "";
+	if (swicol->verbose_levelM >= SWC_VERBOSE_5) {
+		tonullstring = " ";
 	} else {
 		tonullstring = "2>/dev/null";
 	}
 
 	strob_sprintf(enddesc, 0, "%s: status=$sw_retval", task_desc);
 
-	strob_sprintf(command, 1,
-		"(\n"
-		"	%s\n"			/* SWI_TASK_BEGINS */
-		"	swxdir=\"%s\"\n"
-		"	cd \"$swxdir\"\n"
-		"	swret=$?; export swret\n"
-		"	%s\n"
-		"	dd bs=512 count=\"%d\" %s | (\n"
-		"		case $swret in\n"
-		"			0)\n"
-		" 				;;\n"
-		"			*)\n"
-		"				dd count=\"%d\" of=/dev/null\n"
-		"				%s\n"
-		"				exit \"$swret\"\n"
-		"				;;\n"
-		"		esac\n"
-		"		%s"		/* Task main script */
-		"		exit \"$sw_retval\"\n"
-		"	); sw_retval=$?\n"
-		"	%s\n"			/* SWI_TASK_ENDS */
-		"	exit $sw_retval\n"	/* exit the task shell */
-		"); exit $sw_retval\n"
+	ablocks = swlib_umaxtostr(blocks, ubuf);
+
+	strob_sprintf(command, 0,
+		"{\n"
+		"dd bs=512 count=%s %s | (\n"
+		"# trap true 1 2 13 14 15\n"
+		"%s\n"			/* SWI_TASK_BEGINS */
+		"swxdir=\"%s\"\n"
+		"cd \"$swxdir\"\n"
+		"swret=$?; export swret\n"
+		"%s\n"
+		"	case $swret in\n"
+		"		0)\n"
+		" 			;;\n"
+		"		*)\n"
+		"			dd count=%d of=/dev/null 2>/dev/null\n"
+		"			%s\n"
+		"			exit \"$swret\"\n"
+		"			;;\n"
+		"	esac\n"
+		"%s"		/* Task main script */
+		"	exit \"$sw_retval\"\n"
+		"); sw_retval=$?\n"
+		"%s\n"			/* SWI_TASK_ENDS */
+		"exit $sw_retval\n"	/* exit the task shell */
+		"}\n"
 		,
+		ablocks,
+		tonullstring,
 		TEVENT(2, -1, SWI_TASK_BEGINS, task_desc),
 		dir,
 		verbosestring,
-		blocks,
-		tonullstring,
-		blocks,
+		ablocks,
 		TEVENT(2, -1, SW_INTERNAL_ERROR, "No such directory or no access: $swxdir"),
 		task_script,
 		TEVENT(2, -1, SWI_TASK_ENDS, strob_str(enddesc))
 		);
+
+	strob_close(ubuf);
 	strob_close(enddesc);
 	return 0;
 }
@@ -249,7 +443,9 @@ swicol_create(void)
 {
 	SWICOL * swicol;
 	swicol = (SWICOL*)malloc(sizeof(SWICOL));
+	if (swicol == NULL) return NULL;
 	swicol->tmpM = strob_open(10);
+	swicol->scriptM = strob_open(10);
 	swicol->logspecM = (struct sw_logspec *)(NULL);
 	swicol->verbose_levelM = 0;
 	swicol->event_listM = strar_open();
@@ -260,15 +456,29 @@ swicol_create(void)
 	swicol->targetpathM = (char*)NULL;
 	swicol->nhopsM = 1;
 	swicol->event_indexM = 1;
+	swicol->id_stringM = strob_open(24);
+	swicol->debug_task_scriptsM = 0;
+	init_magic_header(swicol);
+	swicol_clear_master_alarm(swicol);
+	swicol_set_event_fd(swicol, -1);
 	return swicol;
 }
 
 void
 swicol_delete(SWICOL * swicol)
 {
+	strob_close(swicol->tmpM);
+	strob_close(swicol->scriptM);
+	strob_close(swicol->id_stringM);
 	free(swicol);
 }
 
+void
+swicol_set_event_fd(SWICOL * swicol, int fd)
+{
+	swicol->event_fdM = fd;
+}
+
 void
 swicol_set_delaytime(SWICOL * swicol, int delay)
 {
@@ -299,11 +509,13 @@ swicol_set_setvx(SWICOL * swicol, char * buf)
 	set_buf(&(swicol->setvxM), buf);
 }
 
+/*
 void
 swicol_set_blocksize(SWICOL * swicol, char * buf)
 {
 	set_buf(&(swicol->blocksizeM), buf);
 }
+*/
 
 void
 swicol_set_targetpath(SWICOL * swicol, char * buf)
@@ -329,149 +541,147 @@ swicol_get_blocksize(SWICOL * swicol)
 	return swicol->blocksizeM;
 }
 
+/**
+ * swicol_rpsh_task_send_script2 - send script to shell stdin
+ *
+ * Return 0 on success, -1 on error
+ */
+
 int
-swicol_rpsh_task_send_group_script(SWICOL * swicol, int fd,
-			char * script_name, int num_shells)
+swicol_rpsh_task_send_script2(SWICOL * swicol, int fd,
+		uintmax_t data_size, char * dir, char * script, char * f_desc)
 {
+	STROB * tmp;
 	int ret;
-	int eret;
-	STROB * tmp = strob_open(1);
+	char * desc;
 
-	if (print_task_group(swicol, tmp, script_name, num_shells)) {
-		return -2;
-	}
-	ret = atomicio(
-			((ssize_t (*)(int, void *, size_t))write),
-			fd, 
-			(void*)(strob_str(tmp)),
-			(size_t)(strob_strlen(tmp))
-		);
-
-	eret = strob_strlen(tmp);
-	strob_close(tmp);
-	if (ret == eret)
-		return 0;
-	else 
+	desc = f_desc;	
+	if (swicol_get_master_alarm_status(swicol) == SWICOL_ABORT_STEP_2) {
 		return -1;
+	} else if (swicol_get_master_alarm_status(swicol) == SWICOL_ABORT_STEP_1) {
+		/* do the task script but intensionally send a wrong task
+		   id_string, this will cause the target main script to abort */
+		swicol->master_alarmM = SWICOL_ABORT_STEP_2;
+		desc =  SWICOL_ALARM_EVENT;
+	}
+
+	tmp = swicol->scriptM;
+	strob_strcpy(tmp, "");
+
+	strob_sprintf(tmp, 1, "%s", swicol->magic_headerM);
+	strob_sprintf(tmp, 1, "# SWI_TASK: %s\n", desc);
+	strob_sprintf(tmp, 1, "\n");
+	ret = atomicio((ssize_t (*)(int, void *, size_t))write,
+		fd, (void*)(strob_str(tmp)), (size_t)(strob_strlen(tmp)));
+	if (ret != (int)strob_strlen(tmp)) {
+		return -1;
+	}
+	E_DEBUG("");
+	swicol_set_task_idstring(swicol, f_desc);
+	ret = swicol_rpsh_task_send_script(swicol, fd, data_size, dir, script, desc);
+	E_DEBUG("");
+
+	if (swicol->debug_task_scriptsM) {
+		E_DEBUG("");
+		swicol_write_debug_task_script(swicol, strob_str(swicol->scriptM));
+	}
+
+	if (swicol->event_fdM >= 0)
+		swicol_rpsh_wait_304(swicol, swicol->event_fdM);
+
+	E_DEBUG("");
+	if (swicol_get_master_alarm_status(swicol) != 0) {
+		if (ret == 0)
+			return -1;
+	}
+	return ret;
 }
 
+/**
+ * swicol_rpsh_task_send_script - send script to shell stdin
+ *
+ * Return 0 on success, -1 on error
+ */
 int
 swicol_rpsh_task_send_script(SWICOL * swicol, int fd,
-		unsigned long int data_size, char * dir, char * script, char * desc)
+		uintmax_t data_size, char * dir, char * script, char * desc)
 {
 	int ret;
 	int eret;
-	STROB * tmp = strob_open(1);
+	STROB * tmp;
 	
+	tmp = swicol->scriptM;
+	strob_strcpy(tmp, "");
+
 	if (data_size == 0) {
 		fprintf(stderr, "%s: Error: a non-zero length data payload is required.\n",
 				swlib_utilname_get());
 	}
 
 	if (print_task_script(swicol, tmp, data_size, dir, script, desc)) {
+		fprintf(stderr, "rpsh_task_send_script: error forming task script\n");
 		return -2;
 	}
+	
+	E_DEBUG2("\n<AA>\n%s</AA>\n", strob_str(tmp));
 
 	ret = atomicio((ssize_t (*)(int, void *, size_t))write,
 		fd, (void*)(strob_str(tmp)), (size_t)(strob_strlen(tmp)));
 
 	eret = strob_strlen(tmp);
-	strob_close(tmp);
-	if (ret == eret)
+	if (ret == eret) {
 		return 0;
-	else 
+	} else {
+		fprintf(stderr, "rpsh_task_send_script: error writing task script to fd=%d: %s\n", fd, strerror(errno));
 		return -1;
+	}
+}
+
+int
+swicol_rpsh_wait_cts(SWICOL * swicol, int event_fd)
+{
+	int ret = 0;
+	int first_current_event;
+	STROB * buf;
+	char * stop_string = SWICOL_CTS_STRING;
+	
+	buf = strob_open(32);
+	ret = task_wait_for(swicol, buf, event_fd, SWICOL_TL_20, stop_string);
+	first_current_event = convert_result_to_event_list(strob_str(buf), swicol->event_listM);
+	swicol->event_indexM = first_current_event;
+	strob_close(buf);
+	return ret;
+}
+
+int
+swicol_rpsh_wait_304(SWICOL * swicol, int event_fd)
+{
+	int ret = 0;
+	char * stop_string = SWICOL_START_STRING;
+	ret = task_wait_for(swicol, (STROB*)NULL, event_fd, SWICOL_TL_9, stop_string);
+	return ret;
 }
 
 int
 swicol_rpsh_task_wait(SWICOL * swicol, STROB * retbuf, 
 			int event_fd, int timelimit)
 {
-	int retval = -1;
 	int ret = 0;
-	int cret;
-	int do_stop;
-	int readReturn;
-	STROB * buf = strob_open(180);
-	char * s;
-	char * stop_string = "305" ":";   /* SWI_TASK_ENDS */
-	time_t start = time(NULL);
-	time_t now = start;
-	struct timespec req;
-              
-	req.tv_sec = 0;  	/* seconds */
-	req.tv_nsec = 30000000;  /* 1/30th second in nanoseconds */
+	char * stop_string = SWICOL_STOP_STRING;   /* SWI_TASK_ENDS */
+	ret = task_wait_for(swicol, retbuf, event_fd, timelimit, stop_string);
+	return ret;
+}
 
-	strob_strcpy(retbuf, "");
-
-	cret = 0;
-	do_stop = 0;
-	while(
-		(cret == 0) || 
-		(do_stop == 0 && ((int)(now - start) < timelimit))
-	     )
-	{
-		nanosleep(&req, (struct timespec *)(NULL));
-		now = time(NULL);
-		ret = swgpReadLine(buf, event_fd, &readReturn);
-		if (cret == 0) {
-			cret = ret;
-		}
-	
-		s = strstr(strob_str(buf), stop_string);
-		if (s) {
-			/*
-			 * Got the terminating event.
-			 * The terminating event is 305 "SWI_TASK_ENDS"
-			 */
-
-			if (
-				strlen(s) == 6 /* e.g. "305:0\n" */ &&
-				*(s + 5) == '\n' 
-			) {
-				/*
-				 * Normal stop
-				 */
-				retval = 0;
-				do_stop = 1;
-			} else if (strlen(s) > 8) {
-				/*
-				 * error
-				 */
-				retval = 1;
-				do_stop = 1;
-				fprintf(stderr, "internal error in swicol_task_wait at line %d\n", __LINE__);
-				fprintf(stderr, "expect string [%s]\n", s);
-			} else {
-				/*
-				* OK
-				* partial read on stop event
-				*/
-				;	
-			}
-		} else {
-			/* fprintf(stderr, "JLXX [%s]\n", strob_str(buf)); */
-			/*
-			 * OK
-			 */
-			;
-		}
-		if (ret) 
-			strob_strcat(retbuf, strob_str(buf));
-	}
-	if (do_stop == 0) {
-		/*
-		 * time limit exceeded.
-		 */
-		retval = 2;
-		swutil_doif_writef(swicol->verbose_levelM, 1, swicol->logspecM,
-				STDERR_FILENO,
-				"SW_RESOURCE_ERROR: time limit of %d seconds exceeded\n",
-							timelimit);
-
-	}
+int
+swicol_rpsh_wait_for_event(SWICOL * swicol, STROB * retbuf, int event_fd, int event)
+{
+	int ret;
+	STROB * buf;
+	buf = strob_open(32);
+	strob_sprintf(buf, 0, "%d:", event);
+	ret = task_wait_for(swicol, (STROB*)NULL, event_fd, SWICOL_TL_10, strob_str(buf));
 	strob_close(buf);
-	return retval;
+	return ret;
 }
 
 char *
@@ -535,6 +745,12 @@ swicol_rpsh_get_event_status(SWICOL * swicol, char * msg,
 	return retval;
 }
 
+/**
+ * swicol_rpsh_task_expect - wait for a task and read exit status
+ *
+ * Return task exit status, or -1 on internal error
+ */
+
 int
 swicol_rpsh_task_expect(SWICOL * swicol, int event_fd, int timelimit)
 {
@@ -561,20 +777,20 @@ swicol_rpsh_task_expect(SWICOL * swicol, int event_fd, int timelimit)
 		/*
 		* SWI_TASK_ENDS event not found.
 		*/
-		fprintf(stderr, "%s: Event error: SWI_TASK_END event not received.\n", swlib_utilname_get());
+		fprintf(stderr, "%s: error: END event not received for task: %s\n",
+			swlib_utilname_get(), strob_str(swicol->id_stringM));
 		if (retval == 0) retval = -1;
 	} else {
 		ev += 4;
 		if (isdigit((int)(*ev)) == 0) {
-			fprintf(stderr, "%s: Event error: SWI_TASK_END event has invalid status. [%s]\n",
-				swlib_utilname_get(),  strob_str(buf));
+			fprintf(stderr, "%s: error: END event has invalid status: %s status=%s\n",
+				swlib_utilname_get(), strob_str(swicol->id_stringM), strob_str(buf));
 			retval = -1;
 		} else {
 			retval = swlib_atoi(ev, NULL);
 		}
 	}
 
-	/* fprintf(stderr, "start\n%s\nend\n", strob_str(buf)); */
 	/*
 	 * Store all the events.
 	 */ 
@@ -661,78 +877,13 @@ char *
 swicol_subshell_marks(STROB * subsh, char * type, int wh, 
 			int nhops, int verbose_level)
 {
-	strob_strcpy(subsh, "");
-	if (wh == (int)'L') {
-		/*
-		 * Left
-		 */
-		strob_strcat(subsh, "(\n");
-	} else if (wh == (int)'R') {
-		/*
-		 * Right
-		 */
-		strob_strcat(subsh, "exit $sw_retval;\n) ");
-		if ( strcmp(type, "source") == 0) {
-			/*
-			* Source
-			*/
-			if (nhops >= 1) {
-				if (verbose_level == 0) {
-					strob_strcat(subsh, 
-						"0</dev/null 2>/dev/null");
-				} else {
-					strob_strcat(subsh, "0</dev/null");
-				}
-			} else {
-				if (verbose_level == 0) {
-					strob_strcat(subsh, "2>/dev/null");
-				} else {
-					strob_strcat(subsh, "0</dev/null");
-				}
-			}
-		} else if ( strcmp(type, "target") == 0) {
-			/*
-			* Target
-			*/
-			if (nhops >= 1) {
-				if (verbose_level == 0) {
-					strob_strcat(subsh, 
-						"1>/dev/null 2>/dev/null");
-				} else {
-					strob_strcat(subsh, "1>/dev/null");
-				}
-			} else {
-				if (verbose_level == 0) {
-					strob_strcat(subsh, 
-						"1>/dev/null 2>/dev/null");
-				} else {
-					strob_strcat(subsh, "1>/dev/null");
-				}
-			}
-		} else if (strcmp(type, "install_target") == 0) {
-			if (nhops >= 1) {
-				if (verbose_level == 0) {
-					strob_strcat(subsh, "");
-				} else {
-					strob_strcat(subsh, "");
-				}
-			} else {
-				if (verbose_level == 0) {
-					strob_strcat(subsh, 
-						"2>/dev/null");
-				} else {
-					strob_strcat(subsh, "");
-				}
-			}
-		} else {
-			fprintf(stderr, "internal error in swicol_task_wait at line %d\n", __LINE__);
-			exit(1);
-		}
-		strob_strcat(subsh, "; exit $?");
-	} else {
-		;
-	}
-	return strob_str(subsh);
+	return subshell_marks(subsh, type, wh, nhops, verbose_level, "(\x00)");
+}
+
+char * 
+swicol_brace_marks(STROB * subsh, char * type, int wh, int nhops, int verbose_level)
+{
+	return subshell_marks(subsh, type, wh, nhops, verbose_level, "{\x00}");
 }
 
 int
@@ -741,64 +892,72 @@ swicol_initiate_fall_thru(SWICOL * swicol)
 	return 0;
 }
 
-int
-swicol_NOT_USED_get_event_status(SWICOL * swicol, int event_value, int first_current_event)
+void
+swicol_clear_task_idstring(SWICOL * swicol)
 {
-	int ret = -1;
-	int ix;
-	char * v;
-	STROB * buf;
-	STROB * tmp;
-
-	tmp = strob_open(48);
-	buf = strob_open(48);
-
-	ix = set_selected_index(swicol, first_current_event);
-
-	/*
-	 * print the events of the selected task script
-	 */
-	swicol_print_events(swicol, buf, ix);
-
-	/*
-	 * buf now has something similar to this
-	 *
-	 * swinstall: swicol: event stack start
-	 * swinstall: swicol: 304:analysis phase 2
-	 * swinstall: swicol: 73:2
-	 * swinstall: swicol: 305:0
-	 * swinstall: swicol: event stack end
-	 */
-
-	strob_sprintf(tmp, STROB_NO_APPEND, ": %d:", event_value);
-
-	/*
-	 * now find this substr in buf
-	 */
-	v = strstr(strob_str(buf), strob_str(tmp));
-
-	if (!v) ret = -1; /* not found */
-
-	v+=(strob_strlen(tmp));
-
-	/*
-	 * the next byte is the status
-         * make sure it is there
-	 */
-
-	if (isdigit((int)(*v)) == 0) {
-		/*
-		 * this event has no status
-		 */
-		 ret = -2;
+	swicol_set_task_idstring(swicol, "__unset__");
+}
+
+void
+swicol_set_task_debug(SWICOL * swicol, int do_debug)
+{
+	swicol->debug_task_scriptsM = do_debug;
+}
+
+void
+swicol_set_task_idstring(SWICOL * swicol, char * id)
+{
+	strob_strcpy(swicol->id_stringM, id);
+}
+
+char *
+swicol_get_task_idstring(SWICOL * swicol)
+{
+	return strob_str(swicol->id_stringM);
+}
+
+void
+swicol_write_debug_task_script(SWICOL * swicol, char * script)
+{
+	int ret;
+	STROB * name;
+	name = strob_open(32);
+	strob_strcpy(name, SWICOL_DEBUG_TASK_SCRIPT_PREFIX );
+	strob_strcat(name, strob_str(swicol->id_stringM));
+	form_debug_task_filename(strob_str(name));
+	ret = swlib_tee_to_file(strob_str(name), -1, script, -1, 0 /*do_append*/);
+	E_DEBUG3("name=[%s] ret=%d", strob_str(name), ret);
+	strob_close(name);
+}
+
+void
+swicol_set_master_alarm(SWICOL * swicol)
+{
+	swicol->master_alarmM = SWICOL_ABORT_STEP_1;
+}
+
+void
+swicol_clear_master_alarm(SWICOL * swicol)
+{
+	swicol->master_alarmM = 0;
+}
+
+int
+swicol_get_master_alarm_status(SWICOL * swicol)
+{
+	return swicol->master_alarmM;
+}
+
+int
+swicol_send_loop_trailer(SWICOL * swicol, int fd)
+{
+	int ret;
+	char * send = SWICOL_TRAILER "\n";
+	ret = atomicio((ssize_t (*)(int, void *, size_t))write,
+			fd, send, strlen(send));
+	if (ret == (int)strlen(send)) {
+		return 0;
 	} else {
-		/*
-		 * convert the status
-		 */
-		ret = swlib_atoi(v, NULL);
+		return -1;
 	}
-
-	strob_close(tmp);
-	strob_close(buf);
-	return ret;
 }
