@@ -2,13 +2,13 @@
  */
 
 /*
-   Copyright (C) 2003,2004 James H. Lowe, Jr.  <jhlowe@acm.org>
+   Copyright (C) 2003,2004,2006 James H. Lowe, Jr. <jhlowe@acm.org>
    All Rights Reserved.
   
    COPYING TERMS AND CONDITIONS
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
   
    This program is distributed in the hope that it will be useful,
@@ -18,9 +18,10 @@
   
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
 
+#include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,6 +48,7 @@
 static int g_active_flag = 1;
 static int g_did_get_command = 0;
 
+
 static
 int
 is_posix_shell_command(char *s)
@@ -59,6 +61,10 @@ is_posix_shell_command(char *s)
 		*/
 		char * sp = strdup(s);
 		char * p = sp;
+
+		/* disable the test below, it is currently broken */
+		return 1;
+
 		while(p && *p) {
 			if (*p == '\\')
 				memmove(p, p+1, strlen(p+1) + 1);
@@ -97,7 +103,7 @@ static
 int
 bail_on_taint(char *s) {
 	if (swlib_is_sh_tainted_string(s)) {
-		fprintf(stderr, "arg contains shell meta characters\n");
+		fprintf(stderr, "arg contains shell meta characters [%s]\n", s);
 		exit(2);
 	}
 	return 0;
@@ -152,6 +158,7 @@ cat_escapes(STROB * command, int nhops)
 	return 0;
 }
 
+static
 int
 form_intermediate_msg(STROB * command, char * host, int nhops)
 {
@@ -212,6 +219,37 @@ add_host_message(int doimsg, SHCMD * cmd, char * host, int cmdcount)
 	strob_close(t);
 }
 
+static
+void
+parse_host_port(char * host, SHCMD * cmd, SHCMD * kmd)
+{
+	char * u;
+	char * s;
+	if ((u=strchr(host, '_')) == NULL) {		
+		safe_shcmd_add_arg(kmd, host);
+		safe_shcmd_add_arg(cmd, host);
+	} else {
+		int status;
+		s = u;
+		s++;		
+		swlib_atoi(s, &status);
+		if (status == 0) {
+			*u = '\0';
+			safe_shcmd_add_arg(kmd, "-p");
+			safe_shcmd_add_arg(cmd, "-p");
+			safe_shcmd_add_arg(kmd, s);
+			safe_shcmd_add_arg(cmd, s);
+			safe_shcmd_add_arg(kmd, host);
+			safe_shcmd_add_arg(cmd, host);
+			*u = '_';
+		} else {
+			/* probably a illegal host name */
+			safe_shcmd_add_arg(kmd, host);
+			safe_shcmd_add_arg(cmd, host);
+		}
+	}
+}
+
 void
 swssh_deactivate_sanity_check(void)
 {
@@ -236,6 +274,8 @@ swssh_landing_command(char * shellname, int opt_no_getconf)
 			return SWSSH_ASH_SHELL_COMMAND;	
 		} else if (strcmp(shellname, "ksh") == 0) {
 			return SWSSH_KSH_SHELL_COMMAND;	
+		} else if (strcmp(shellname, "dash") == 0) {
+			return SWSSH_DASH_SHELL_COMMAND;	
 		} else if (strcmp(shellname, "posix") == 0) {
 			return SWSSH_POSIX_SHELL_COMMAND;	
 		} else {
@@ -281,11 +321,13 @@ swssh_parse_target(SHCMD * cmd,
 			char ** terminal_host, 
 			char * tty_option, 
 			int do_imsg,
-			char * sshoption)
+			char * sshoption,
+			int do_forward_auth_agent)
 {
 	int cmdcount = 0;
 	int icmdcount = 0;
 	int is_multihop = 0;
+	int nhops;
 	STROB * tmp = strob_open(10);
 	STROB * tmp2 = strob_open(10);
 	char * remote_ssh_command;
@@ -360,10 +402,20 @@ swssh_parse_target(SHCMD * cmd,
 			}
 		}
 	}
+
 	strob_strcpy(tmp, target);
-	if (strstr(target, TARGET_DELIM)) {
+	s = strob_str(tmp);
+	nhops = 0;
+	while((tok = strob_strstrtok(tmp, s, TARGET_DELIM))) {
+		nhops++;
+		s = NULL;
+	}
+	
+	if (nhops > 1) {
 		is_multihop = 1;
 	}
+
+	strob_strcpy(tmp, target);
 	s = strob_str(tmp);
 	while((tok = strob_strstrtok(tmp, s, TARGET_DELIM))) {
 		user = (char*)NULL;
@@ -406,8 +458,16 @@ swssh_parse_target(SHCMD * cmd,
 					safe_shcmd_add_arg(kmd, tty_option);
 				}
 				if (is_multihop) {
-					safe_shcmd_add_arg(cmd, "-A");
-					safe_shcmd_add_arg(kmd, "-A");
+					if (do_forward_auth_agent) {
+						if (cmdcount+1 < nhops) {
+							safe_shcmd_add_arg(cmd, "-A");
+							safe_shcmd_add_arg(kmd, "-A");
+						} else {
+							;
+							/* don't apply -A to the last host
+							   because it is not needed there */
+						}
+					}
 				}
 				if (sshoption && strlen(sshoption)) {
 					strob_strcpy(tmp2, "-");
@@ -437,8 +497,11 @@ swssh_parse_target(SHCMD * cmd,
 		}
 		
 		if (strlen(host)) {
+			parse_host_port(host, cmd, kmd);
+			/*
 			safe_shcmd_add_arg(kmd, host);
 			safe_shcmd_add_arg(cmd, host);
+			*/
 		} else if (strlen(host) == 0 && icmdcount == 0) {
 			safe_shcmd_add_arg(cmd, "-c");
 		}	
