@@ -19,15 +19,16 @@
 #define SWSSH_BASH_SHELL_COMMAND      "/bin/bash -s"
 #define SWSSH_KSH_SHELL_COMMAND      "/bin/ksh -s"
 #define SWSSH_ASH_SHELL_COMMAND      "/bin/ash -s"
+#define SWSSH_DASH_SHELL_COMMAND      "/bin/dash -s"
 
-#define SWSSH_SSH_BIN "/usr/bin/ssh"
+/* NOT USED #define SWSSH_SSH_BIN "/usr/bin/ssh" */
 #define SWC_KILL_PID_MARK "_"
 #define SWSSH_TRACK_PID "$PPID"
 
 int swssh_parse_target(SHCMD * cmd, SHCMD * killcmd, 
 			char * target, char * ssh_command, char * remote_ssh_command,
 			char ** path, char ** last_host, char * tty_option, int do_imsg,
-			char * sshoption);
+			char * sshoption, int do_forward_auth_agent);
 int swssh_run_ssh_cmd(SHCMD * shcmd, int verbose, int outfd, int infd);
 int swssh_assemble_ssh_cmd(SHCMD * shcmd, STRAR * cmdlist, STRAR * delimlist, int nhops);
 int swssh_protect_shell_metacharacters(STROB * command, int nhops, char * taints);
