@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -25,11 +25,16 @@
 #include <limits.h>
 #include "ahs.h"
 #include "swvarfs.h"
+#include "swutilname.h"
 #include "strar.h"
 #include "swlib.h"
 #include "fnmatch_u.h"
 
 #include "debug_config.h"
+
+#define SWVARFSNEEDDEBUG 1
+#undef SWVARFSNEEDDEBUG
+
 #ifdef SWVARFSNEEDDEBUG
 #define SWVARFS_E_DEBUG(format) SWBISERROR("SWVARFS DEBUG: ", format)
 #define SWVARFS_E_DEBUG2(format, arg) SWBISERROR2("SWVARFS DEBUG: ", format, arg)
@@ -247,7 +252,7 @@ int
 filename_does_match(SWVARFS * swvarfs, char * name, char * candidate)
 {
 	int ret;
-	ret = swlib_dir_compare(name, candidate, SWC_FC_NOAB /*ignore_absolute_path*/);
+	ret = swlib_compare_8859(name, candidate);
 	return ret;
 }
 
@@ -388,6 +393,7 @@ construct_new_path_from_pathspec(SWVARFS * swvarfs, STROB * newvcwd,
 	pathcomps = strar_open();
 	cmdcomps = strar_open();
 	sx = malloc(strob_strlen(swvarfs->vcwdM) + 2);
+	if (sx == NULL) exit(31);
 
 	if (*strob_str(swvarfs->vcwdM) == '/') {
 		strcpy(sx, "/");	
@@ -401,7 +407,7 @@ construct_new_path_from_pathspec(SWVARFS * swvarfs, STROB * newvcwd,
 	swlib_squash_all_dot_slash(pathspec);
 	process_vcwd(cmdcomps, pathspec);
 
-	ncomp = strar_max_index(pathcomps);
+	ncomp = strar_num_elements(pathcomps);
 
 	j = 0;
 	cx = strar_get(cmdcomps, i++);
@@ -428,7 +434,7 @@ construct_new_path_from_pathspec(SWVARFS * swvarfs, STROB * newvcwd,
 		}
 		strob_strcat(newvcwd, "/");
 		i = j;
-		while (i < strar_max_index(cmdcomps)) {
+		while (i < strar_num_elements(cmdcomps)) {
 			if (i > j) strob_strcat(newvcwd, "/");
 			strob_strcat(newvcwd, strar_get(cmdcomps, i++));
 		}
@@ -446,9 +452,10 @@ static
 void
 dump_filechain_link(struct fileChain *  link)
 {
-	fprintf(stderr, "%s header_offset=%d data_offset=%d\n",
-		link->nameFC, link->header_offsetFC, link->data_offsetFC);
-
+	STROB * ubuf = strob_open(24);
+	fprintf(stderr, "%s header_offset=%d data_offset=%s\n",
+		link->nameFC, link->header_offsetFC, swlib_imaxtostr(link->data_offsetFC, ubuf));
+	strob_close(ubuf);
 }
 
 static
@@ -991,8 +998,8 @@ swvarfs_get_next_dirent_archive_NEW(SWVARFS * swvarfs,
 				int *peoa)
 {
 	char * name;
-	int old_pos;
-	int new_pos;
+	intmax_t old_pos;
+	intmax_t new_pos;
 	*peoa = 0;
 
 	if (swvarfs->did_u_openM == 0 && swvarfs->u_current_name_) {
@@ -1006,8 +1013,8 @@ swvarfs_get_next_dirent_archive_NEW(SWVARFS * swvarfs,
 	SWVARFS_E_DEBUG("ENTERING");
 	SWVARFS_E_DEBUG2("OFFSET = %d", (int)uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR));
 
-	old_pos = (int)uxfio_lseek(swvarfs->fdM, 0 , UXFIO_SEEK_VCUR);
-	swvarfs->current_header_offsetM = (int)uxfio_lseek(swvarfs->fdM, 0, SEEK_CUR);
+	old_pos = (intmax_t)uxfio_lseek(swvarfs->fdM, 0 , UXFIO_SEEK_VCUR);
+	swvarfs->current_header_offsetM = uxfio_lseek(swvarfs->fdM, 0, SEEK_CUR);
 
 	if (taru_read_header(swvarfs->taruM,
 				ahs_vfile_hdr(swvarfs->ahsM),
@@ -1030,19 +1037,18 @@ swvarfs_get_next_dirent_archive_NEW(SWVARFS * swvarfs,
 	* the byte offset in the file (as if it were a regular file
 	* even if it is a pipe).
 	*/
-	swvarfs->current_data_offsetM = (int)uxfio_lseek(swvarfs->fdM, 
-							0, UXFIO_SEEK_VCUR);
+	swvarfs->current_data_offsetM = uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 
 	/*
 	* seek back to the start of the header because
 	* the <>_u_open routine reads the header again.
 	*/
 
-	new_pos = (int)(swvarfs->current_data_offsetM);
+	new_pos = swvarfs->current_data_offsetM;
 	SWVARFS_E_DEBUG3("swvarfs->fdM = [%d] old_pos = %d ", swvarfs->fdM, old_pos); 
 	SWVARFS_E_DEBUG3("swvarfs->fdM = [%d] new_pos = %d ", swvarfs->fdM, new_pos); 
 	SWVARFS_E_DEBUG3("swvarfs->fdM = [%d] old_pos - new_pos = %d ", swvarfs->fdM, old_pos - new_pos); 
-	if (uxfio_lseek(swvarfs->fdM, old_pos - new_pos, UXFIO_SEEK_VCUR) < 0) {
+	if (uxfio_lseek(swvarfs->fdM, (off_t)(old_pos - new_pos), UXFIO_SEEK_VCUR) < 0) {
 		SWBIS_E_FAIL("swvarfs_get_next_dirent_archive: seek error");
 		exit(88);
 		return NULL;
@@ -1078,16 +1084,17 @@ swvarfs_get_next_dirent_archive_NEW(SWVARFS * swvarfs,
 static
 struct fileChain * 
 swvarfs_i_make_link(struct new_cpio_header * file_hdr,
-				int header_offset, int data_offset) {
+				intmax_t header_offset, intmax_t data_offset) {
 	struct fileChain * link = (struct fileChain*)malloc
 						(sizeof(struct fileChain));
 	
 	SWVARFS_E_DEBUG("ENTERING"); 
+	if (link == NULL) return NULL;
 	link->nameFC = swlib_strdup(ahsStaticGetTarFilename(file_hdr));
 	swlib_slashclean(link->nameFC);
 	swlib_process_hex_escapes(link->nameFC);  /* Collapse Hex sequences into single non-ascii octet */
-	SWVARFS_E_DEBUG3("link made name=[%s] header_offset = [%d]", link->nameFC, header_offset); 
-	SWVARFS_E_DEBUG3("link made name=[%s]   data_offset = [%d]", link->nameFC, data_offset); 
+	SWVARFS_E_DEBUG3("link made name=[%s] header_offset = [%s]", link->nameFC, swlib_imaxtostr(header_offset, NULL)); 
+	SWVARFS_E_DEBUG3("link made name=[%s]   data_offset = [%s]", link->nameFC, swlib_imaxtostr(data_offset, NULL)); 
 	link->header_offsetFC = header_offset;
 	link->data_offsetFC = data_offset;
 	link->prevFC=link->nextFC = NULL;
@@ -1100,18 +1107,18 @@ struct fileChain *
 swvarfs_i_readin_filechain(SWVARFS * swvarfs, char * name)
 {
 	struct fileChain * last = swvarfs->tailM, *new_link;
-	int header_offset;
-	int data_offset;
+	intmax_t header_offset;
+	intmax_t data_offset;
 	
 	SWVARFS_E_DEBUG2("ENTERING [%s]", name); 
 	if (last) {
 		SWVARFS_E_DEBUG2("OFFSET BEFORE= %d", (int)uxfio_lseek(swvarfs->fdM, 0 , UXFIO_SEEK_VCUR));
 	
-		uxfio_lseek(swvarfs->fdM, last->header_offsetFC, SEEK_SET);
+		uxfio_lseek(swvarfs->fdM, (off_t)(last->header_offsetFC), SEEK_SET);
 	
 		SWVARFS_E_DEBUG2("OFFSET AFTER= %d", (int)uxfio_lseek(swvarfs->fdM, 0 , UXFIO_SEEK_VCUR));
 	
-		header_offset=uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
+		header_offset = uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 
 		if (taru_read_header(
 				swvarfs->taruM,
@@ -1124,10 +1131,9 @@ swvarfs_i_readin_filechain(SWVARFS * swvarfs, char * name)
 			return NULL;
 		}
 		SWVARFS_E_DEBUG2("I-section AFTER taru_read_header name=[%s]", ahsStaticGetTarFilename(ahs_vfile_hdr(swvarfs->ahsM)));
-		swvarfs->current_data_offsetM = (int)uxfio_lseek(swvarfs->fdM,
-							0, UXFIO_SEEK_VCUR);
+		swvarfs->current_data_offsetM = uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 	
-		data_offset=uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
+		data_offset = uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 	
 		if ((ahs_vfile_hdr(swvarfs->ahsM)->c_mode & CP_IFMT) == 
 								CP_IFREG) {
@@ -1159,7 +1165,7 @@ swvarfs_i_readin_filechain(SWVARFS * swvarfs, char * name)
 	}
 		
 	do{
-		header_offset=uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
+		header_offset = uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 		
 		SWVARFS_E_DEBUG3("readin name = [%s]  header offset = %d", name, (int)header_offset);
 	
@@ -1173,8 +1179,7 @@ swvarfs_i_readin_filechain(SWVARFS * swvarfs, char * name)
 			break;
 		}
 
-		swvarfs->current_data_offsetM = (int)uxfio_lseek(swvarfs->fdM, 0,
-							 UXFIO_SEEK_VCUR);
+		swvarfs->current_data_offsetM = (int)uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 		
 		SWVARFS_E_DEBUG2("Loop-section AFTER taru_read_header name=[%s]", ahsStaticGetTarFilename(ahs_vfile_hdr(swvarfs->ahsM)));
 
@@ -1248,17 +1253,22 @@ swvarfs_i_search_filechain(SWVARFS * swvarfs, char * name_p,
 		if (strncmp(p, "./", 2) == 0) {
 			p += 2;
 		}
-		SWVARFS_E_DEBUG3("fnmatch pattern=[%s] name=[%s]", name, p); 
-		if (strncmp(name, "./", 2) == 0) {
+
+		if (strcmp(name, "./") == 0) {
+			name[1] = '\0';
+		} else if (strncmp(name, "./", 2) == 0) {
 			name += 2;
 		}
+
 		handle_trailing_slash("drop", name, &did_drop);
 		handle_leading_slash(swvarfs, "drop", name, &did_omit_lslash);
-		if (fnmatch(name, p, 0) == 0) {
+		SWVARFS_E_DEBUG3("strcmp/fnmatch pattern=[%s] name=[%s]", name, p);
+		/* if (fnmatch(name, p, 0) == 0) { */
+		if (strcmp(name, p) == 0) {
 			/*
 			* Its a match, but it may be out of scope.
 			*/
-			SWVARFS_E_DEBUG3("Got match fnmatch pattern=[%s] name=[%s]", name, p); 
+			SWVARFS_E_DEBUG3("Got match strcmp/fnmatch pattern=[%s] name=[%s]", name, p); 
 			if (is_name_in_scope(strob_str(swvarfs->dirscopeM),
 						name, (struct stat *)NULL)) {
 				handle_trailing_slash("restore", name, 
@@ -1290,8 +1300,8 @@ static int
 swvarfs_i_u_open_archive_seek_to_name(SWVARFS * swvarfs, char * name)
 {
 	int tret;
-	int header_offset = 0;
-	int data_offset;
+	intmax_t header_offset = 0;
+	intmax_t data_offset;
 	int * peoa = &(swvarfs->eoaM);
 	struct fileChain * new_link;
 	struct fileChain * existing = NULL;
@@ -1448,13 +1458,11 @@ swvarfs_i_u_open_archive_seek_to_name(SWVARFS * swvarfs, char * name)
 		return -3;
 	}
 	
-	swvarfs->current_data_offsetM = (int)uxfio_lseek(swvarfs->fdM, 
-							0, SEEK_CUR);
-	SWVARFS_E_DEBUG2("swvarfs->current_data_offsetM = [%d]", swvarfs->current_data_offsetM); 
+	swvarfs->current_data_offsetM = uxfio_lseek(swvarfs->fdM, 0, SEEK_CUR);
+	SWVARFS_E_DEBUG2("swvarfs->current_data_offsetM = [%s]", swlib_imaxtostr(swvarfs->current_data_offsetM, NULL)); 
 
 	if (swvarfs->makefilechainM && swvarfs->current_filechainM == NULL) {
-		data_offset = (int)uxfio_lseek(swvarfs->fdM, 
-						0, UXFIO_SEEK_VCUR);
+		data_offset = uxfio_lseek(swvarfs->fdM, 0, UXFIO_SEEK_VCUR);
 		new_link=swvarfs_i_make_link(ahs_vfile_hdr(swvarfs->ahsM),
 						header_offset, data_offset);
 		swvarfs_i_attach_filechain(swvarfs, new_link);
@@ -1570,7 +1578,7 @@ swvarfs_i_iopen(SWVARFS * swvarfs, int oflags)
 			swvarfs->fdM=-1;
 			return -1;
 		}
-		if (uxfio_espipe(swvarfs->fdM) == ESPIPE) {
+		if (uxfio_espipe(swvarfs->fdM)) {
 			int current_buftype;
 			/*
 			* Its a pipe, need to turn on the window buffering.
@@ -1597,9 +1605,11 @@ swvarfs_i_iopen(SWVARFS * swvarfs, int oflags)
 			* Its not a pipe, turn buffering off.
 			*/
 			swvarfs->is_unix_pipeM = 0;
+			/*
 			uxfio_fcntl(swvarfs->fdM,
 					UXFIO_F_SET_BUFACTIVE,
 					UXFIO_OFF);
+			*/
 		}
 	}	
 	return 0;
@@ -1609,6 +1619,7 @@ static
 int 
 swvarfs_i_u_close_archive(SWVARFS * swvarfs, int fd)
 {
+	int ret;
 	int u_index;
 	SWVARFS_E_DEBUG2("ENTERING [%d]", fd); 
 	SWVARFS_E_DEBUG3("u_fdM=%d  fdM=%d", swvarfs->u_fdM, swvarfs->fdM); 
@@ -1635,18 +1646,18 @@ swvarfs_i_u_close_archive(SWVARFS * swvarfs, int fd)
 		*/
 		if (swvarfs->uxfio_buftype != UXFIO_BUFTYPE_NOBUF) {
 			SWVARFS_E_DEBUG("swvarfs->uxfio_buftype != UXFIO_BUFTYPE_NOBUF"); 
-			SWVARFS_E_DEBUG2("uxfio dump %s", uxfio_dump_string(fd));
-			if (uxfio_lseek(fd, (off_t)0, SEEK_END) < 0) {
-				fprintf(stderr,
-				"swvarfs_i_u_close_archive: uxfio_lseek.\n");
+			/* SWVARFS_E_DEBUG2("uxfio dump %s", uxfio_dump_string(fd)); */
+			ret = uxfio_lseek(fd, (off_t)0, SEEK_END);
+			if (ret < 0) {
+				fprintf(stderr, "%s: swvarfs_i_u_close_archive: uxfio_lseek error\n", swlib_utilname_get());
 				return -1;
 			}
-			SWVARFS_E_DEBUG2("uxfio dump %s", uxfio_dump_string(fd));
+			/* SWVARFS_E_DEBUG2("uxfio dump %s", uxfio_dump_string(fd)); */
 			SWVARFS_E_DEBUG(""); 
 		}
 	}
 
-	SWVARFS_E_DEBUG2("uxfio dump on swvarfs->fdM %s", uxfio_dump_string(swvarfs->fdM));
+	/* SWVARFS_E_DEBUG2("uxfio dump on swvarfs->fdM %s", uxfio_dump_string(swvarfs->fdM)); */
 	if ((ahs_vfile_hdr(swvarfs->ahsM)->c_mode & CP_IFMT) == CP_IFREG) {
 		/*
 		* We assume the data has already been read by
@@ -1672,10 +1683,8 @@ swvarfs_i_u_close_archive(SWVARFS * swvarfs, int fd)
 			SWBIS_E_FAIL("uxfio_lseek failed-1");
 		}
 
-		SWVARFS_E_DEBUG3("XXX running uxfio_lseek fd=%d offset=%d", swvarfs->fdM, swvarfs->current_data_offsetM - current_pos);
-		if (uxfio_lseek(swvarfs->fdM,
-			swvarfs->current_data_offsetM - current_pos,
-					SEEK_CUR) < 0) {
+		SWVARFS_E_DEBUG3("XXX running uxfio_lseek fd=%d offset=%d", swvarfs->fdM, (int)(swvarfs->current_data_offsetM - current_pos));
+		if (uxfio_lseek(swvarfs->fdM, (off_t)(swvarfs->current_data_offsetM - current_pos), SEEK_CUR) < 0) {
 			SWBIS_E_FAIL("uxfio_lseek failed-2");
 		}
 	}
@@ -1745,10 +1754,12 @@ swvarfs_open_serial_access_file_internal(SWVARFS *swvarfs,
 				&(swvarfs->format_descM), oflags);
 	swvarfs->did_dupM=1;
 	if (swvarfs->fdM < 0) {
+		fprintf(stderr,"error (swvarfs_open_serial_access_file_internal) loc=1\n");
 		swbis_free(swvarfs);
 		return (SWVARFS*)(NULL);
 	}
 	if (swvarfs_i_iopen(swvarfs, oflags))  {
+		fprintf(stderr,"error (swvarfs_open_serial_access_file_internal) loc=2\n");
 		swbis_free(swvarfs);
 		return (SWVARFS*)(NULL);
 	}
@@ -1810,7 +1821,7 @@ swvarfs_open_archive_file(char * name, int oflags)
 {
 	SWVARFS *swvarfs=(SWVARFS*)malloc(sizeof(SWVARFS));
 	SWVARFS * ret;
-	if(!swvarfs) return (SWVARFS*)(NULL);
+	if(swvarfs == NULL) return (SWVARFS*)(NULL);
 	swvarfs_i_init(swvarfs);
 	ret = swvarfs_open_serial_access_file_internal(swvarfs, name, oflags);
 	if (ret) {
@@ -1825,7 +1836,7 @@ swvarfs_open_directory(char * name)
 {
 	SWVARFS *swvarfs=(SWVARFS*)malloc(sizeof(SWVARFS));
 	SWVARFS * ret;
-	if(!swvarfs) return (SWVARFS*)(NULL);
+	if(swvarfs == NULL) return (SWVARFS*)(NULL);
 	swvarfs_i_init(swvarfs);
 	ret = swvarfs_open_directory_internal(swvarfs, name);
 	if (ret) {
@@ -1871,9 +1882,14 @@ swvarfs_opendup(int ifd, int oflags, mode_t mode)
 	SWVARFS *swvarfs=(SWVARFS*)malloc(sizeof(SWVARFS));
 
 	SWVARFS_E_DEBUG2("ENTERING mode=%d", oflags); 
+	if (swvarfs == NULL) return NULL;
+
+	SWVARFS_E_DEBUG(""); 
 	swvarfs->fdM = uinfile_opendup(ifd, (mode_t)(0),
 				&(swvarfs->format_descM), oflags);
+	SWVARFS_E_DEBUG(""); 
 	if (swvarfs->fdM < 0) {
+		SWVARFS_E_DEBUG(""); 
 		swbis_free(swvarfs);
 		return (SWVARFS*)(NULL);
 	} else if (swvarfs->fdM == ifd) {
@@ -1882,7 +1898,9 @@ swvarfs_opendup(int ifd, int oflags, mode_t mode)
 		swvarfs->did_dupM=1;
 	}
 
+	SWVARFS_E_DEBUG(""); 
 	swvarfs_i_init(swvarfs);
+	SWVARFS_E_DEBUG(""); 
 	if (swvarfs_i_iopen(swvarfs, oflags))
 		return NULL;
 	swvarfs->formatM=uinfile_get_type(swvarfs->format_descM);
@@ -1998,7 +2016,7 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		"directory [%s] not in current scope of %s openpath=%s\n", 
 				path, strob_str(swvarfs->dirscopeM),
 					strob_str(swvarfs->openpathM));
-			return -1;
+			return -2;
 		}
 	}
 	if (swvarfs->formatM == UINFILE_FILESYSTEM) {
@@ -2027,7 +2045,7 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		if ( (dp = opendir(path)) == NULL) {
 			fprintf(stderr, "%s : %s\n",path, strerror(errno));
 			swvarfs->derrM = -1;
-			return -1;
+			return -3;
 		}
 
 		dirx.dp = dp;
@@ -2036,7 +2054,7 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		ret = 0;
 	} else {
 		int header_offset;
-		int old_pos;
+		intmax_t old_pos;
 		struct fileChain * existing = NULL;
 		struct stat st;
 		int espipe = 0;
@@ -2079,7 +2097,7 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 				/*
 				* error, cannot comply.
 				*/
-				return -1;	
+				return -4;	
 			} else {
 				/*
 				* already seeked to beggining, done.
@@ -2095,7 +2113,7 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		* Use the lstat function to search.
 		*/
 		if (swvarfs_u_lstat(swvarfs, path, &st)) {
-			return -2;
+			return -5;
 		}
 
 
@@ -2107,16 +2125,16 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		existing = swvarfs->current_filechainM;
 		if (existing == (struct fileChain*)NULL) {
 			SWBIS_E_FAIL("(struct fileChain*)existing is null. fatal.");
-			return -2;
+			return -6;
 		}
 
 		/*
 		* Get the current position.
 		*/
-		old_pos = (int)uxfio_lseek(swvarfs->fdM, 0 , UXFIO_SEEK_VCUR);
+		old_pos = (intmax_t)uxfio_lseek(swvarfs->fdM, 0 , UXFIO_SEEK_VCUR);
 		if (old_pos < 0) {
-			SWBIS_E_FAIL2("uxfio_lseek error ret = %d", old_pos);
-			return -1;
+			SWBIS_E_FAIL2("uxfio_lseek error ret = %d", (int)old_pos);
+			return -7;
 		}
 			
 		header_offset = existing->header_offsetFC;
@@ -2125,8 +2143,9 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		* Perform sanity check.
 		*/
 		if (header_offset - old_pos >= 0) {
-			SWBIS_E_FAIL3("internal error old_pos=%d  header_offset=%d", old_pos, header_offset);
-			return -4;
+			SWBIS_E_FAIL3("internal error old_pos=%s  header_offset=%s", swlib_imaxtostr(old_pos, NULL),
+										swlib_imaxtostr(header_offset, NULL));
+			return -8;
 		}
 
 		/* 
@@ -2134,7 +2153,7 @@ swvarfs_setdir(SWVARFS * swvarfs, char * path)
 		*/	
 		if (uxfio_lseek(swvarfs->fdM, header_offset - old_pos, UXFIO_SEEK_VCUR) < 0) {
 			SWBIS_E_FAIL("uxfio_lseek error");
-			return -5;
+			return -9;
 		}
 
 		/*
@@ -2182,6 +2201,7 @@ swvarfs_close(SWVARFS * swvarfs)
 		ret = uinfile_close(swvarfs->format_descM);
 	swvarfs_i_delete_filechain(swvarfs);
 	hllist_close(swvarfs->link_recordM);
+	taru_delete(swvarfs->taruM);
 	swbis_free(swvarfs);
 	return ret;
 }
@@ -2377,9 +2397,23 @@ swvarfs_stop_function(SWVARFS * swvarfs, int (*fc)(void *, char *, char *))
 	swvarfs->f_do_stop_ = fc;
 }
 
+int
+swvarfs_dirent_reset(SWVARFS * swvarfs)
+{
+	if (swvarfs->formatM==UINFILE_FILESYSTEM) {
+		;  /* nothing to do */
+	} else {
+		uxfio_lseek(swvarfs->fdM, (off_t)0, SEEK_SET);
+		swvarfs->eoaM = 0;
+	}
+	return 0;
+}
+
+
 char *
 swvarfs_get_next_dirent(SWVARFS * swvarfs, struct stat *st)
 {
+	int tartype;
 	int is_at_end;
 	char * name;
 	SWVARFS_E_DEBUG("ENTERING"); 
@@ -2393,7 +2427,12 @@ swvarfs_get_next_dirent(SWVARFS * swvarfs, struct stat *st)
 	}
 
 	if (is_at_end == 0) {
-		if (ahs_get_tar_typeflag(swvarfs->ahsM) == DIRTYPE) {
+		tartype = ahs_get_tar_typeflag(swvarfs->ahsM);
+		if (tartype < 0) {
+			fprintf(stderr, "%s: unrecognized file type [%d] for file: %s\n",
+				swlib_utilname_get(), (int)tartype, name?name:"<null>");
+			set_vcwd(swvarfs->vcwdM, name?name:"");
+		} else if (tartype == DIRTYPE) {
 			set_vcwd(swvarfs->vcwdM, name?name:"");
 		} else {
 			/*
@@ -2409,6 +2448,7 @@ swvarfs_get_next_dirent(SWVARFS * swvarfs, struct stat *st)
 		/*
 		* end of archive.
 		*/
+		name = NULL;
 		;
 	}
 	SWVARFS_E_DEBUG2("LEAVING %s", name?name:"<nil>"); 
