@@ -1,12 +1,12 @@
-/* topsf.c  --  Open an any format and translate to a PSF with the archive.
+/* topsf.c  --  Open (almost) any format and translate to a PSF and archive.
 
-   Copyright (C) 1999  Jim Lowe 
+   Copyright (C) 1999,2007  Jim Lowe 
    All rights reserved.
 
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,8 +14,10 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+#define FILENEEDDEBUG 1
+#undef FILENEEDDEBUG
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -31,6 +33,9 @@
 #include "swutilname.h"
 #include "rpmpsf.h"
 #include "um_header.h"
+#include "swgp.h"
+#include "debpsf.h"
+#include "strar.h"
 
 static int print_links(TOPSF * topsf);
 static void topsf_i_package_close(TOPSF * topsf);
@@ -39,6 +44,561 @@ static TOPSF * topsf_i_package_open(TOPSF * topsf, int oflags);
 static int topsf_rpm_find_linkname(TOPSF * topsf, char * baselink, int fd, struct new_cpio_header *file_hdr, int format);
 static int topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp);
 
+static
+int
+is_blank_line(char * s)
+{
+	while(*s && *s != ':' && *s != '\n') {
+		if (
+			isspace(*s)
+		) {
+			;
+		} else {
+			return 0;
+		}
+		s++;
+	}
+	return 1;
+}
+
+static
+int
+is_name(char * s)
+{
+	while(*s && *s != ':' && *s != '\n') {
+		if (
+			isalpha(*s) ||
+			isdigit(*s) ||
+			*s == '_' || *s == '-'
+		) {
+			;
+		} else {
+			return 0;
+		}
+		s++;
+	}
+	if (*s != ':') return 0;
+	return 1;
+}
+
+static
+int
+parse_slack_desc(STROB * desc, STROB * title, char * src)
+{
+	int n;
+	int fd;
+	STROB * tmp;
+	char * line;
+	char * s;
+	char * t;
+	
+	tmp = strob_open(32);
+	fd = swlib_open_memfd();
+	uxfio_write(fd, src, strlen(src));
+	uxfio_lseek(fd, 0, SEEK_SET);
+
+	strob_strcpy(title, "");
+	strob_strcpy(desc, "");
+
+	while ((n=swgp_read_line(fd, (STROB*)tmp, DO_NOT_APPEND)) > 0) {
+		line = strob_str(tmp);
+		s = line;
+
+		if (*s == '#') {
+			;
+		} else if (is_blank_line(s)) {
+			;
+		} else if (is_name(s)) {
+			t = strchr(s, ':');
+			if (!t) return -1;
+			t++;
+			if (*t == ' ') t++;
+			if (strob_strlen(title) == 0) {
+				strob_strcpy(title, t);		
+			} else if (is_blank_line(t)) {
+				;
+			} else {
+				strob_strcat(desc, t);		
+			}
+		} else {
+			;
+		}
+	}
+	strob_close(tmp);
+	uxfio_close(fd);
+	return 0;
+}
+
+static
+int
+parse_slack_pkg_name(char * pkgfilename, STRAR * st)
+{
+	char * n;
+	char * s;
+	STROB * tmp;
+	char * name = NULL;
+	char * version = NULL;
+	char * arch = NULL;
+	char * build_num = NULL;
+
+	/* example: gzip-1.3.12-i486-1.tgz */
+
+	tmp = strob_open(32);
+	strob_strcpy(tmp, pkgfilename);
+
+	n = strob_str(tmp);
+	s = strrchr(n, '.');
+	if (!s) return -1;
+	*s = '\0';   /* squash the ``.tgz'' */
+
+	s = strrchr(n, '/');
+	if (s) {
+		s++;
+		n = s;
+	}
+
+	s = strrchr(n, '-');
+	if (!s) return -1;
+	*s = '\0';
+	s++;
+	build_num = s;
+
+	s = strrchr(n, '-');
+	if (!s) return -1;
+	*s = '\0';
+	s++;
+	arch = s;
+
+	s = strrchr(n, '-');
+	if (!s) return -1;
+	*s = '\0';
+	s++;
+	version = s;
+
+	name = n;
+
+	strar_add(st, name);
+	strar_add(st, version);
+	strar_add(st, arch);
+	strar_add(st, build_num);
+
+	strob_close(tmp);
+	return 0;
+}
+
+
+static
+int
+parse_src_pkg_name(char * pkgfilename, STRAR * st)
+{
+	char * n;
+	char * s;
+	STROB * tmp;
+	char * name = NULL;
+	char * version = NULL;
+
+	/* example: foo-devel-1.3.12.tar.gz */
+
+	tmp = strob_open(32);
+	strob_strcpy(tmp, pkgfilename);
+
+	n = strob_str(tmp);
+	s = (
+		strstr(n, ".tgz") ||
+		strstr(n, ".tar") ||
+		strstr(n, ".tbz") ||
+		(char *)(NULL)
+		);
+	if (!s) {
+		;
+		/* return -1; */
+	} else {
+		*s = '\0';   /* squash the ``.tgz'' */
+	}
+
+	s = strrchr(n, '-');
+	if (!s) return -1;
+	*s = '\0';
+	s++;
+	version = s;
+	name = n;
+	strar_add(st, name);
+	strar_add(st, version);
+	strob_close(tmp);
+	return 0;
+}
+
+static
+int
+plain_source_tarball_write_psf_buf(TOPSF * topsf, STROB * psf)
+{
+	unsigned long int prefix_mode;
+	time_t dir_mtime;
+	int ret;
+	XFORMAT * data_xformat;
+	STROB * tmp;
+	STROB * xtmp;
+	STROB * desc;
+	STROB * title;
+	STROB * doinst;
+
+	STRAR * name_fields;
+	XFORMAT * xformat;
+	AHS * ahs;
+	struct stat st;
+	char * name;
+	char * version;
+	char * arch;
+	char * build_num;
+
+	dir_mtime = (time_t)0;
+	doinst = NULL;
+	name_fields = strar_open();
+	xtmp = strob_open(32);
+	tmp = strob_open(32);
+	desc = strob_open(32);
+	title = strob_open(32);
+
+	if (topsf->format_desc_->pathname_prefixM == NULL) {
+		return -1;
+	}
+
+	ret = parse_src_pkg_name(topsf->format_desc_->pathname_prefixM, name_fields);
+
+	if (ret < 0) {
+		fprintf(stderr, "%s: error parsing name, should be name-version{.tar|.tgz|*}: %s\n",
+			swlib_utilname_get(), topsf->format_desc_->pathname_prefixM);
+		return -2;
+	}
+
+	if (topsf->format_desc_->slackheaderM) {
+		/* this is a tar header, use it to set the <distribution>.mode
+		   attribute which set the mode of the leading path prefix. */
+		struct tar_header *tar_hdr;
+		tar_hdr = topsf->format_desc_->slackheaderM;
+		taru_otoul(tar_hdr->mode, &prefix_mode);
+		taru_otoul(tar_hdr->mtime, &dir_mtime);
+	} else {
+		prefix_mode = 0;
+	}
+
+	/*
+	fprintf(stderr, "SOURCE TARBALL: [%s][%s]\n",
+			strar_get(name_fields, 0),
+			strar_get(name_fields, 1));
+	*/
+
+	strob_sprintf(psf, STROB_NO_APPEND, "");
+	strob_sprintf(psf, STROB_DO_APPEND, "distribution\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "layout_version 1.0\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "dfiles dfiles\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "pfiles pfiles\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_control_directory " %s\n", topsf->format_desc_->pathname_prefixM);
+	if (topsf->checkdigestnameM) {
+		strob_sprintf(psf, STROB_DO_APPEND, "checkdigest <catalog/checkdigest\n");
+	}
+	if (prefix_mode)
+		strob_sprintf(psf, STROB_DO_APPEND, SW_A_mode " %04o\n",
+			prefix_mode & (S_IRWXU|S_IRWXG|S_IRWXO|S_ISUID|S_ISGID|S_ISVTX));
+	if (dir_mtime)
+		strob_sprintf(psf, STROB_DO_APPEND, SW_A_mtime " %lu\n", (unsigned long)dir_mtime);
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_owner " 0\n"); /* swbis imposed policy */
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_group " 0\n"); /* swbis imposed policy */
+
+
+	strob_sprintf(psf, STROB_DO_APPEND, "\n");
+
+	xformat = xformat_open(-1, -1, arf_ustar);
+	ret = uxfio_lseek(topsf->fd_, 0, SEEK_SET);
+	if (ret < 0) {
+		E_DEBUG("error");
+		return -2;
+	}
+	
+	ret = xformat_open_archive_by_fd(xformat, topsf->fd_,
+			UINFILE_DETECT_OTARFORCE|UINFILE_DETECT_DEB_CONTROL, (mode_t)(0));
+	if (ret < 0) {
+		E_DEBUG("error");
+		return -3;
+	}
+
+	ret = xformat_read_header(xformat);
+	if (xformat_is_end_of_archive(xformat)) {
+		ret = -5; fprintf(stderr, "%s: empty data archive [%d]\n", swlib_utilname_get(), ret);
+		E_DEBUG("error");
+		return ret;
+	}
+
+	if (xformat_get_tar_typeflag(xformat) != DIRTYPE) {
+		ret = -6; fprintf(stderr, "%s: first file not a directory  [%d]\n", swlib_utilname_get(), ret);
+		return ret;
+	}
+
+	ahs = xformat_ahs_object(xformat);
+	name = ahsStaticGetTarFilename(ahs->file_hdrM);
+
+	if (swlib_dir_compare(name, topsf->format_desc_->pathname_prefixM, 0)) {
+		ret = -7; fprintf(stderr, "%s: first file not %s %s\n",
+			swlib_utilname_get(),
+			topsf->format_desc_->pathname_prefixM, name);
+		return ret;
+	}
+
+	/* Make a temporary copy of the explicit file definition */
+	strob_sprintf(xtmp, STROB_NO_APPEND,  SW_A_file " -t d -o 0 -g 0 -m %o %s\n",
+		(unsigned int)(ahs_get_perms(ahs)), topsf->format_desc_->pathname_prefixM
+		);
+
+	E_DEBUG2("title: [%s]", strob_str(title));
+	E_DEBUG2("description: [%s]", strob_str(desc));
+
+	E_DEBUG("");
+	swlib_squash_illegal_tag_chars(strar_get(name_fields, 0));
+	strob_sprintf(psf, STROB_DO_APPEND, "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_product "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_tag " %s\n", strar_get(name_fields, 0));
+	
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_revision " %s\n", strar_get(name_fields, 1));
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_control_directory " \"\"\n");
+
+	strob_sprintf(psf, STROB_DO_APPEND, "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_fileset "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_tag " bin\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_control_directory " \"\"\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_directory " . #/\n");
+
+	/* Use an explicit definition for the "./" archive member */
+	strob_sprintf(psf, STROB_DO_APPEND, "file_permissions -o 0 -g 0\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_file " *\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " PSF\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " catalog\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " catalog/*\n");
+
+	strar_close(name_fields);
+	strob_close(xtmp);
+	strob_close(tmp);
+	strob_close(desc);
+	strob_close(title);
+	return 0;
+}
+
+static
+int
+slack_write_psf_buf(TOPSF * topsf, STROB * psf)
+{
+	int ret;
+	XFORMAT * data_xformat;
+	STROB * tmp;
+	STROB * xtmp;
+	STROB * desc;
+	STROB * title;
+	STROB * slack_desc;
+	STROB * doinst;
+
+	STRAR * name_fields;
+	XFORMAT * xformat;
+	AHS * ahs;
+	struct stat st;
+	char * name;
+	char * version;
+	char * arch;
+	char * build_num;
+
+	slack_desc = NULL;
+	doinst = NULL;
+	name_fields = strar_open();
+	xtmp = strob_open(32);
+	tmp = strob_open(32);
+	desc = strob_open(32);
+	title = strob_open(32);
+	
+	if (topsf->pkgfilenameM == NULL) {
+		fprintf(stderr, "%s: must specify a slackware package as a regular file arg\n", swlib_utilname_get());
+		E_DEBUG("error");
+		return -1;
+	}
+
+	ret = parse_slack_pkg_name(topsf->pkgfilenameM, name_fields);
+	if (ret < 0) {
+		E_DEBUG("error");
+		return -2;
+	}
+
+	/*
+	fprintf(stderr, "SLACK: [%s][%s][%s][%s]\n",
+			strar_get(name_fields, 0),
+			strar_get(name_fields, 1),
+			strar_get(name_fields, 2),
+			strar_get(name_fields, 3));
+	*/	
+
+	strob_sprintf(psf, STROB_NO_APPEND, "");
+	strob_sprintf(psf, STROB_DO_APPEND, "distribution\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "layout_version 1.0\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "dfiles dfiles\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "pfiles pfiles\n");
+
+	strob_sprintf(psf, STROB_DO_APPEND, "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "vendor\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "the_term_vendor_is_misleading True\n");
+	swlib_squash_illegal_tag_chars(strar_get(name_fields, 3));
+	strob_sprintf(psf, STROB_DO_APPEND, "tag \"%s\"\n", strar_get(name_fields, 3));
+	strob_sprintf(psf, STROB_DO_APPEND, "title The Slackware Linux Project\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "url \"http://www.slackware.com\"\n");
+
+	xformat = xformat_open(-1, -1, arf_ustar);
+	ret = uxfio_lseek(topsf->fd_, 0, SEEK_SET);
+	if (ret < 0) {
+		E_DEBUG("error");
+		return -2;
+	}
+	
+	ret = xformat_open_archive_by_fd(xformat, topsf->fd_,
+			UINFILE_DETECT_OTARFORCE|UINFILE_DETECT_DEB_CONTROL, (mode_t)(0));
+	if (ret < 0) {
+		E_DEBUG("error");
+		return -3;
+	}
+
+	ret = xformat_read_header(xformat);
+	if (xformat_is_end_of_archive(xformat)) {
+		ret = -5; fprintf(stderr, "%s: empty data archive [%d]\n", swlib_utilname_get(), ret);
+		E_DEBUG("error");
+		return ret;
+	}
+
+	if (xformat_get_tar_typeflag(xformat) != DIRTYPE) {
+		ret = -6; fprintf(stderr, "%s: first file not a directory  [%d]\n", swlib_utilname_get(), ret);
+		return ret;
+	}
+
+	ahs = xformat_ahs_object(xformat);
+	name = ahsStaticGetTarFilename(ahs->file_hdrM);
+
+	/* we expect name to be "./" */
+
+	if (strcmp(name, "./")) {
+		ret = -7; fprintf(stderr, "%s: first file not ./\n", swlib_utilname_get());
+		return ret;
+	}
+
+	/* Make a temporary copy of the explicit file definition */
+	strob_sprintf(xtmp, STROB_NO_APPEND,  SW_A_file " -t d -o %s -g %s -m %o /\n",
+		ahsStaticGetTarUsername(ahs->file_hdrM),
+		ahsStaticGetTarGroupname(ahs->file_hdrM),
+		(unsigned int)(ahs_get_perms(ahs))
+		);
+
+	/* Now audit the ./install/ directory contents */
+
+	while (
+		(name=xformat_get_next_dirent(xformat, &st)) != NULL &&
+		strstr(name, "install/") == name
+	) {
+		E_DEBUG("");
+		if (strcmp(name, "install/doinst.sh") == 0) {
+			topsf_h_write_to_buf(xformat, name, &doinst);
+		} else if (strcmp(name, "install/") == 0) {
+			;
+		} else if (strcmp(name, "install/slack-desc") == 0) {
+			topsf_h_write_to_buf(xformat, name, &slack_desc);
+		} else {
+			topsf_h_write_to_buf(xformat, name, NULL);
+			fprintf(stderr, "%s: unrecognized skackware control file: %s\n", swlib_utilname_get(), name);
+		}
+	}
+
+	if (slack_desc) {
+		/* E_DEBUG2("%s", strob_str(slack_desc)); */
+		parse_slack_desc(desc, title, strob_str(slack_desc));
+		if (strrchr(strob_str(desc), '\n')) *strrchr(strob_str(desc), '\n') = '\0';
+		if (strrchr(strob_str(desc), '\r')) *strrchr(strob_str(desc), '\r') = '\0';
+		if (strrchr(strob_str(title), '\n')) *strrchr(strob_str(title), '\n') = '\0';
+		if (strrchr(strob_str(title), '\r')) *strrchr(strob_str(title), '\r') = '\0';
+	}
+
+	E_DEBUG2("title: [%s]", strob_str(title));
+	E_DEBUG2("description: [%s]", strob_str(desc));
+
+	E_DEBUG("");
+	swlib_squash_illegal_tag_chars(strar_get(name_fields, 0));
+	strob_sprintf(psf, STROB_DO_APPEND, "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_product "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_tag " %s\n", strar_get(name_fields, 0));
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_vendor_tag " %s\n", strar_get(name_fields, 3));
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_title " \"%s\"\n", strob_str(title));
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_description " \"%s\"\n", strob_str(desc));
+	
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_revision " %s\n", strar_get(name_fields, 1));
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_control_directory " \"\"\n");
+	if (doinst)
+		strob_sprintf(psf, STROB_DO_APPEND, SW_A_postinstall " install/doinst.sh\n");
+
+	strob_sprintf(psf, STROB_DO_APPEND, "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_fileset "\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_tag " bin\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_control_directory " \"\"\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_directory " . /\n");
+
+	/* Use an explicit definition for the "./" archive member */
+	strob_sprintf(psf, STROB_DO_APPEND,  "%s", strob_str(xtmp));
+
+	/* Recursive spec after the leading ./ */
+
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_file " *\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "#" SW_A_exclude " control\n");
+	strob_sprintf(psf, STROB_DO_APPEND, "#" SW_A_exclude " control/*\n");
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " PSF\n");
+
+	strar_close(name_fields);
+	strob_close(xtmp);
+	strob_close(tmp);
+	strob_close(desc);
+	strob_close(title);
+	return 0;
+}
+
+static
+int
+slack_write_psf(TOPSF * topsf, int fd_out)
+{
+	int ret;
+	STROB * psf;
+
+	psf = strob_open(32);
+	ret = slack_write_psf_buf(topsf, psf);
+	if (ret < 0) {
+		return -1;
+	}
+	ret = uxfio_write(fd_out, strob_str(psf), strob_strlen(psf));
+	if (ret != (int)strob_strlen(psf)) {
+		ret = -1;
+	}
+	strob_close(psf);
+	return ret;
+}
+
+static
+int
+plain_source_tarball_psf(TOPSF * topsf, int fd_out)
+{
+	int ret;
+	STROB * psf;
+	
+	psf = strob_open(32);
+	/* ret = slack_write_psf_buf(topsf, psf); */
+	ret = plain_source_tarball_write_psf_buf(topsf, psf);
+	if (ret < 0) {
+		return -1;
+	}
+	ret = uxfio_write(fd_out, strob_str(psf), strob_strlen(psf));
+	if (ret != (int)strob_strlen(psf)) {
+		ret = -1;
+	}
+	strob_close(psf);
+	return ret;
+}
+
 static
 int
 find_link(TOPSF * topsf, char * keyname)
@@ -68,18 +628,23 @@ TOPSF * topsf_open(char * package_name, int oflags)
 {
 	TOPSF * topsf=(TOPSF*)malloc(sizeof(TOPSF));
 	
-	oflags |= UINFILE_DETECT_NATIVE;
+	/* oflags |= UINFILE_DETECT_NATIVE; */
 	oflags |= UINFILE_DETECT_FORCEUNIXFD;
 
+	E_DEBUG("");
 	topsf->swacfl_=swacfl_open();
-	topsf->fd_=uinfile_open(package_name, (mode_t)(NULL), &(topsf->format_desc_), oflags); 
+	E_DEBUG("");
+	topsf->fd_ = uinfile_open(package_name, (mode_t)(NULL), &(topsf->format_desc_), oflags); 
+	E_DEBUG("");
 	
 	if (topsf->fd_ < 0) {
 		fprintf(stderr, "%s: error opening package.\n", swlib_utilname_get());
 		swacfl_close(topsf->swacfl_);
 		swbis_free(topsf);
+		E_DEBUG("");
 		return NULL;
 	}
+	E_DEBUG("");
 	topsf->cwd_prefix_=(char*)(NULL);
 	topsf->prefix_=(char*)(NULL);
 	topsf->hl_node_names_ = cplob_open(1);
@@ -99,6 +664,13 @@ TOPSF * topsf_open(char * package_name, int oflags)
 	topsf->control_directoryM = strob_open(10);
 	topsf->taruM = taru_create();
 	time(&(topsf->mtimeM));
+	topsf->debpsfM = NULL;
+	topsf->rpmpsfM = NULL;
+	if (strcmp(package_name, "-") == 0)
+		topsf->pkgfilenameM = NULL;
+	else
+		topsf->pkgfilenameM = strdup(package_name);
+	topsf->checkdigestnameM = NULL;
 	return topsf_i_package_open(topsf, oflags);
 }
 
@@ -114,11 +686,26 @@ topsf_i_bail(TOPSF * topsf)
 void 
 topsf_close(TOPSF * topsf)
 {
+	switch (uinfile_get_type(topsf->format_desc_)) {
+		case PLAIN_TARBALL_SRC_FILEFORMAT:
+			break;
+		case SLACK_FILEFORMAT:
+			break;
+		case DEB_FILEFORMAT:
+			debpsf_close((DEBPSF*)(topsf->debpsfM));
+			break;
+	}
+	E_DEBUG("");
 	swacfl_close(topsf->swacfl_);
+	E_DEBUG("");
 	topsf_i_package_close(topsf);
+	E_DEBUG("");
 	uinfile_close(topsf->format_desc_);
+	E_DEBUG("");
 	uxfio_close(topsf->fd_);	
+	E_DEBUG("");
 	if(topsf->prefix_) swbis_free(topsf->prefix_);
+	E_DEBUG("");
 	swbis_free(topsf);
 }
 
@@ -131,7 +718,7 @@ topsf_i_package_open(TOPSF * topsf, int oflags) {
 	int deb_data_fd;
 	int ret;
 
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
 			topsf->rpmfd_=rpmfd_open((FD_t)NULL, topsf->fd_);	
@@ -139,7 +726,7 @@ topsf_i_package_open(TOPSF * topsf, int oflags) {
 			if (rc || !topsf->h_) {
 				fprintf(stderr, "lxpsf: error reading package in rpmReadPackageHeader().\n");
 				topsf_i_bail(topsf);
-				SWBIS_E_DEBUG("ABNORMAL FUNCTION EXIT");
+				E_DEBUG("ABNORMAL FUNCTION EXIT");
 				strob_close(tagval);
 				return (TOPSF*)(NULL);
 			}
@@ -153,63 +740,58 @@ topsf_i_package_open(TOPSF * topsf, int oflags) {
 
 			if ((oflags & TOPSF_OPEN_NO_AUDIT) == 0) {
 				if (topsf_rpm_do_audit_hard_links(topsf, topsf->fd_, oflags, strob_str(tagval))) {
-					SWBIS_E_DEBUG("ABNORMAL FUNCTION EXIT");
+					E_DEBUG("ABNORMAL FUNCTION EXIT");
 					strob_close(tagval);
 					return NULL;
 				}
 			}
 			break;
-		case DEB_FILEFORMAT:
-				/* fprintf(stderr, "lxpsf: deb format detected.\n"); */
-				topsf->rpmfd_ = NULL;
-				swlib_pipe_pump(STDOUT_FILENO, topsf->fd_);
-				ret = uxfio_lseek(topsf->format_desc_->underlying_fdM, TARU_AR_HEADER_SIZE+1, SEEK_CUR);
-				SWLIB_ASSERT(ret >= 0);
-				deb_data_fd = topsf->format_desc_->underlying_fdM;
-				swlib_pipe_pump(-1/*STDOUT_FILENO*/, deb_data_fd);
+		case PLAIN_TARBALL_SRC_FILEFORMAT:
 			break;
-				/*
-				rc = topsf_deb_read_control_archive(topsf, topsf->fd_);
-				if (topsf_deb_do_audit_hard_links(topsf, topsf->fd_, oflags)) {
-					SWBIS_E_DEBUG("ABNORMAL FUNCTION EXIT");
-					strob_close(tagval);
-					return NULL;
+		case SLACK_FILEFORMAT:
+			break;
+		case DEB_FILEFORMAT:
+			{
+				DEBPSF * d;
+				d = debpsf_create();
+				ret = debpsf_open(d, (void*)topsf);
+				if (ret < 0) {
+					fprintf(stderr, "error decoding deb format, retval=%d\n", ret);
 				}
-				*/
-				/*
-				 * the package file descriptor is now at the end of the
-				 * control.tar.gz file
-				 */
-
+				topsf->debpsfM = (void*)d;
+			}
 			break;
 		default:
 				fprintf(stderr, "lxpsf: unrecognized package format.\n");
 				topsf_i_bail(topsf);
-				SWBIS_E_DEBUG("ABNORMAL FUNCTION EXIT");
+				E_DEBUG("ABNORMAL FUNCTION EXIT");
 				strob_close(tagval);
 				return (TOPSF*)(NULL);
 			break;
 	}
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	strob_close(tagval);
 	return topsf;
 }
 
 static void
 topsf_i_package_close(TOPSF * topsf) {
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
 			headerFree(topsf->h_);
 		break;
 		case DEB_FILEFORMAT:
-		
-		break;
+			break;
+		case PLAIN_TARBALL_SRC_FILEFORMAT:
+			break;
+		case SLACK_FILEFORMAT:
+			break;
 		default:
 			fprintf(stderr, "unrecognized format in topsf_close().\n");
 		break;
 	}
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 }
 
 o__inline__
@@ -256,7 +838,7 @@ topsf_get_archive_filelist(TOPSF * topsf)
 void 
 topsf_add_fl_entry(TOPSF * topsf, char * to_name, char * from_name, int source_code)
 {
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	/*
 	* from_name is the  name in the converted archive.
 	* to_name is the name the original rpm archive.
@@ -264,7 +846,7 @@ topsf_add_fl_entry(TOPSF * topsf, char * to_name, char * from_name, int source_c
 	/* fprintf(stderr, "to_name=[%s], from_name=[%s]\n", to_name, from_name);  */
 	if (topsf->cwd_prefix_) { 
 		if (!strstr(from_name, topsf->cwd_prefix_)) {
-			SWBIS_E_DEBUG("exception");
+			E_DEBUG("exception");
 		} else {
 			/* int i = strlen(topsf->cwd_prefix_); */
 			from_name+=strlen(topsf->cwd_prefix_);
@@ -285,7 +867,7 @@ topsf_add_fl_entry(TOPSF * topsf, char * to_name, char * from_name, int source_c
 		}
 	}
 	swacfl_add(topsf->swacfl_, to_name, from_name, source_code);
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 }
 
 o__inline__
@@ -327,7 +909,7 @@ topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp)
 	SHCMD *cmd[100];	
 	UINFORMAT * uinformat;
 	
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 
 	buffertype = uinfile_decode_buftype(oflags, UXFIO_BUFTYPE_DYNAMIC_MEM);
 
@@ -346,15 +928,15 @@ topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp)
 		close(opipe[0]);
 		shcmd_set_srcfd(cmd[0], fd);
 		shcmd_set_dstfd(cmd[0], opipe[1]);
-		swlib_exec_filter(cmd, 0);
+		swlib_exec_filter(cmd, -1, NULL);
 		uxfio_close(fd);
 		_exit(0);
 	}
 	close(opipe[1]);
-	SWBIS_E_DEBUG("just before uxfio_opendup");
+	E_DEBUG("just before uxfio_opendup");
 	uxfio_fd=uxfio_opendup(opipe[0], buffertype);
-	SWBIS_E_DEBUG("just after uxfio_opendup");
-	SWBIS_E_DEBUG2("uxfio_fd\n%s", uxfio_dump_string(uxfio_fd));
+	E_DEBUG("just after uxfio_opendup");
+	/* E_DEBUG2("uxfio_fd\n%s", uxfio_dump_string(uxfio_fd)); */
 	
 	
 	/* Peek at at cpio header magic, rpm (depending on the vintage)
@@ -369,12 +951,12 @@ topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp)
 	} else if (!strncmp(buf, "070702", 6)) {
 		uinfile_set_type(uinformat, CPIO_CRC_FILEFORMAT);
 	} else {
-		SWBIS_E_DEBUG("error");
+		E_DEBUG("error");
 		return -1;
 	}
 
 	if (uxfio_lseek(uxfio_fd,(long int)(-6), SEEK_CUR) < 0) {
-		SWBIS_E_DEBUG("uxfio_lseek error");
+		E_DEBUG("uxfio_lseek error");
 		return -1;
 	}
 	
@@ -387,12 +969,12 @@ topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp)
 		SWBIS_E_FAIL("error from uxfio_lseek");
 		return -1;
 	}
-	SWBIS_E_DEBUG2("closing fd=[%d]", fd);
-	SWBIS_E_DEBUG2("fd\n%s", uxfio_dump_string(fd));
+	E_DEBUG2("closing fd=[%d]", fd);
+	/* E_DEBUG2("fd\n%s", uxfio_dump_string(fd)); */
 	uxfio_close(fd);
-	SWBIS_E_DEBUG2("setting topsf fd to [%d]", uxfio_fd);
+	E_DEBUG2("setting topsf fd to [%d]", uxfio_fd);
 	topsf_set_fd(topsf, uxfio_fd);
-	SWBIS_E_DEBUG2("topsf fd\n%s", uxfio_dump_string(topsf_get_fd(topsf)));
+	/* E_DEBUG2("topsf fd\n%s", uxfio_dump_string(topsf_get_fd(topsf))); */
 	uinfile_set_type(uinformat, RPMRHS_FILEFORMAT);
 	if (topsf->debug_link_) print_links(topsf);
 	return 0;	
@@ -411,7 +993,6 @@ add_link_pair(TOPSF *topsf, char * base_link, char * link) {
 	cplob_additem(topsf->hl_linkto_names_, cplob_get_nused(topsf->hl_linkto_names_), NULL);
 }
 
-
 int
 topsf_rpm_audit_hard_links(TOPSF * topsf, int fd)
 {
@@ -419,7 +1000,7 @@ topsf_rpm_audit_hard_links(TOPSF * topsf, int fd)
 	struct new_cpio_header *file_hdr=ahsStaticCreateFilehdr();
 	int format=uinfile_get_type(topsf->format_desc_);
 
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	while(1) {
 		if (taru_read_header(topsf->taruM, file_hdr, fd, format, NULL, 0) < 0){
 			fprintf(stderr,"header error %s\n", ahsStaticGetTarFilename(file_hdr));
@@ -502,7 +1083,7 @@ topsf_rpm_find_linkname(TOPSF * topsf, char * base_link, int fd, struct new_cpio
 	struct new_cpio_header *file_hdr1=ahsStaticCreateFilehdr();
 	int oldoffset = uxfio_lseek(fd, 0L, SEEK_CUR);
 
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	while(1) {
 		if (taru_read_header(topsf->taruM, file_hdr1, fd, format, NULL, 0) < 0){
 			fprintf(stderr,"header error %s\n", ahsStaticGetTarFilename(file_hdr1));
@@ -531,7 +1112,7 @@ topsf_rpm_find_linkname(TOPSF * topsf, char * base_link, int fd, struct new_cpio
 	}
 	ahsStaticSetTarLinkname(base_file_hdr, lastmatch);
 	ahsStaticDeleteFilehdr(file_hdr1);
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	if (uxfio_lseek(fd, oldoffset, SEEK_SET) != oldoffset) {
 		fprintf(stderr, "error seeking back in topsf_rpm_find_linkname\n");
 		exit(2);
@@ -567,7 +1148,7 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 			case SWACFL_SRCCODE_RPMHEADER:
 			case SWACFL_SRCCODE_PSF:
 
-				SWBIS_E_DEBUG("in SWACFL_SRCCODE_PSF");
+				E_DEBUG("in SWACFL_SRCCODE_PSF");
 				file_hdr->c_magic=CPIO_NEWASCII_MAGIC;	
 				file_hdr->c_ino=1;
 				file_hdr->c_mode = (0644);
@@ -585,17 +1166,17 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 
 				switch(en->source_code_){
 					case SWACFL_SRCCODE_RPMHEADER:
-						SWBIS_E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
+						E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
 						topsf->user_addr=p;
 						file_hdr->c_filesize=rpmpsf_get_rpmtag_length(topsf, p) + 1;
 						do_write_file=rpmpsf_write_out_tag;
-						SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
+						E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
 						break;
 					case SWACFL_SRCCODE_PSF:
-						SWBIS_E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
+						E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
 						file_hdr->c_filesize=rpmpsf_get_psf_size(topsf);
 						do_write_file=rpmpsf_write_beautify_psf;
-						SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
+						E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
 						break;	
 					default:
 						SWBIS_E_FAIL("fatal");
@@ -610,15 +1191,15 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 					close(pheader[0]);
 					(*do_write_file)(topsf, pheader[1]);
 					if (en->source_code_ == SWACFL_SRCCODE_RPMHEADER) {
-						write(pheader[1], "\n", 1);
+						uxfio_unix_safe_write(pheader[1], "\n", 1);
 					}
 					_exit(0);
 				}
 				close(pheader[1]);
 				
 				xformat_write_by_fd(xxformat, pheader[0], file_hdr);
-				SWBIS_E_DEBUG3("OK %s %s", p, en->from_name_);
-				SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_PSF");
+				E_DEBUG3("OK %s %s", p, en->from_name_);
+				E_DEBUG("leaving SWACFL_SRCCODE_PSF");
 				break;
 			
 			case SWACFL_SRCCODE_FILESYSTEM:
@@ -627,7 +1208,7 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 		}
 	}
 	ahsStaticDeleteFilehdr(file_hdr);
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	return 0;
 }
 
@@ -659,39 +1240,40 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 		savep = swlib_strdup(strob_str(en->archiveNameM));
 		p = savep;
 		swlib_process_hex_escapes(p);
+		swlib_unexpand_escapes(NULL, p);
 		if (*p == '/') p++;
 		switch(en->source_code_){
 			case SWACFL_SRCCODE_ARCHIVE_STREAM:	
-				SWBIS_E_DEBUG("in SWACFL_SRCCODE_ARCHIVE_STREAM");
-				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1a");
+				E_DEBUG("in SWACFL_SRCCODE_ARCHIVE_STREAM");
+				E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1a");
 				u_fd=swvarfs_u_open(swvarfs, p);
-				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1b");
+				E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1b");
 				if (u_fd < 0) {	
-					SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1c");
+					E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1c");
 					fprintf(stderr,"swvarfs_u_open error: file %s not found\n", p);
 					break;
 				}
-				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1");
+				E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1");
 				swvarfs_u_fstat(swvarfs, u_fd, &st);	
-				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #2");
+				E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #2");
 				taru_statbuf2filehdr(file_hdr, &st,
 						(char*)NULL, p/*en->from_name_*/,
 						swvarfs_u_get_linkname(swvarfs, u_fd));
-				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #3");
+				E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #3");
 				file_hdr->c_uid=getuid();
 				file_hdr->c_gid=getgid();
 				xformat_write_by_fd(xxformat, u_fd, file_hdr);
-				SWBIS_E_DEBUG3("OK %s %s", p, en->from_name_);
+				E_DEBUG3("OK %s %s", p, en->from_name_);
 				ahsStaticSetTarFilename(file_hdr, NULL);
 				ahsStaticSetTarLinkname(file_hdr, NULL);
 				swvarfs_u_close(swvarfs, u_fd);
-				SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_ARCHIVE_STREAM");
+				E_DEBUG("leaving SWACFL_SRCCODE_ARCHIVE_STREAM");
 				break;
 
 			case SWACFL_SRCCODE_RPMHEADER:
 			case SWACFL_SRCCODE_PSF:
 
-				SWBIS_E_DEBUG("in SWACFL_SRCCODE_PSF");
+				E_DEBUG("in SWACFL_SRCCODE_PSF");
 				file_hdr->c_magic=CPIO_NEWASCII_MAGIC;	
 				file_hdr->c_ino=1;
 				file_hdr->c_mode = (0644);
@@ -709,17 +1291,17 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 
 				switch(en->source_code_){
 					case SWACFL_SRCCODE_RPMHEADER:
-						SWBIS_E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
+						E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
 						topsf->user_addr=p;
 						file_hdr->c_filesize = rpmpsf_get_rpmtag_length(topsf, p);
 						do_write_file = rpmpsf_write_out_tag;
-						SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
+						E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_RPMHEADER");
 						break;
 					case SWACFL_SRCCODE_PSF:
-						SWBIS_E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
+						E_DEBUG("in SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
 						file_hdr->c_filesize=rpmpsf_get_psf_size(topsf);
 						do_write_file=rpmpsf_write_beautify_psf;
-						SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
+						E_DEBUG("leaving SWACFL_SRCCODE_PSF: SWACFL_SRCCODE_PSF");
 						break;	
 					default:
 						SWBIS_E_FAIL("fatal");
@@ -738,8 +1320,8 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 				close(pheader[1]);
 				
 				xformat_write_by_fd(xxformat, pheader[0], file_hdr);
-				SWBIS_E_DEBUG3("OK %s %s", p, en->from_name_);
-				SWBIS_E_DEBUG("leaving SWACFL_SRCCODE_PSF");
+				E_DEBUG3("OK %s %s", p, en->from_name_);
+				E_DEBUG("leaving SWACFL_SRCCODE_PSF");
 				break;
 			
 			case SWACFL_SRCCODE_FILESYSTEM:
@@ -749,7 +1331,7 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 	}
 	if (savep) free(savep);
 	ahsStaticDeleteFilehdr(file_hdr);
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	return 0;
 }
 
@@ -761,7 +1343,7 @@ topsf_copypass_swacfl_list(TOPSF * topsf, int ofd)
 	XFORMAT * xxformat;
 	xxformat = xformat_open(-1, ofd, (int)(arf_newascii));
 	swvarfs=swvarfs_opendup(topsf->fd_, 
-		UINFILE_DETECT_FORCEUXFIOFD | UINFILE_DETECT_NATIVE | UINFILE_UXFIO_BUFTYPE_DYNAMIC_MEM, (mode_t)(NULL));
+		UINFILE_DETECT_FORCEUXFIOFD | /*UINFILE_DETECT_NATIVE|*/ UINFILE_UXFIO_BUFTYPE_DYNAMIC_MEM, (mode_t)(NULL));
 	if (!swvarfs) {
 		return -1;
 	}
@@ -781,14 +1363,14 @@ topsf_copypass_swacfl_list(TOPSF * topsf, int ofd)
 int
 topsf_get_fd_fd(TOPSF * topsf) {
 	int ret;
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
 			ret= rpmfd_get_fd_fd(topsf->rpmfd_);
-			SWBIS_E_DEBUG("FUNCTION EXIT");
+			E_DEBUG("FUNCTION EXIT");
 			return ret;
 		default:
-			SWBIS_E_DEBUG("FUNCTION EXIT");
+			E_DEBUG("FUNCTION EXIT");
 			return topsf->fd_;
 			break;
 	}
@@ -799,13 +1381,17 @@ topsf_make_package_prefix(TOPSF * topsf, char * cwdir)
 {
 	char * ret;
 	
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
 			ret = rpmpsf_make_package_prefix(topsf, cwdir);
-			SWBIS_E_DEBUG("FUNCTION EXIT");
+			E_DEBUG("FUNCTION EXIT");
 			return ret;
 			break;
+		case PLAIN_TARBALL_SRC_FILEFORMAT:
+			break;
+		case SLACK_FILEFORMAT:
+			break;
 		case DEB_FILEFORMAT:
 			ret = strdup("");
 			return ret;
@@ -814,7 +1400,7 @@ topsf_make_package_prefix(TOPSF * topsf, char * cwdir)
 			fprintf(stderr, "unrecognized package type.\n");
 			break;
 	}
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	return (char*)(NULL);
 }
 
@@ -823,26 +1409,41 @@ topsf_write_psf(TOPSF * topsf, int fd_out, int do_indent)
 {
 	int ret;
 
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
+			E_DEBUG("");
 			if(do_indent) {
 				ret = rpmpsf_write_beautify_psf(topsf, fd_out);
-				SWBIS_E_DEBUG("FUNCTION EXIT");
+				E_DEBUG("FUNCTION EXIT");
 				return ret;
 			} else {
 				ret = rpmpsf_write_psf(topsf, fd_out);
-				SWBIS_E_DEBUG("FUNCTION EXIT");
+				E_DEBUG("FUNCTION EXIT");
 				return ret;
 			}
 			break;
 		case DEB_FILEFORMAT:
+			E_DEBUG("");
+			ret = debpsf_write_psf((void *)topsf, fd_out);
+			E_DEBUG("");
+			return ret;
+			break;
+		case SLACK_FILEFORMAT:
+			E_DEBUG("");
+			ret = slack_write_psf(topsf, fd_out);
+			return ret;
+			break;
+		case PLAIN_TARBALL_SRC_FILEFORMAT:
+			ret = plain_source_tarball_psf(topsf, fd_out);
+			return ret;
 			break;
 		default:
+			E_DEBUG("");
 			fprintf(stderr, "unrecognized package type.\n");
 			break;
 	}
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	return -1;
 }
 
@@ -851,16 +1452,16 @@ topsf_write_info(TOPSF * topsf, int fd_out, int do_indent)
 {
 	int ret;
 
-	SWBIS_E_DEBUG("FUNCTION ENTER");
+	E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
 			if(do_indent) {
 				ret = rpmpsf_write_beautify_info(topsf, fd_out);
-				SWBIS_E_DEBUG("FUNCTION EXIT");
+				E_DEBUG("FUNCTION EXIT");
 				return ret;
 			} else {
 				ret = rpmpsf_write_info(topsf, fd_out);
-				SWBIS_E_DEBUG("FUNCTION EXIT");
+				E_DEBUG("FUNCTION EXIT");
 				return ret;
 			}
 			break;
@@ -868,6 +1469,28 @@ topsf_write_info(TOPSF * topsf, int fd_out, int do_indent)
 			fprintf(stderr, "unrecognized package type.\n");
 			break;
 	}
-	SWBIS_E_DEBUG("FUNCTION EXIT");
+	E_DEBUG("FUNCTION EXIT");
 	return -1;
 }
+
+int
+topsf_h_write_to_buf(XFORMAT * xformat, char * name, STROB ** buf)
+{
+	int ufd;
+	int ret;
+	if (buf) *buf = strob_open(32);
+
+	ufd = xformat_u_open_file(xformat, name);
+	if (ufd < 0) return -1;
+	if (buf) {
+		ret = swlib_ascii_text_fd_to_buf(*buf, ufd);
+	} else {
+		/* throw this file away */
+		STROB * tmp = strob_open(32);
+		ret = swlib_ascii_text_fd_to_buf(tmp, ufd);
+		strob_close(tmp);
+		ret = 0;
+	}
+	xformat_u_close_file(xformat, ufd);
+	return ret;
+}
