@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,13 +17,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
 #ifndef SWPARSE_H_02dec
 #define SWPARSE_H_02dec
 
+#include "swuser_config.h"
+
 #define SWPARSE_ACMD_COPY 0
 #define SWPARSE_ACMD_CAT 1
 #define SWPARSE_ACMD_EMIT 2
@@ -73,7 +75,10 @@ int swparse_write_attribute_att (int outputfd, char * key, char * val,
 				int level, int offset);
 int swparse_write_attribute_obj (int outputfd, char * key, int level,
 					int offset);
-int swparse__is_escapable (char c);
+/* int swparse__is_escapable (char c); */
+int swparse_print_filename_by_fd (char * buf, int buflen, int fd, char * filetype,
+				char * ws_level_string, char * lex_filename,
+				int form_flag);
 int swparse_print_filename (char * buf, int buflen, char * filetype,
 				char * ws_level_string, char * lex_filename,
 				int form_flag);
@@ -81,4 +86,5 @@ int sw_yyparse (int uxfio_ifd, int  uxfio_ofd, char * metadata_filename,
 					int atlevel, int mark_up_flag);
 int swparse_is_psf_ignore_attribute(char * line);
 int swparse_ignore_attribute(int filetype, int location, char * line);
+int utf8_valid(const unsigned char *buf, unsigned int len);
 #endif
