@@ -1,12 +1,12 @@
-/* swi.c -- Posix package decoding. 
+/* swi.c -- POSIX package decoding. 
 
-  Copyright (C) 2004,2005,2006 Jim Lowe
+  Copyright (C) 2004,2005,2006,2007 Jim Lowe
   All Rights Reserved.
   
   COPYING TERMS AND CONDITIONS:
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
+  the Free Software Foundation; either version 3, or (at your option)
   any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -38,6 +38,9 @@
 /* #include "swevents_array.h" */
 
 #include "debug_config.h"
+#define SWSWINEEDDEBUG 1
+#undef SWSWINEEDDEBUG 
+
 #ifdef SWSWINEEDDEBUG
 #define SWSWI_E_DEBUG(format) SWBISERROR("SWI DEBUG: ", format)
 #define SWSWI_E_DEBUG2(format, arg) SWBISERROR2("SWI DEBUG: ", format, arg)
@@ -315,6 +318,9 @@ swi_package_create(void)
 	s->target_pathM = (char*)NULL;
 	s->target_hostM = (char*)NULL;
 	s->installed_software_catalogM = (char*)NULL;
+	s->locationM = (char*)NULL;
+	s->qualifierM = (char*)NULL;
+	s->is_minimal_layoutM = 0;
 	return s;
 }
 
@@ -344,12 +350,13 @@ SWI * swi_create(void)
 	memset(swi->tarbufM, '\0', sizeof(swi->tarbufM));
 	swi->distdataM = NULL;
 	swi->sh_dash_sM = (char*)NULL;
-	swi->dash_rM = 0;
+	swi->opt_alt_catalog_rootM = 0;
 	swi->swicolM = swicol_create();
 	swi->exported_catalog_prefixM = NULL;
 	swi->verboseM = 0;
 	swi->debug_eventsM = 0;
-	swi->version_idM = swverid_open("HOST", NULL);
+	swi->target_version_idM = swverid_open("HOST", NULL);
+	swi->does_have_payloadM = -1; /* unset */
 	return swi;
 }
 
@@ -396,6 +403,7 @@ swi_product_create(SWHEADER * global_index_header, SWPATH_EX * current)
 	swiInitListOfObjects((void**)(s->swi_coM));
 	s->pfilesM = NULL;
 	s->is_selectedM = 1;
+	s->is_compatibleM = 0;
 	return s;
 }
 
@@ -803,7 +811,7 @@ swi_decode_catalog(SWI * swi)
 	int ret;
 	int handle_ret;
 	int swpath_ret;
-	int retval = -99;
+	int retval;
 	int is_catalog;
 	char * name;
 	int header_bytes;
@@ -815,6 +823,8 @@ swi_decode_catalog(SWI * swi)
 	int catalog_start_offset = -1;
 	int catalog_end_offset = 0;
 
+	retval = 0;
+
 	nullfd = swi->nullfdM;
 	if (nullfd < 0) return -22;		
 
@@ -939,33 +949,13 @@ swi_decode_catalog(SWI * swi)
 			/*
 			 * Good.
 			 * Finished, read past catalog section
+			 * This is the first file of the storage section.
 			 */
 			SWSWI_E_DEBUG("");
-			if (catalog_start_offset < 0) {
-				catalog_start_offset = 0;
-				fprintf(stderr, "internal error: %s:%d\n",
-					__FILE__, __LINE__);
-			}
-			swi->swi_pkgM->catalog_lengthM = 
-				catalog_end_offset - catalog_start_offset; 
-	
-			/*
-			 * Now fix-up the control scripts that share
-			 * a common control file.
-			 */
-			ret = swi_expand_shared_file_control_sripts(swi, swi->swi_pkgM->swi_coM[0]->pfilesM);
-			swi_com_assert_value(ret == 0, __FILE__, __LINE__);
-
-			{
-				/*
-				 * Test area
-				 */
-
-			}
-			retval = 0;
-			SWSWI_E_DEBUG("");
+			swi->does_have_payloadM = 1;
 			break;
 		}
+
 		SWSWI_E_DEBUG("");
 		oldoffset = offset;
 		name = xformat_get_next_dirent(xformat, &st);
@@ -973,6 +963,32 @@ swi_decode_catalog(SWI * swi)
 		SWSWI_E_DEBUG("");
 	}
 
+	if (retval < 0) {
+		return retval;
+	}
+	
+	if (swi->does_have_payloadM < 0 /* unset */) {
+		/* Must not have a payload such as the
+		   catalog.tar file from the installed_software_catalog
+		   or a package with no files */
+		swi->does_have_payloadM = 0;
+	}
+
+	/* sanity check */
+	if (catalog_start_offset < 0) {
+		catalog_start_offset = 0;
+		fprintf(stderr, "internal error: %s:%d\n",
+			__FILE__, __LINE__);
+	}
+
+	swi->swi_pkgM->catalog_lengthM = 
+		catalog_end_offset - catalog_start_offset; 
+
+	/* Now fix-up the control scripts that share
+	   a common control file.  */
+	ret = swi_expand_shared_file_control_sripts(swi, swi->swi_pkgM->swi_coM[0]->pfilesM);
+	swi_com_assert_value(ret == 0, __FILE__, __LINE__);
+
 	SWSWI_E_DEBUG("");
 
 	swi_recurse_swi_tree(swi, (char *)(NULL), swi_vbase_set_verbose_level, (void*)(&(swi->verboseM)));
@@ -1095,36 +1111,36 @@ swi_get_uts_attributes_from_current(SWI * swi, SWUTS * uts, SWHEADER * swheader)
 
 	value = swheader_get_single_attribute_value(swheader, SW_A_os_name);
 	if (value) {
-		swuts_store_value(uts, uts->sysnameM, value);
+		swuts_add_attribute(uts, SW_A_os_name, value);
 	} else {
-		swuts_store_value(uts, uts->sysnameM, "*");
+		swuts_add_attribute(uts, SW_A_os_name, "");
 	}	
 
 	value = swheader_get_single_attribute_value(swheader, SW_A_os_version);
 	if (value) {
-		swuts_store_value(uts, uts->versionM, value);
+		swuts_add_attribute(uts, SW_A_os_version, value);
 	} else {
-		swuts_store_value(uts, uts->versionM, "*");
+		swuts_add_attribute(uts, SW_A_os_version, "");
 	}	
 
 	value = swheader_get_single_attribute_value(swheader, SW_A_os_release);
 	if (value) {
-		swuts_store_value(uts, uts->releaseM, value);
+		swuts_add_attribute(uts, SW_A_os_release, value);
 	} else {
-		swuts_store_value(uts, uts->releaseM, "*");
+		swuts_add_attribute(uts, SW_A_os_release, "");
 	}	
 
 	value = swheader_get_single_attribute_value(swheader, SW_A_machine_type);
 	if (value) {
-		swuts_store_value(uts, uts->machineM, value);
+		swuts_add_attribute(uts, SW_A_machine_type, value);
 	} else {
-		swuts_store_value(uts, uts->machineM, "*");
+		swuts_add_attribute(uts, SW_A_machine_type, "");
 	}	
 	return;
 }
 
 int
-swi_do_decode(SWI * swi, SWUTIL * swutil,
+swi_do_decode(SWI * swi, SWLOG * swutil,
 	int target_fd1,
 	int source_fd0,
 	char * target_path,
@@ -1150,6 +1166,7 @@ swi_do_decode(SWI * swi, SWUTIL * swutil,
 	int format;
 	int flags;
 
+	SWSWI_E_DEBUG("");
 	SWLIB_ASSERT(source_fd0 >= 0);
 	swlib_doif_writef(verboseG, SWC_VERBOSE_IDB,
 					g_logspec, swutil->swu_efdM, 
@@ -1164,6 +1181,7 @@ swi_do_decode(SWI * swi, SWUTIL * swutil,
 	   forced up a level so the top level utility can
 	   control the policy */
 
+	SWSWI_E_DEBUG("");
 	if (uinfile_open_flags < 0) {
 		flags = UINFILE_DETECT_FORCEUXFIOFD |
 			UINFILE_DETECT_UNCPIO |
@@ -1174,6 +1192,7 @@ swi_do_decode(SWI * swi, SWUTIL * swutil,
 	
 	/* Now, open the package */
 
+	SWSWI_E_DEBUG("");
 	xformat = swutil_setup_xformat(swutil,
 			xformat,
 			source_fd0,
@@ -1185,8 +1204,10 @@ swi_do_decode(SWI * swi, SWUTIL * swutil,
 			flags);
 
 	if (xformat == (XFORMAT*)NULL) {
+		SWSWI_E_DEBUG("error");
 		return -1;
 	}
+	SWSWI_E_DEBUG("");
 	
 	/* The package is now opened and partially decoded */
 
@@ -1268,6 +1289,8 @@ swi_do_decode(SWI * swi, SWUTIL * swutil,
 		/*
 		* error, swi_decode_catalog failed
 		*/
+		swlib_doif_writef(verboseG, SWC_VERBOSE_1, g_logspec, swutil->swu_efdM, 
+				"swi_decode_catalog: return code %d\n", ret);
 		retval = -1;
 		swi = (SWI*)(NULL);
 	} else {
@@ -1275,3 +1298,407 @@ swi_do_decode(SWI * swi, SWUTIL * swutil,
 	}
 	return retval;
 }
+
+int
+swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int *pn_products)
+{
+	SWI_PRODUCT * product;
+	SWHEADER * global_index;
+	SWUTS * product_uts;
+	int product_number;
+	int ret;
+	int n_matches;
+
+	if (pn_products) *pn_products = 0;
+	n_matches = 0;
+
+	/* Get the global INDEX access header object */
+	global_index = swi_get_global_index_header(swi);
+	swheader_store_state(global_index, NULL);
+
+	/* Loop over the products */
+	product_number = 0;
+	while ((product=swi_package_get_product(swi->swi_pkgM, product_number++))) {
+
+		/* Now set the offset of the relevant part of the global INDEX file
+		   which is the start of the product definition */
+		if (pn_products) ++(*pn_products);
+		swheader_reset(global_index);
+		swheader_set_current_offset(global_index, product->baseM.header_indexM);
+
+		product_uts = swuts_create();
+		swi_get_uts_attributes_from_current(swi, product_uts, global_index);
+		ret = swuts_compare(target_uts, product_uts, 0 /*verbose*/);
+		if (ret == 0) {
+			n_matches++;
+			product->is_compatibleM = SW_TRUE;
+		}
+		swuts_delete(product_uts);
+	}
+	swheader_restore_state(global_index, NULL);
+	return n_matches;
+}
+
+SWHEADER *
+swi_get_global_index_header(SWI * swi)
+{
+	SWHEADER * ret;
+	ret = swi->swi_pkgM->baseM.global_headerM;
+	return ret;
+}
+
+SWHEADER *
+swi_get_fileset_info_header(SWI * swi, int product_index, int fileset_index)
+{
+	SWHEADER * ret;
+	ret = swi->swi_pkgM->swi_coM[product_index]->swi_coM[fileset_index]->info_headerM;
+	return ret;
+}
+
+SWI_FILELIST *
+swi_make_fileset_file_list(SWI * swi, int product_num, int fileset_num, char * location)
+{
+	int ret;
+	char * object_line;
+	char * object_keyword;
+	char * directory;
+	char * path;
+	char * prefix;
+	char * store;
+	char * filetype;
+	char * is_volatile;
+	int tar_filetype;
+	STROB * tmp;
+	STROB * relocated_path;
+	SWI_FILELIST * flist;
+	SWI_PRODUCT * product;
+	SWHEADER * infoheader;
+	SWHEADER * global_index;
+	SWHEADER_STATE state;
+
+	SWSWI_E_DEBUG("");
+	flist = swi_fl_create();
+	tmp = strob_open(12);
+	relocated_path = strob_open(12);
+
+	if (swi == NULL) {
+		SWLIB_ERROR("");
+		return NULL;
+	}
+
+	SWSWI_E_DEBUG("");
+
+	/* Get the product */
+	product = swi_package_get_product(swi->swi_pkgM, product_num);
+	if (product == NULL) {
+		SWLIB_ERROR("");
+		return NULL;
+	}
+
+	SWSWI_E_DEBUG("");
+
+	/* Get the INFO file for the fileset */
+	infoheader = swi_get_fileset_info_header(swi, product_num, fileset_num);
+	if (infoheader == NULL) {
+		SWLIB_ERROR("");
+		return NULL;
+	}
+
+	global_index = swi_get_global_index_header(swi);
+	swheader_set_current_offset(global_index, product->baseM.header_indexM);
+	directory = swheader_get_single_attribute_value(global_index, SW_A_directory);
+	directory = swlib_attribute_check_default(SW_A_product, SW_A_directory, directory);
+	
+	SWSWI_E_DEBUG("");
+	swheader_store_state(infoheader, &state);
+	swheader_reset(infoheader);
+
+	SWSWI_E_DEBUG("");
+	while ((object_line = swheader_get_next_object(infoheader, (int)(UCHAR_MAX), (int)UCHAR_MAX)) != NULL) {
+		SWSWI_E_DEBUG("");
+		if (swheaderline_get_type(object_line) != SWPARSE_MD_TYPE_OBJ) {
+			SWLIB_ERROR("");
+			return NULL;
+		}
+		object_keyword = swheaderline_get_keyword(object_line);		
+		if (strcmp(object_keyword, SW_A_control_file) == 0) {
+			continue;
+		}
+		path = swheader_get_single_attribute_value(infoheader, SW_A_path);
+		if (path == NULL || strlen(path) == 0) {
+			SWLIB_ERROR("unexpected null path");
+		}
+		/* need to check if its a volatile file */
+		SWSWI_E_DEBUG2("path=[%s]", path);
+
+
+		/* apply relocation */
+		if (strcmp(location, "/") != 0) {
+			SWSWI_E_DEBUG2("path=[%s]", path);
+			swlib_apply_location(relocated_path, path, location, directory);
+			SWSWI_E_DEBUG2("path=[%s]", path);
+			path =  strob_str(relocated_path);
+			SWSWI_E_DEBUG2("path=[%s]", path);
+		}
+		/* SWLIB_ERROR2("path=[%s]", path); */
+
+		/*
+		 * Impose the every pathname begins with ''./''
+		 */
+		if (*path == '.' && *(path+1) == '/') {
+			prefix = "";
+		} else {
+			SWSWI_E_DEBUG2("path=[%s]", path);
+			path = swlib_return_relative_path(path);
+			SWSWI_E_DEBUG2("path=[%s]", path);
+			prefix = "./";
+		}
+		SWSWI_E_DEBUG2("path=[%s]", path);
+		store = swi_fl_add_store(flist, strlen(path)+strlen(prefix));
+		swi_fl_set_path(flist, store, prefix, path);
+		SWSWI_E_DEBUG2("path=[%s]", path);
+		
+		/* Set the file type */
+		filetype = swheader_get_single_attribute_value(infoheader, SW_A_type);
+		if (filetype == NULL || strlen(filetype) == 0) {
+			filetype = "0"; /* REGTYPE */
+			SWLIB_ERROR("unexpected error, type attribute not present");
+		}
+		tar_filetype = swheader_getTarTypeFromTypeAttribute(*filetype);
+		swi_fl_set_type(flist, store, tar_filetype);
+
+		/* Set the volatile flag */
+		is_volatile = swheader_get_single_attribute_value(infoheader, SW_A_is_volatile);
+		ret = swlib_is_option_true(is_volatile);
+		swi_fl_set_is_volatile(flist, store, -1, ret);
+	}
+	SWSWI_E_DEBUG("");
+out:
+	swheader_restore_state(infoheader, &state);
+	strob_close(tmp);
+	strob_close(relocated_path);
+	return flist;
+}
+
+
+SWI_FILELIST *
+swi_make_file_list(SWI * swi, SWUTS * swuts, char * location, int disallow_incompatible)
+{
+	int n_compat;
+	int check_found;
+	int product_number;
+	int ret;
+	char * object_line;
+	char * object_keyword;
+	char * directory;
+	char * path;
+	char * prefix;
+	char * store;
+	char * filetype;
+	char * is_volatile;
+	int tar_filetype;
+	int n_products;
+	STROB * tmp;
+	STROB * relocated_path;
+	SWI_FILELIST * flist;
+	SWI_PRODUCT * product;
+	SWHEADER * infoheader;
+	SWHEADER * global_index;
+	SWHEADER_STATE state;
+
+	SWSWI_E_DEBUG("");
+	flist = swi_fl_create();
+	tmp = strob_open(12);
+	relocated_path = strob_open(12);
+
+	if (swi == NULL) {
+		SWLIB_ERROR("");
+		return NULL;
+	}
+
+	SWSWI_E_DEBUG("");
+	/*  swi_find_compatible_product() marks product->is_compatibleM = SW_TRUE */
+	n_compat = swi_find_compatible_product(swi, swuts, &n_products);
+
+	/* Now check for non-root "/" target and for packages that only have
+	   one product, claim it is compatible */
+
+	if (
+		strcmp(swi->swi_pkgM->target_pathM, SW_ROOT_DIR) != 0 &&
+		n_products == 1 &&
+		n_compat == 0 &&
+		disallow_incompatible == 0 &&
+		1
+	) {
+		/* this is useful for using swremove on an alernate root */
+		product = swi_package_get_product(swi->swi_pkgM, SWI_PRODUCT_0);
+		product->is_compatibleM = SW_TRUE;
+		swutil_doif_writef(swi->verboseM, SWC_VERBOSE_1, NULL, STDERR_FILENO,
+			"Warning: no compatible found, continuing with first and only product\n");
+		n_compat = 1;
+	}
+
+	if (n_compat  <= 0) {
+		SWLIB_INTERNAL("no compatible product found in entry");
+		return NULL;
+	} else if (n_compat  > 1) {
+		SWLIB_WARN("more than one compatible product found");
+		;
+	}
+	SWSWI_E_DEBUG("");
+
+	check_found = 0;
+	product_number = 0;
+	while ((product=swi_package_get_product(swi->swi_pkgM, product_number))) {
+		if (product->is_compatibleM == SW_TRUE) {
+			check_found = 1;
+			break;
+		}
+		product_number++;
+	}
+
+	if (check_found != 1) {
+		SWLIB_ERROR("");
+		return NULL;
+	}
+	SWSWI_E_DEBUG("");
+
+	/* Ok, if we've made it this far the product given by
+	   product_number is the correct product for this installation
+	   and we have the location from the catalog and we have the
+	   decoded catalog.tar file in the (SWI) object.
+
+	   we can now combine them and produce the installed file list */
+
+	flist = swi_make_fileset_file_list(swi, product_number, SWI_FILESET_0, location);
+	return flist;
+}
+
+SWI_FILELIST *
+swi_fl_create(void) {
+	return (SWI_FILELIST *)strar_open();	
+}
+
+void
+swi_fl_delete(SWI_FILELIST * fl) {
+	strar_close((STRAR*)fl);
+}
+
+char *
+swi_fl_add_store(SWI_FILELIST * fl, int len) {
+	STRAR * ar = (STRAR*)fl;
+	char * s;
+	s = strar_return_store(ar, len + SWI_FL_HEADER_LEN);
+	return s;	
+}
+
+void
+swi_fl_set_path(SWI_FILELIST * fl, char * s,  char * prefix, char * path) {
+	SWSWI_E_DEBUG2("path=[%s]", path);
+	SWSWI_E_DEBUG2("prefix=[%s]", prefix);
+	if (prefix == NULL) prefix = "";
+	if (strlen(prefix) > 0) {
+		strncpy(s + (SWI_FL_HEADER_LEN), prefix, strlen(prefix));
+		SWSWI_E_DEBUG2("s=[%s]", s);
+	}
+	strncpy(s + (SWI_FL_HEADER_LEN + strlen(prefix)), path, strlen(path));
+	SWSWI_E_DEBUG2("s=[%s]", s);
+		/* NUL is added by strar_return_store() it must
+		   be the only terminating NUL for the record otherwise
+		   the STRAR object is corrupted. */
+}
+
+void
+swi_fl_set_user_flag(SWI_FILELIST * fl, char * hdr, int msg) {
+	*(hdr + SWI_FL_OFFSET_USER_FLAG) = msg;
+}
+
+int
+swi_fl_get_user_flag(SWI_FILELIST * fl, int index) {
+	STRAR * ar = (STRAR*)fl;
+	char * s;
+	s = strar_get(ar, index);
+	if (s == NULL) return 0;
+	return (int)(*(s + SWI_FL_OFFSET_USER_FLAG));
+}
+
+void
+swi_fl_set_is_volatile(SWI_FILELIST * fl, char * hdr, int index, int is_vol) {
+	if (hdr == NULL) {
+		STRAR * ar = (STRAR*)fl;
+		char * s;
+		s = strar_get(ar, index);
+		if (s == NULL) return;
+		hdr = s;
+	}
+	if (is_vol)
+		*(hdr + SWI_FL_OFFSET_IS_VOLATILE) = 'T';
+	else
+		*(hdr + SWI_FL_OFFSET_IS_VOLATILE) = 'F';
+}
+
+void
+swi_fl_set_type(SWI_FILELIST * fl, char * hdr, int tartype) {
+	*(hdr + SWI_FL_OFFSET_TARTYPE) = (unsigned char)tartype;
+}
+
+int
+swi_fl_get_type(SWI_FILELIST * fl, int index) {
+	STRAR * ar = (STRAR*)fl;
+	char * s;
+	s = strar_get(ar, index);
+	if (s == NULL) return 0;
+	return (int)(*(s + SWI_FL_OFFSET_TARTYPE));
+}
+
+
+char *
+swi_fl_get_path(SWI_FILELIST * fl, int index) {
+	STRAR * ar = (STRAR*)fl;
+	char * s;
+	s = strar_get(ar, index);
+	if (s == NULL) return NULL;
+	return s + SWI_FL_OFFSET_PATH;
+}
+
+int
+swi_fl_is_volatile(SWI_FILELIST * fl, int index) {
+	STRAR * ar = (STRAR*)fl;
+	char * s;
+	int ret;
+	s = strar_get(ar, index);
+	if (s == NULL) return 0;
+	if ((int)(*(s + SWI_FL_OFFSET_IS_VOLATILE)) == 'T') {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
+int
+swi_fl_compare_reverse(const void * vf1, const void * vf2)
+{
+	return -swi_fl_compare(vf1, vf2);
+}
+
+int
+swi_fl_compare(const void * vf1, const void * vf2)
+{
+	char * f1;
+	char * f2;
+	f1 = *((char**)(vf1)); 	
+	f2 = *((char**)(vf2)); 	
+	return strcmp(f1 + SWI_FL_HEADER_LEN, f2 + SWI_FL_HEADER_LEN);
+}
+
+void
+swi_fl_qsort_forward(SWI_FILELIST * fl) {
+	STRAR * ar = (STRAR*)fl;
+	strar_qsort(ar, swi_fl_compare);
+}
+
+void
+swi_fl_qsort_reverse(SWI_FILELIST * fl) {
+	STRAR * ar = (STRAR*)fl;
+	strar_qsort(ar, swi_fl_compare_reverse);
+}
