@@ -6,7 +6,7 @@
   
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef swi_h_20030523
 #define swi_h_20030523
@@ -36,6 +36,7 @@
 #include "uinfile.h"
 #include "swheader.h"
 #include "swheaderline.h"
+#include "strar.h"
 #include "swicol.h"
 #include "swi_base.h"
 #include "swi_common.h"
@@ -47,6 +48,33 @@
 #define SWI_GET_PRODUCT(W, N) (swi_package_get_product(W->swi_pkgM, N))
 #define SWI_INDEX_HEADER(W) (W->swi_pkgM->baseM.global_headerM)
 
+#define SWI_FL_TRUE			'T'
+#define SWI_FL_FALSE			'F'
+#define SWI_FL_OFFSET_TARTYPE		0
+#define SWI_FL_OFFSET_IS_VOLATILE	1
+#define SWI_FL_OFFSET_USER_FLAG		2
+#define SWI_FL_OFFSET_PATH		10
+#define SWI_FL_HEADER_LEN		10
+
+typedef union {
+	STRAR * arM;
+} SWI_FILELIST;
+
+SWI_FILELIST * swi_fl_create(void);
+void swi_fl_delete(SWI_FILELIST *);
+char * swi_fl_add_store(SWI_FILELIST * fl, int len);
+void  swi_fl_set_path(SWI_FILELIST*, char * hdr, char * prefix, char * path);
+void  swi_fl_set_is_volatile(SWI_FILELIST*, char * hdr, int index, int is_volatile);
+void  swi_fl_set_type(SWI_FILELIST*, char * hdr, int );
+void  swi_fl_set_user_flag(SWI_FILELIST*, char * hdr, int );
+int swi_fl_get_user_flag(SWI_FILELIST*, int );
+char * swi_fl_get_path(SWI_FILELIST*, int index);
+int swi_fl_is_volatile(SWI_FILELIST*, int index);
+int swi_fl_get_type(SWI_FILELIST*, int index);
+int swi_fl_compare(const void * f1, const void * f2);
+int swi_fl_compare_reverse(const void * f1, const void * f2);
+void swi_fl_qsort_reverse(SWI_FILELIST * fl);
+void swi_fl_qsort_forward(SWI_FILELIST * fl);
 
 typedef struct {		/* the Bundle structure			*/
 	char type_idM;		/* 'B'					*/
@@ -65,6 +93,7 @@ typedef struct {		/* the Product structure		*/
 	SWI_XFILE * pfilesM;
 	STRAR * filesetsM;	/* list of installed filesets		*/
 	int is_selectedM;	/* is selected				*/
+	int is_compatibleM;	/* is compatible			*/
 	int INDEX_ordinalM;     /* Order in INDEX file.  0 is first     */      
 } SWI_PRODUCT;
 
@@ -91,6 +120,8 @@ typedef struct {			/* The global package structure	*/
 					/* relative to  target_path	*/
 	char * installed_software_catalogM; /* Extended option value	*/
 	int is_minimal_layoutM;		/* does have empty control dirs */
+	char * locationM;		/* location spec */
+	char * qualifierM;		/* qualifier spec */
 } SWI_PACKAGE;
 
 typedef struct {
@@ -115,7 +146,7 @@ typedef struct {	/* The Global package control structure */
 	char tarbufM[1024];
 	SWI_DISTDATA * distdataM;
 	char * sh_dash_sM;	/* the shell command			*/
-	int dash_rM;		/* swinstall -r option			*/
+	int opt_alt_catalog_rootM; /* -r option				*/
 	char * exported_catalog_prefixM; /* i.e.  <path>/catalog/ from  */
 				/* the distribution 			*/
 	SWICOL * swicolM;
@@ -123,7 +154,8 @@ typedef struct {	/* The Global package control structure */
 	void * optaM;		/* extended options structure		*/
 	int swc_idM;		/* SWC_?   which utility		*/ 
 	int debug_eventsM;      /*                                      */
-	SWVERID * version_idM;  /* target Version Id                    */
+	SWVERID * target_version_idM;  /* Version Id of the target host        */
+	int does_have_payloadM; /* -1, 0 or 1, is there a payload? */
 } SWI;
 
 void swi_base_set_is_active(SWI_BASE * base, int n);
@@ -164,6 +196,7 @@ int swi_get_distribution_attributes(SWI * swi, SWHEADER * swheader);
 void swi_xfile_add_control_script(SWI * swi, SWI_XFILE * xfile, char * name, int fd, char * tag);
 int swi_distdata_resolve(SWI * swi, SWI_DISTDATA * part1, int);
 char * swi_dump_string_s(SWI * xx, char * prefix);
+char * swi_base_dump_string_s(SWI_BASE * xx, char * prefix);
 int swi_examine_signature_blocks(SWI_XFILE * dfiles, int * sig_block_start, int * sig_block_end);
 SWI_PRODUCT * swi_package_get_product(SWI_PACKAGE * swi_pkg, int arr_index);
 SWI_XFILE * swi_package_get_fileset(SWI_PRODUCT * swi_prod, int arr_index);
@@ -176,7 +209,7 @@ SWI_XFILE * swi_find_fileset_by_swsel(SWI_PRODUCT * parent, char * swsel);
 void swi_get_uts_attributes_from_current(SWI * swi, SWUTS * uts, SWHEADER * swheader);
 void swi_set_utility_id(SWI * swi, int swc_u_id);
 
-int swi_do_decode(SWI * swi, SWUTIL * swutil,
+int swi_do_decode(SWI * swi, SWLOG * swutil,
 	int target_fd1,
 	int source_fd0,
 	char * target_path,
@@ -189,4 +222,10 @@ int swi_do_decode(SWI * swi, SWUTIL * swutil,
 	int verboseG,
 	struct sw_logspec * g_logspec,
 	int uinfile_open_flags);
+
+int swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int * num_products);
+SWHEADER * swi_get_global_index_header(SWI * swi);
+SWHEADER * swi_get_fileset_info_header(SWI * swi, int product_index, int fileset_index);
+SWI_FILELIST * swi_make_fileset_file_list(SWI * swi, int product_num, int fileset_num, char * location);
+SWI_FILELIST * swi_make_file_list(SWI * swi, SWUTS * swuts, char * location, int disallow_incompatible);
 #endif
