@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -158,6 +158,12 @@ swi_xfile_set_state(SWI_XFILE * s, char * state)
 	s->stateM[sizeof(s->stateM)-1] = '\0';
 }
 
+SWHEADER *
+swi_xfile_get_infoheader(SWI_XFILE * xfile)
+{
+	return xfile->info_headerM;
+}
+
 SWI_FILE_MEMBER * 
 swi_xfile_get_control_file_by_path(SWI_XFILE * xfile, char * path)
 {
