@@ -4,7 +4,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,10 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+#define FILENEEDDEBUG 1
+#undef FILENEEDDEBUG
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -34,9 +36,12 @@
 #include "hllist.h"
 #include "defer.h"
 #include "swlib.h"
+#include "inttostr.h"
+#include "strtoint.h"
+#include "swutilname.h"
 
 static
-int
+intmax_t
 taru_write_archive_member_data_(TARU * taru,
 				struct new_cpio_header *file_hdr0,
 			      	int out_file_des,
@@ -44,18 +49,20 @@ taru_write_archive_member_data_(TARU * taru,
 			      	int input_fd,
 				int (*fout)(int),
 			  	enum archive_format archive_format_out,
-				int adjuct_ofd, char * md5buf)
+				int adjuct_ofd, FILE_DIGS * digs)
 {
 	int fd = -1;
-	int ret = 0;
+	intmax_t ret = 0;
 	char * tmpp = ahsStaticGetTarLinkname(file_hdr0);
 
-	if (md5buf) *md5buf = (char)0;
+	E_DEBUG("");
 
 	if ( (((file_hdr0->c_mode & CP_IFMT) == CP_IFREG) &&
 				(tmpp == (char*)NULL || !strlen(tmpp)))
 	) {
+		E_DEBUG("");
 		if (input_fd < 0) {
+			E_DEBUG("");
 			fd = open(ahsStaticGetTarFilename(file_hdr0),
 					O_RDONLY, 0);
 			if (fd < 0) {
@@ -69,39 +76,47 @@ taru_write_archive_member_data_(TARU * taru,
 		}
 
 		if (fout == NULL) {
-			if (md5buf == (char*)NULL) {
-				ret = swlib_pump_amount7(out_file_des, input_fd,
-						file_hdr0->c_filesize,
-						adjuct_ofd);
-			} else {
-				ret = swlib_md5_copy(input_fd, (int)(file_hdr0->c_filesize), md5buf, out_file_des);
-			}
+			E_DEBUG("");
+			/*
+			ret = swlib_digs_copy(out_file_des, input_fd, (intmax_t)(file_hdr0->c_filesize), digs, adjuct_ofd);
+			*/
+			ret = swlib_pump_amount8(out_file_des, input_fd, (intmax_t)(file_hdr0->c_filesize), adjuct_ofd, digs);
+			E_DEBUG2("ret=%s", swlib_imaxtostr(ret, NULL));
 
 			if (ret < 0) {
+				E_DEBUG("");
 				if (fd >= 0) close(fd);
-				fprintf(stderr,
-				"error in taru_write_archive_member_data (md5buf=%p): ret=%d\n", md5buf, ret);
+				fprintf(stderr, "%s: error in taru_write_archive_member_data: ret=%s\n",
+					swlib_utilname_get(), swlib_imaxtostr(ret, NULL));
 				return -1;
 			} else {
-				if (ret != (int)(file_hdr0->c_filesize)) {
+				E_DEBUG("");
+				if (ret != (intmax_t)(file_hdr0->c_filesize)) {
+					E_DEBUG2("Error: ret=%s", swlib_imaxtostr(ret, NULL));
+					E_DEBUG2("Error: filesize=%s", swlib_umaxtostr(file_hdr0->c_filesize, NULL));
 					fprintf(stderr,
-					"taru_write_archive_member_data_ internal error. ret=%d\n", ret);
+					"%s: taru_write_archive_member_data_ internal error. ret=%s\n",
+						swlib_utilname_get(), swlib_imaxtostr(ret, NULL));
 				} else {
+					E_DEBUG("");
 					/*
 					fprintf(stderr, "%s loaded OK\n", ahsStaticGetTarFilename(file_hdr0));
 					*/
 				}
+				E_DEBUG("");
 				if ((ret += taru_tape_pad_output(
 						pad_out_file_des,
-						file_hdr0->c_filesize,
+						(intmax_t)(file_hdr0->c_filesize),
 						archive_format_out)) < 0) {
+					E_DEBUG("");
 					if (fd >= 0) close(fd);
 					return -1;
 				}
+				E_DEBUG("");
 				if (adjuct_ofd >= 0) {
 					if (taru_tape_pad_output(
 							adjuct_ofd,
-							file_hdr0->c_filesize,
+							(intmax_t)(file_hdr0->c_filesize),
 							archive_format_out) < 0
 					) {
 						if (fd >= 0) close(fd);
@@ -109,11 +124,15 @@ taru_write_archive_member_data_(TARU * taru,
 					}
 				}
 			}
+			E_DEBUG("");
 		} else {
+			E_DEBUG("");
 			ret = (*fout)(out_file_des);
+			E_DEBUG2("ret=%d", ret);
 		}
 		if (fd >= 0) close(fd);
 	} 
+	E_DEBUG2("ret=%s", swlib_imaxtostr(ret, NULL));
 	return ret;
 }
 
@@ -133,10 +152,10 @@ tarui_tape_buffered_write(char *in_buf, int out_des, long num_bytes,
 
 static int
 tape_clear_rest_of_block(int out_file_des, char * zeros_512,
-			int io_block_size, unsigned long output_size)
+			int io_block_size, uintmax_t output_size)
 {
-	unsigned long rem = (int)(output_size % (unsigned long)io_block_size); 
-	int n = rem;
+	intmax_t rem = output_size % (intmax_t)io_block_size; 
+	intmax_t n = rem;
   	int ret = 0;
 	int retval = 0;
 	while (n < io_block_size && ret >= 0) {
@@ -183,7 +202,7 @@ taru_read_for_crc_checksum (int in_file_des, int file_size, char * file_name)
    return crc;
 }
 
-static int
+static intmax_t
 tarui_write_archive_member_(TARU * taru, 
 				struct stat *file_stat,
 				struct new_cpio_header *file_hdr0,
@@ -195,7 +214,8 @@ tarui_write_archive_member_(TARU * taru,
 				enum archive_format archive_format_out,
 				int tarheaderflags)
 {
-	int ret = 0, ret1 = 0;
+	intmax_t ret = 0;
+	intmax_t ret1 = 0;
 	size_t sz;
 	off_t old_pos;
 	struct new_cpio_header * file_hdr = ahsStaticCreateFilehdr();
@@ -234,11 +254,14 @@ tarui_write_archive_member_(TARU * taru,
 		ret1 = taru_write_archive_member_data(taru, file_hdr,
 				out_file_des, input_fd,
 				(int(*)(int))(NULL),
-				archive_format_out, -1, NULL);
+				archive_format_out, -1, file_hdr0->digsM);
 		if (ret1 < 0) {
 			ahsStaticDeleteFilehdr(file_hdr);
 			return ret1;
 		}
+	} else {
+		if (file_hdr0->digsM)
+			taru_digs_init(file_hdr0->digsM, DIGS_ENABLE_CLEAR);
 	}
 	ahsStaticDeleteFilehdr(file_hdr);
 	return ret + ret1;
@@ -265,7 +288,7 @@ int
 taru_write_archive_trailer(TARU * taru,
 				enum archive_format archive_format_out,
 				int fd, int io_block_size,
-				unsigned long bytes_written, int flags)
+				uintmax_t bytes_written, int flags)
 {
 	int ret;
 	int retval = 0;
@@ -304,9 +327,10 @@ taru_write_archive_trailer(TARU * taru,
 	} else {
 		int nblock;
 		if (bytes_written % 512) {
+			char tmpbuf[UINTMAX_STRSIZE_BOUND];
 			fprintf(stderr, 
-			"internal error, bytes written not multiple of 512, bytes=%lu\n",
-			bytes_written);
+			"internal error, bytes written not multiple of 512, bytes=%s\n",
+				umaxtostr(bytes_written, tmpbuf));
 			flags = 0;	
 			retval = 1;
 		}
@@ -335,7 +359,7 @@ taru_write_archive_trailer(TARU * taru,
 	return ret;
 }
 
-int
+intmax_t
 taru_write_archive_member(TARU * taru, char *filename, 
 				struct stat *statbuf, 
 				struct new_cpio_header * file_hdr0, 
@@ -349,7 +373,8 @@ taru_write_archive_member(TARU * taru, char *filename,
 	char linkname[101];
 	struct stat st;
 	struct new_cpio_header * file_hdr = ahsStaticCreateFilehdr();
-	int fd = -1, ret = 0;
+	int fd = -1;
+	intmax_t ret = 0;
 
 	*linkname='\0';;
 	if (statbuf) {
@@ -409,26 +434,27 @@ taru_write_archive_member(TARU * taru, char *filename,
 	return ret;
 }
 
-int
+intmax_t
 taru_write_archive_member_data(TARU * taru, 
 				struct new_cpio_header *file_hdr0,
 			      	int out_file_des, 
 			      	int input_fd, int (*fout)(int),
 			  	enum archive_format archive_format_out,
-				int adjuct_ofd, char * md5buf)
+				int adjuct_ofd, FILE_DIGS * digs)
 {
-	int ret = taru_write_archive_member_data_(taru, 
+	intmax_t ret;
+	ret = taru_write_archive_member_data_(taru, 
 				file_hdr0,
 			      	out_file_des, 
 			      	out_file_des, 
 			      	input_fd,
 				fout,
 			  	archive_format_out,
-				adjuct_ofd, md5buf);
+				adjuct_ofd, digs);
 	return ret;
 }
 
-int
+intmax_t
 taru_write_archive_file_data(TARU * taru, 
 				struct new_cpio_header *file_hdr0,
 			      	int out_file_des, 
@@ -437,7 +463,7 @@ taru_write_archive_file_data(TARU * taru,
 			  	enum archive_format archive_format_out,
 				int adjuct_ofd)
 {
-	int ret;
+	intmax_t ret;
 	int nullfd;
 
 	if (!taru) 
@@ -445,6 +471,10 @@ taru_write_archive_file_data(TARU * taru,
 	else
 		nullfd = taru->nullfdM;
 
+	/* adjunct_ofd is not used and should be removed */
+	if (adjuct_ofd >= 0) {
+		SWLIB_FATAL("adjuct_ofd");
+	}
 	ret = taru_write_archive_member_data_(taru, 
 				file_hdr0,
 			      	out_file_des, 
@@ -452,8 +482,8 @@ taru_write_archive_file_data(TARU * taru,
 			      	input_fd,
 				fout,
 			  	archive_format_out,
-				adjuct_ofd, NULL);
-
+				-1/* adjuct_fd */,
+				file_hdr0->digsM);
 	if (!taru) swlib_close_nullfd(nullfd);	
 	return ret;
 }
@@ -481,6 +511,7 @@ taru_write_archive_member_header(TARU * taru,
 	int do_strip_leading_slash = 0;
 	int linkrecord_active;
 
+	E_DEBUG("");
 	file_hdr = ahsStaticCreateFilehdr();
 	linkrecord_active = (linkrecord != NULL && taru->linkrecord_disableM == 0);
 
@@ -494,6 +525,7 @@ taru_write_archive_member_header(TARU * taru,
 	 */
 	taru_filehdr2filehdr(file_hdr, file_hdr0);
 	if (file_stat) {
+		E_DEBUG("file_stat is true");
 		taru_statbuf2filehdr(file_hdr, file_stat, NULL,
 				ahsStaticGetTarFilename(file_hdr),
 				ahsStaticGetTarLinkname(file_hdr));
@@ -513,6 +545,7 @@ taru_write_archive_member_header(TARU * taru,
 		archive_format_out == arf_crcascii)
 	) {
 		int mylink = file_hdr->c_nlink;
+		E_DEBUG("");
 		switch (file_hdr->c_mode & CP_IFMT) {
 			case CP_IFLNK:
 			case CP_IFDIR:
@@ -529,6 +562,7 @@ taru_write_archive_member_header(TARU * taru,
 	}	
 
 	if (do_strip_leading_slash) {
+		E_DEBUG("");
 		ahsStatic_strip_name_leading_slash(file_hdr);
 	}
 
@@ -537,6 +571,7 @@ taru_write_archive_member_header(TARU * taru,
 	/*
 	* Check the name length.
 	*/
+	E_DEBUG2("name is [%s]", ahsStaticGetTarFilename(file_hdr));
 	file_hdr->c_namesize = strlen(ahsStaticGetTarFilename(file_hdr)) + 1;
 	if ((archive_format_out == arf_tar || archive_format_out == arf_ustar)
 	    && taru_is_tar_filename_too_long(
@@ -544,6 +579,7 @@ taru_write_archive_member_header(TARU * taru,
 				tarheaderflags, 
 				&do_gnu_long_link,
 				is_dir)) {
+		E_DEBUG("");
 		fprintf(stderr, "error: file name too long: %s\n",
 				ahsStaticGetTarFilename(file_hdr));
 		ahsStaticDeleteFilehdr(file_hdr);
@@ -551,6 +587,7 @@ taru_write_archive_member_header(TARU * taru,
 	}
 
 	if (do_gnu_long_link) {
+		E_DEBUG("");
 		if ((do_gnu_long_link=taru_write_long_link_member
 				(taru, out_file_des,
 					ahsStaticGetTarFilename(file_hdr),
@@ -586,17 +623,26 @@ taru_write_archive_member_header(TARU * taru,
 			*(ahsStaticGetTarFilename(file_hdr) + TARNAMESIZE) = '\0';
 		}
 	} else {
+		E_DEBUG("");
 		swlib_strncpy(input_name,
 				ahsStaticGetTarFilename(file_hdr), 512);
 	}
 
+	E_DEBUG2("c_is_link = %d", file_hdr->c_is_tar_lnktype);
+	if (file_hdr->c_is_tar_lnktype == 1) {
+		;
+		E_DEBUG2("linkname is [%s]", ahsStaticGetTarLinkname(file_hdr));
+	}
+	E_DEBUG("");
 	switch (file_hdr->c_mode & CP_IFMT) {
 	case CP_IFREG:
 	{
+	E_DEBUG("CP_IFREG");
 	if ((archive_format_out == arf_oldascii ||
 		archive_format_out == arf_newascii ||
 		archive_format_out == arf_crcascii)
 	    && (file_hdr->c_nlink > 1)) {
+		E_DEBUG("");
 		if (defer){
 			defer_set_taru(defer, taru);
 			if (defer_is_last_link(defer,
@@ -610,13 +656,15 @@ taru_write_archive_member_header(TARU * taru,
 		}
 		else {
 			fprintf(stderr,
-				"taru internal error: cpio hard link handling is disabled.\n");
+				"taru internal error: cpio hard link handling is disabled (loc=2).\n");
 		}
 	} else if (archive_format_out == arf_ustar ||
 		archive_format_out == arf_tar) {
 		if (linkrecord_active) {
+			E_DEBUG("");
 			nfound=0;	
 			if (file_hdr->c_nlink > 1)
+				E_DEBUG("");
 				link_record_buf = 
 				hllist_find_file_entry(linkrecord, 
 					(dev_t) makedev(file_hdr->c_dev_maj,\
@@ -624,11 +672,19 @@ taru_write_archive_member_header(TARU * taru,
 	 				file_hdr->c_ino,
 					1,
 					&nfound);
-				if (nfound) {
+				if (nfound || file_hdr->c_is_tar_lnktype == 1) {
 					/* fprintf(stderr, "JLDX here here [%s] \n", link_record_buf->path_); */
 					/* hllist_show_to_file(linkrecord, stderr); */
-					ahsStaticSetTarLinkname(file_hdr,
-						link_record_buf->path_);
+					E_DEBUG("");
+					
+					if (nfound) {
+						ahsStaticSetTarLinkname(file_hdr,
+							link_record_buf->path_);
+
+					} else {
+						; /* already set */
+					}	
+
 					retval = taru_write_out_header
 						(taru, file_hdr, 
 						out_file_des, 
@@ -636,14 +692,18 @@ taru_write_archive_member_header(TARU * taru,
 						(char *)(NULL),
 						tarheaderflags);
 					break;
+				} else {
+					E_DEBUG("not found");
 				}
 			}
 			else {
+				E_DEBUG("");
 				if (!linkrecord)
 					fprintf(stderr, 
-					"taru internal error: tar hard link handling is disabled.\n");
+					"taru internal error: tar hard link handling is disabled (loc=3).\n");
 			}
 		} else {
+			E_DEBUG("");
 			/* 
 			is traversed.
 			*/
@@ -655,12 +715,14 @@ taru_write_archive_member_header(TARU * taru,
 		if (archive_format_out == arf_ustar ||
 			archive_format_out == arf_tar) 
 		{
+			E_DEBUG("");
 			if (linkrecord_active) {
 				if (file_hdr->c_nlink > 1) {
 					/*
 					fprintf(stderr, "JLDX hllist_add_record %s %d :%s\n",
 						 __FILE__, __LINE__, ahsStaticGetTarFilename(file_hdr));
 					*/
+					E_DEBUG("");
 					hllist_add_record(linkrecord,
 					ahsStaticGetTarFilename(file_hdr), 
 					(dev_t)makedev(file_hdr->c_dev_maj,\
@@ -669,14 +731,16 @@ taru_write_archive_member_header(TARU * taru,
 				}
 			}
 			else {
+				E_DEBUG("");
 				if (!linkrecord)
 				fprintf(stderr,
-					"taru internal error: tar hard link handling is disabled.\n");
+					"taru internal error: tar hard link handling is disabled (loc=4).\n");
 			}
 		}
 		break;
 	}
 	case CP_IFDIR:
+		E_DEBUG("CP_IFDIR");
 		file_hdr->c_filesize = 0;
 		retval = taru_write_out_header
 			(taru, file_hdr, out_file_des, 
@@ -686,8 +750,9 @@ taru_write_archive_member_header(TARU * taru,
 
 #ifdef CP_IFSOCK
 	case CP_IFSOCK:
+		E_DEBUG("CP_IFSOCK");
 		fprintf(stderr, 
-		"taru: %s: socket ignored\n",
+		"%s: %s: socket ignored\n", swlib_utilname_get(),
 				ahsStaticGetTarFilename(file_hdr));
 		retval = 0;
 		break;
@@ -697,19 +762,23 @@ taru_write_archive_member_header(TARU * taru,
 #endif
 	case CP_IFCHR:
 	case CP_IFBLK:
+		E_DEBUG("CP_FIFO CP_IFCHR CP_IFBLK");
 	if (archive_format_out == arf_tar) {
 		   fprintf (stderr, "%s not dumped: not a regular file",
 		   		ahsStaticGetTarFilename(file_hdr));
 		   ahsStaticDeleteFilehdr(file_hdr);
 		   return 0;
 	} else if (archive_format_out == arf_ustar) {
+		E_DEBUG("");
 		if (linkrecord_active) {
+			E_DEBUG("");
 			link_record_buf = hllist_find_file_entry(
 				linkrecord,
 				(dev_t)makedev(file_hdr->c_dev_maj,\
 					file_hdr->c_dev_min),
 					file_hdr->c_ino, 1, &nfound);
 			if (nfound) {
+				E_DEBUG("");
 				file_hdr->c_mode |= CP_IFREG;
 				ahsStaticSetTarLinkname(file_hdr,
 						link_record_buf->path_);
@@ -723,11 +792,15 @@ taru_write_archive_member_header(TARU * taru,
 			}
 		}
 		else {
+			E_DEBUG("");
+		/*
 			fprintf(stderr,
-		"taru internal error: tar hard link handling is disabled.\n");
+		"taru internal error: tar hard link handling is disabled (loc=5).\n");
+		*/
 		}
 	}
 	file_hdr->c_filesize = 0;	
+	E_DEBUG("");
 	retval = taru_write_out_header
 		(taru, file_hdr, out_file_des,
 		 archive_format_out,
@@ -735,6 +808,7 @@ taru_write_archive_member_header(TARU * taru,
 		tarheaderflags);
 	if (archive_format_out == arf_ustar ||
 			archive_format_out == arf_tar) {
+			E_DEBUG("");
 			if (linkrecord_active) {
 				if (file_hdr->c_nlink > 1) {
 					/*
@@ -748,17 +822,23 @@ taru_write_archive_member_header(TARU * taru,
 				}
 			}
 			else {
+				E_DEBUG("");
+				/*
 				fprintf(stderr, 
-	"taru internal error: tar hard link handling is disabled.\n");
+					"taru internal error: tar hard link handling is disabled (loc=1).\n");
+				*/
 			}
 	}
 	break;
 
 #ifdef CP_IFLNK
+	
 	case CP_IFLNK:
+	E_DEBUG("CP_IFLNK");
 	{
 		char link_name[128];	
 		int link_size;
+		E_DEBUG("");
 		link_size = strlen(ahsStaticGetTarLinkname(file_hdr));
 		file_hdr->c_filesize = link_size;
 		if (archive_format_out == arf_tar ||
@@ -776,6 +856,7 @@ taru_write_archive_member_header(TARU * taru,
 						tarheaderflags);
 			}
 		} else {
+			E_DEBUG("");
 			retval = taru_write_out_header
 				(taru, file_hdr, out_file_des,
 					archive_format_out,	
@@ -785,13 +866,14 @@ taru_write_archive_member_header(TARU * taru,
 					ahsStaticGetTarLinkname(file_hdr),
 					out_file_des, link_size, NULL);
 			retval += taru_tape_pad_output(
-					out_file_des, link_size,
+					out_file_des, (intmax_t)link_size,
 					archive_format_out);
 		}	
 	}
 	break;
 #endif
 	default:
+		E_DEBUG("");
 		fprintf(stderr, 
 		"%s: unknown file type in archive header : mode=%o FMT=%o \n",
 			input_name, 
@@ -800,18 +882,21 @@ taru_write_archive_member_header(TARU * taru,
 			);
 	}
 	if (file_hdr_return) {
+		E_DEBUG("");
 		taru_filehdr2filehdr(file_hdr_return, file_hdr);
 	}
+	E_DEBUG("");
 	ahsStaticDeleteFilehdr(file_hdr);
 	return retval + do_gnu_long_link;
 }
 
 int
-taru_tape_pad_output(int out_file_des, int offset, 
+taru_tape_pad_output(int out_file_des, intmax_t offset, 
 				enum archive_format archive_format)
 {
-	int pad;
+	intmax_t pad;
 	char zeros_512[512];
+
 	memset(zeros_512, 0x00, 512);
 
 	if (archive_format == arf_newascii ||
@@ -878,7 +963,7 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 			magic_string,
 		      file_hdr->c_ino, file_hdr->c_mode, file_hdr->c_uid,
 		   file_hdr->c_gid, file_hdr->c_nlink, file_hdr->c_mtime,
-			file_hdr->c_filesize, file_hdr->c_dev_maj,
+			(unsigned long)(file_hdr->c_filesize), file_hdr->c_dev_maj,
 			file_hdr->c_dev_min, file_hdr->c_rdev_maj,
 			file_hdr->c_rdev_min, file_hdr->c_namesize,
 			file_hdr->c_chksum);
@@ -894,7 +979,7 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 		if (ret < 0) return ret;
 		ret += taru_tape_pad_output(
 				out_des,
-				file_hdr->c_namesize + 110,
+				(intmax_t)(file_hdr->c_namesize + 110),
 				archive_format_out);
 		return ret;
 	} else if (archive_format_out == arf_oldascii ||
@@ -953,7 +1038,7 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 		      file_hdr->c_uid & 0xFFFF, file_hdr->c_gid & 0xFFFF,
 			file_hdr->c_nlink & 0xFFFF, (int) (rdev & 0xFFFF),
 			file_hdr->c_mtime, file_hdr->c_namesize & 0xFFFF,
-			file_hdr->c_filesize);
+			(unsigned long)(file_hdr->c_filesize));
 		ret+=tarui_tape_buffered_write(ascii_header,
 						out_des,
 						76L, header_buffer);
@@ -1041,7 +1126,7 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 					header_buffer);
 		if (ret < 0) return ret;
 		ret+=taru_tape_pad_output(out_des,
-					 file_hdr->c_namesize + 26,
+					 (intmax_t)(file_hdr->c_namesize + 26),
 					archive_format_out);
 		return ret;
 	}
@@ -1056,20 +1141,44 @@ taru_write_out_header(TARU * taru, struct new_cpio_header *file_hdr,
 int
 taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 			DEFER * defer, PORINODE * porinode,
-			enum archive_format archive_format_out)
+			enum archive_format archive_format_out,
+			int ls_fd, int ls_verbose_level, intmax_t * nb, FILE_DIGS * digs)
 {
+	STROB * ls_tmp;
 	struct stat file_stat;
-	struct new_cpio_header * file_hdr0 = ahsStaticCreateFilehdr();
-	char zeros_512[512], buf[512];
+	struct new_cpio_header * file_hdr0;
+	union tar_record tar_rec;
+	struct tar_header *tar_hdr = (struct tar_header *) &tar_rec;
+	char zeros_512[512];
+	char buf[512+512+512];
 	int block_number = 0;
 	int eoa = 0;
+	int ret;
+	intmax_t retx;
+	intmax_t retvalx;
+	HLLIST * hllist;
+	int header_length;
+	char intbuf[UINTMAX_STRSIZE_BOUND];
 
+	file_hdr0 = ahsStaticCreateFilehdr();
 	memset(zeros_512, 0x00, 512);
+	hllist = hllist_open();
+	hllist_disable_add(hllist);
+	hllist_disable_find(hllist);
+	retvalx = 0;
 
-	while (uxfio_sfread(input_fd, buf, 512) == 512) {
+	file_hdr0->digsM = digs;	
 
-		if (!memcmp(buf, zeros_512, 512)) {
-			if (uxfio_sfread(input_fd, buf, 512) == 512) {
+	if (ls_fd >= 0) {
+		ls_tmp = strob_open(120);
+	} else {
+		ls_tmp = NULL;
+	}
+
+	while (uxfio_sfa_read(input_fd, buf, 512) == 512) {
+
+		if (memcmp(buf, zeros_512, 512) == 0) {
+			if (uxfio_sfa_read(input_fd, buf, 512) == 512) {
 				if (!memcmp(buf, zeros_512, 512)) {
 					block_number++;
 					eoa = 1;
@@ -1080,26 +1189,45 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 				}
 			} else {
 				ahsStaticDeleteFilehdr(file_hdr0);
-				return -1;
+				return -2;
 			}
 		}
 		if (taru_tarheader_check(buf)) {
-			fprintf(stderr, "tarheader error\n");
+			fprintf(stderr, "tarheader error ifd=%d ofd=%d\n", input_fd, out_file_des);
 			ahsStaticDeleteFilehdr(file_hdr0);
-			return -1;
+			return -4;
 		}
+
+		memcpy((char*)tar_hdr, buf, 512);
+		if (*(((char*)tar_hdr) + 156) == 'L') {
+			taru_otoumax(tar_hdr->size, &file_hdr0->c_filesize);
+			if (file_hdr0->c_filesize > 511) {
+				/* FIXME support arbitrary length name */
+				return -5;
+			}
+			ret = uxfio_sfa_read(input_fd, buf+512, 512);
+			if (ret != 512) return -6;
+			ret = uxfio_sfa_read(input_fd, buf+512+512, 512);
+			if (ret != 512) return -7;
+			header_length = TARRECORDSIZE+TARRECORDSIZE+TARRECORDSIZE;
+		} else {
+			/* FIXME, check for extended headers */
+			header_length = TARRECORDSIZE;
+		}
+
 		taru_read_in_tar_header2(taru, file_hdr0, -1,
-							buf, (int*)NULL, 0);
+					buf, (int*)NULL, 0, header_length);
+
 
 		if (taru_filehdr2statbuf(&file_stat, file_hdr0) < 0) {
 			fprintf(stderr, "%s",
 					ahsStaticGetTarFilename(file_hdr0));
 			ahsStaticDeleteFilehdr(file_hdr0);
-			return -1;
+			return -8;
 		} else {
-			tarui_write_archive_member_
+			retx = tarui_write_archive_member_
 				(taru, &file_stat, file_hdr0,
-					NULL, defer, porinode,
+					hllist, defer, porinode,
 					out_file_des, 
 					input_fd, archive_format_out,
 					/*tarheaderflags*/ 0);
@@ -1107,13 +1235,32 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 			taru_tape_skip_padding(input_fd,
 						file_hdr0->c_filesize,
 							arf_ustar);
+			if (retx < 0)
+				return -9;
+			retvalx += retx;
 		}
-	}			/* while read 512-byte blocks */
+
+		if (ls_fd >= 0) {
+			taru_print_tar_ls_list(ls_tmp, file_hdr0, ls_verbose_level);
+			uxfio_write(ls_fd, strob_str(ls_tmp), strob_strlen(ls_tmp));
+		} else {
+			;
+		}
+
+
+	}	/* while read 512-byte blocks */
 
 	/* The collection is complete; append the trailer.  */
-	taru_write_archive_trailer(taru,
+	ret = taru_write_archive_trailer(taru,
 				archive_format_out,
 				out_file_des, 512, 0, 0);
+
+	if (ret <= 0) return -10;
+	retvalx += (intmax_t)ret;
+	if (nb) *nb = (intmax_t)retvalx;
+	if (ls_tmp) strob_close(ls_tmp);
+	file_hdr0->digsM = NULL; /* Prevents deleting passed-in object */
 	ahsStaticDeleteFilehdr(file_hdr0);
+	hllist_close(hllist);
 	return 0;
 }
