@@ -4,6 +4,7 @@
 #ifndef TARU_H_INC_20031101
 #define TARU_H_INC_20031101
 
+#include "swuser_config.h"
 #include <sys/stat.h>
 #include "cpiohdr.h"
 #include "tarhdr.h"
@@ -12,6 +13,7 @@
 #include "defer.h"
 #include "hllist.h"
 #include "porinode.h"
+#include "config_remains.h"
 
 #define TARU_AR_HEADER_SIZE  60  /* GNU ar format */
 #define TARU_AR_SIZE_OFFSET  48  /* GNU ar format */
@@ -83,7 +85,7 @@ typedef struct {
 	STROB * u_name_bufferM;
 	STROB * u_ent_bufferM;
 	int nullfdM;
-	int do_md5M;
+	/* int do_md5M; */
 	STROB * md5bufM;
 	int linkrecord_disableM;
 	int preview_fdM;
@@ -132,15 +134,19 @@ typedef struct {
 #define TARU_PV_1	1  /* preview level, print filename */
 #define TARU_PV_2	2  /* preview level, print tar listing */
 #define TARU_PV_3	3  /* preview level, print verbose ad-hoc tar listing */
+
 /*
 * These control the /etc/passwd /etc/group
 * name,id lookup and mapping policy.
 */
+
+#define TARU_C_BY_UNA		4 /* not available, not set */
 #define TARU_C_BY_UNAME		3 /* get uid from uname. */
 #define TARU_C_BY_UID		2 /* get uname from uid */
 #define TARU_C_BY_USYS		1 /* Default existing policy */
 #define TARU_C_BY_UNONE		0 /* no database lookups, use id and name as is */
 
+#define TARU_C_BY_GNA		4 /* not available, not set */
 #define TARU_C_BY_GNAME		3 /* get gid from gname */
 #define TARU_C_BY_GID		2 /* get gname from gid */
 #define TARU_C_BY_GSYS		1 /* Default existing policy */
@@ -164,31 +170,31 @@ void taru_set_tarheader_flag(TARU * taru, int flag, int n);
 
 ssize_t taru_tape_buffered_read(int fd, void * buf, size_t count);
 
-unsigned long int ** taru_pump_get_ppstatbytes(void);
+intmax_t taru_pump_amount2(int discharge_fd, int suction_fd, intmax_t amount, int adjunct_ofd);
 
-int taru_pump_amount2(int discharge_fd, int suction_fd, int amount, int adjunct_ofd);
+intmax_t taru_pump_amount3(int discharge_fd, int suction_fd, intmax_t amount, int adjunct_ofd);
 
-int taru_pump_amount3(int discharge_fd, int suction_fd, int amount, int adjunct_ofd);
+intmax_t taru_pump_amount4(int discharge_fd, int suction_fd, intmax_t amount, int adjunct_ofd);
 
-int taru_pump_amount4(int discharge_fd, int suction_fd, int amount, int adjunct_ofd);
-
-int taru_read_amount(int suction_fd, int amount);
-
-/* int fnmatch(const char *, const char *, int); */
+intmax_t taru_read_amount(int suction_fd, intmax_t amount);
 
 void taru_mode_to_chars(mode_t v, char *p, size_t s, int termch);
 
+intmax_t taru_hdr_get_filesize(struct new_cpio_header * file_hdr);
+
 /*
  * Allocate/deallocate the header struct
  */
 struct new_cpio_header *	taru_make_header(void);
 void 				taru_free_header(struct new_cpio_header *h);
 void 				taru_init_header(struct new_cpio_header * file_hdr);
+void				taru_init_header_digs(struct new_cpio_header * file_hdr);
 
 /*
  * Translate a format to another format 
  */
-int 	taru_process_copy_out(TARU * taru, int input_fd, int output_fd, DEFER * defer, PORINODE * porindoe, enum archive_format archive_format);
+int 	taru_process_copy_out(TARU * taru, int input_fd, int output_fd, DEFER * defer,
+		PORINODE * porindoe, enum archive_format archive_format, int ls_fd, int ls_verbose, intmax_t*, FILE_DIGS * digs);
 int 	taru_format_translate(int ifd, int ofd, enum archive_format output_format);
 int 	taru_process_copy_in(int input_fd, int output_fd);
 
@@ -222,7 +228,7 @@ int taru_safewrite(int fd, void *vbuf, size_t amount);
 /* =========
  * Primary reading/writing functions.  
  */
-int taru_write_archive_member
+intmax_t taru_write_archive_member
 			(TARU * taru, char *filename, 
 			struct stat *statbuf, 
 			struct new_cpio_header *file_hdr0, 
@@ -241,7 +247,7 @@ int taru_write_archive_trailer
 			TARU * taru, enum archive_format archive_format_out,
 			int fd, 
 			int block_size, 
-			unsigned long bytes_written, 
+			uintmax_t bytes_written, 
 			int header_flags
 			);
 
@@ -265,7 +271,7 @@ int taru_read_in_header(
 			int * eoa, 
 			int flags);
 
-int taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr, int in_des, char *header_buffer, int * eoa, int flags);
+int taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr, int in_des, char *header_buffer, int * eoa, int flags, int);
 int taru_read_in_old_ascii2(TARU * taru, struct new_cpio_header *file_hdr, int in_des, char *buf);
 int taru_read_in_new_ascii(TARU * taru, struct new_cpio_header *file_hdr, int in_des, enum archive_format archive_format_in);
 
@@ -312,16 +318,17 @@ int     taru_write_long_link_member(
 /*
  * write out member data 
  */
-int	taru_write_archive_member_data(
+intmax_t
+taru_write_archive_member_data(
 		TARU * taru, 
 		struct new_cpio_header *file_hdr0, 
 		int out_file_des, 
 		int input_fd, 
 		int (*f)(int output_fd),
 		enum archive_format archive_format_out, 
-		int adjunct_ofd, char * md5buf);
+		int adjunct_ofd, FILE_DIGS * digs);
 
-int
+intmax_t
 taru_write_archive_file_data(TARU * taru, 
 				struct new_cpio_header *file_hdr0,
 			      	int out_file_des, 
@@ -333,8 +340,8 @@ taru_write_archive_file_data(TARU * taru,
 /*
  * Archive format padding 
  */
-int 	taru_tape_pad_output(int out_file_des, int offset, enum archive_format archive_format);
-int 	taru_tape_skip_padding(int in_file_des, int offset, enum archive_format archive_format);
+int 	taru_tape_pad_output(int out_file_des, intmax_t offset, enum archive_format archive_format);
+int 	taru_tape_skip_padding(int in_file_des, uintmax_t offset, enum archive_format archive_format);
 
 
 
@@ -360,6 +367,7 @@ long taru_from_oct(int digs, char *where);
 void taru_to_oct(register long value, register int digits, register char *where);
 
 int taru_otoul(char *s, unsigned long *n);
+int taru_otoumax (char * s, uintmax_t *n);
 
 int taru_datoul (char * s, unsigned long *n);
 
@@ -395,4 +403,8 @@ int taru_get_preview_level(TARU * taru);
 
 int taru_set_tar_header_policy(TARU * taru, char * user_format, int * p_arf_format);
 
+FILE_DIGS * taru_digs_create(void);
+void taru_digs_init(FILE_DIGS *, int);
+void taru_digs_print(FILE_DIGS * digs, STROB * );
+
 #endif
