@@ -4,7 +4,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,12 +14,19 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+#include "swuser_config.h"
 #include "xformat.h"
+#include "inttostr.h"
 #include "swlib.h"
+#include "swutilname.h"
 
+#define XFORMATNEEDDEBUG 1
 #include "debug_config.h"
+#define XFORMATNEEDDEBUG 1
+#undef XFORMATNEEDDEBUG
+
 #ifdef XFORMATNEEDDEBUG
 #define XFORMAT_E_DEBUG(format) SWBISERROR("XFORMAT DEBUG: ", format)
 #define XFORMAT_E_DEBUG2(format, arg) \
@@ -163,9 +170,13 @@ common_setup_write_header(XFORMAT * xux, struct new_cpio_header **p_hdr1,
 				* set the size and filetype because you
 				* probably don't want to lie about these.
 				*/
-		 		xformat_set_filesize(xux, (xux->lt2M).st_size);
+		 		xformat_set_filesize(xux, (intmax_t)((xux->lt2M).st_size));
 				tartypeflag = taru_get_tar_filetype(
 							(xux->lt2M).st_mode);
+				if (tartypeflag < 0) {
+					fprintf(stderr, "%s: unrecognized file type in mode [%d] for file: %s\n",
+						swlib_utilname_get(), (int)((xux->lt2M).st_mode), t_source);
+				}
 				xformat_set_filetype_from_tartype(xux,
 								tartypeflag);
 				taru_filehdr2filehdr(hdr0, *p_hdr1);
@@ -561,7 +572,7 @@ xformat_set_gid (XFORMAT * xux, gid_t gid)
 }    
   
 void
-xformat_set_filesize (XFORMAT * xux, int filesize)
+xformat_set_filesize (XFORMAT * xux, intmax_t filesize)
 { 
 	XFORMAT_E_DEBUG("");
         ahs_set_filesize(xux->ahsM, filesize);
@@ -769,14 +780,15 @@ int xformat_setdir(XFORMAT * xux, char * path)
 	return swvarfs_setdir(xux->swvarfsM, path);
 }
 
-int 
+intmax_t 
 xformat_read_file_data(XFORMAT * xux, int dst_fd)
 {
-       struct  new_cpio_header *file_hdr0 = 
-			(struct new_cpio_header *)(xformat_vfile_hdr(xux));
-	int ret; 
-	int retval; 
+	struct  new_cpio_header *file_hdr0;
+	intmax_t ret; 
+	intmax_t retval; 
+
 	XFORMAT_E_DEBUG("");
+	file_hdr0 = (struct new_cpio_header *)(xformat_vfile_hdr(xux));
 	ret = taru_write_archive_member_data(xux->taruM, file_hdr0,
 		dst_fd, xux->ifdM, (int(*)(int))NULL, xux->format_codeM, -1, NULL);
 	if (ret < 0)
@@ -791,14 +803,14 @@ xformat_read_file_data(XFORMAT * xux, int dst_fd)
 	return retval;
 }
 
-int
+intmax_t
 xformat_write_file_data(XFORMAT * xux, int source_fd)
 {
-	int ret;
-	struct new_cpio_header *file_hdr0 = 
-			(struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
+	intmax_t ret;
+	struct new_cpio_header *file_hdr0;
 	    
 	XFORMAT_E_DEBUG("");
+	file_hdr0 = (struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
 	ret = taru_write_archive_member_data(xux->taruM, file_hdr0,
 				xux->ofdM, source_fd, (int(*)(int))NULL,
 					xux->output_format_codeM, -1, NULL);
@@ -806,7 +818,7 @@ xformat_write_file_data(XFORMAT * xux, int source_fd)
 	if (source_fd == xux->ifdM ) { 
 		taru_tape_skip_padding(xux->ifdM, file_hdr0->c_filesize,
 						xux->output_format_codeM);
-		}
+	}
 	xux->bytes_writtenM += ret;
 	XFORMAT_E_DEBUG("LEAVING");
 	return ret;
@@ -852,7 +864,7 @@ xformat_write_header_wn(XFORMAT * xux, char * name)
 	}
 }
 
-int
+intmax_t
 xformat_write_file_by_fd (XFORMAT * xux,
 			struct stat *t,
 			char * name,
@@ -862,7 +874,8 @@ xformat_write_file_by_fd (XFORMAT * xux,
         struct  new_cpio_header * hdr0 = ahsStaticCreateFilehdr();
 	struct new_cpio_header *hdr1;
 	struct stat *lt1;
-	int ret, ret1;
+	intmax_t ret;
+	intmax_t ret1;
 	XFORMAT_E_DEBUG("");
 
         if (source_fd < 0 && fout == NULL) {
@@ -888,20 +901,22 @@ xformat_write_file_by_fd (XFORMAT * xux,
 					hdr0, 
 					xux->taruM->taru_tarheaderflagsM)) > 0)
 		{
-			ret1=taru_write_archive_member_data(xux->taruM,
+			ret1 = taru_write_archive_member_data(xux->taruM,
 					hdr0, xux->ofdM, source_fd,
 					(int(*)(int))fout,
 					xux->output_format_codeM, -1, NULL);
 			if (ret1 < 0) {
 				fprintf(stderr, "taru_write_archive_member_data failed unexpectedly\n");
-				return ret;
+				return -1;
 			}
 			ret += ret1;
 		} else {
-			fprintf(stderr, "taru_write_archive_member_header failed return code=%d\n", ret);
+			fprintf(stderr, "taru_write_archive_member_header failed return code=%s\n", swlib_imaxtostr(ret, NULL));
+			return -1;
 		}
 	} else {
 		fprintf(stderr, "common_setup_write_header failed\n");
+		return -1;
 	}
 	xux->bytes_writtenM += ret;
 	ahsStaticDeleteFilehdr(hdr0);
@@ -909,10 +924,10 @@ xformat_write_file_by_fd (XFORMAT * xux,
         return ret;
 }
 
-int
+intmax_t
 xformat_write_by_name(XFORMAT * xux, char * name, struct stat *st)
 {
-	int  ret;
+	intmax_t ret;
 	
 	XFORMAT_E_DEBUG("");
 	ret = taru_write_archive_member(xux->taruM, name,
@@ -927,18 +942,18 @@ xformat_write_by_name(XFORMAT * xux, char * name, struct stat *st)
 					xux->taruM->taru_tarheaderflagsM	
 					);
 	
-	if (ret>0) xux->bytes_writtenM += ret;
+	if (ret > 0) xux->bytes_writtenM += ret;
 	XFORMAT_E_DEBUG("LEAVING");
 	return ret;
 }
 
-int
+intmax_t
 xformat_write_by_fd(XFORMAT * xux, int srcfd,
 			struct new_cpio_header * file_hdr)
 {
-	int ret;
+	intmax_t ret;
 	XFORMAT_E_DEBUG("");
-	ret =  taru_write_archive_member(xux->taruM, (char*)(NULL),
+	ret = taru_write_archive_member(xux->taruM, (char*)(NULL),
 					(struct stat*)(NULL),
 					file_hdr,
 					xux->link_recordM, 
@@ -949,16 +964,16 @@ xformat_write_by_fd(XFORMAT * xux, int srcfd,
 					xux->output_format_codeM,
 					xux->taruM->taru_tarheaderflagsM	
 					);
-	if (ret>0) xux->bytes_writtenM += ret;
+	if (ret > 0) xux->bytes_writtenM += ret;
 	XFORMAT_E_DEBUG("LEAVING");
 	return ret;
 }
 
-int 
+intmax_t
 xformat_write_file(XFORMAT * xux, struct stat *t, char * name, char * source)
 {
-        int ret=0, ret1, fd=-1; 
-        struct  new_cpio_header * hdr0 = ahsStaticCreateFilehdr();
+        intmax_t ret=0, ret1, fd=-1; 
+        struct new_cpio_header * hdr0 = ahsStaticCreateFilehdr();
 	struct new_cpio_header *hdr1;
 	struct stat *lt1;
 		
@@ -1035,6 +1050,10 @@ void
 xformat_write_archive_stats(XFORMAT * xux, char * name, int fd)
 {
 	char * format;
+	char buf[UINTMAX_STRSIZE_BOUND];
+	char buf2[UINTMAX_STRSIZE_BOUND];
+	char * buf_p;
+	char * buf2_p;
 	STROB * tmp = strob_open(10);
 
 	switch(xux->output_format_codeM) {
@@ -1065,9 +1084,11 @@ xformat_write_archive_stats(XFORMAT * xux, char * name, int fd)
 			break;
 	}
 
-	swlib_writef(fd, tmp, "%s: %s vol 1, %d+%d %d octets out.\n", 
-		name, format, xux->bytes_writtenM - xux->trailer_bytesM,
-				xux->trailer_bytesM, xux->bytes_writtenM);
+	buf_p = umaxtostr(xux->bytes_writtenM, buf);
+	buf2_p = umaxtostr((uintmax_t)(xux->bytes_writtenM - xux->trailer_bytesM), buf2);
+
+	swlib_writef(fd, tmp, "%s: %s vol 1, %s+%d %s octets out.\n", 
+		name, format, buf2_p, xux->trailer_bytesM, buf_p);
 	strob_close(tmp);
 }
 
@@ -1253,27 +1274,30 @@ xformat_read(XFORMAT * xux, void * buf, size_t count)
 	return uxfio_sfread(xux->ifdM, buf, count);
 }
 
-int 
+intmax_t
 xformat_copy_pass_thru(XFORMAT * xux)
 {
 	XFORMAT_E_DEBUG("");
 	return xformat_copy_pass(xux, xux->ofdM, xux->ifdM);
 }
 
-int 
+intmax_t
 xformat_copy_pass_by_dst(XFORMAT * xux, int dst_fd)
 {
 	XFORMAT_E_DEBUG("");
 	return xformat_copy_pass(xux, dst_fd, xux->ifdM);
 }
 
-int 
+intmax_t 
 xformat_copy_pass_md5(XFORMAT * xux, int dst_fd, int src_fd, char * md5buf)
 {
-	int retval=0;
-	struct  new_cpio_header *file_hdr0 = 
-		(struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
+	intmax_t retval=0;
+	FILE_DIGS * digs;
+	FILE_DIGS * old_digs;
+	struct  new_cpio_header * file_hdr0;
 
+	file_hdr0 = (struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
+	
 	XFORMAT_E_DEBUG("");
 	if ((file_hdr0->c_mode & CP_IFMT) == CP_IFLNK) {
 		if (xux->format_codeM <= (int)(arf_crcascii)) {
@@ -1283,15 +1307,21 @@ xformat_copy_pass_md5(XFORMAT * xux, int dst_fd, int src_fd, char * md5buf)
      
 	if (dst_fd < 0) {
 		if (taru_read_amount(src_fd, file_hdr0->c_filesize) !=
-					(int)(file_hdr0->c_filesize)) {
+					(intmax_t)(file_hdr0->c_filesize)) {
 			XFORMAT_E_FAIL("");
 			retval = -3;
 		}
 	} else {
+		digs = taru_digs_create();
+		taru_digs_init(digs, DIGS_ENABLE_OFF);
+		digs->do_md5 = DIGS_ENABLE_ON;
+		old_digs = file_hdr0->digsM;
+		file_hdr0->digsM = digs;
 		retval = taru_write_archive_member_data(xux->taruM,
 				file_hdr0, dst_fd, src_fd,
 				(int(*)(int))NULL,
-				xux->output_format_codeM, -1, md5buf); 
+				xux->output_format_codeM, -1, digs); 
+		file_hdr0->digsM = old_digs;
 	}
 
 	if (uxfio_getfd(xux->ifdM, (int*)(NULL)) == src_fd ||
@@ -1299,16 +1329,29 @@ xformat_copy_pass_md5(XFORMAT * xux, int dst_fd, int src_fd, char * md5buf)
 		taru_tape_skip_padding(xux->ifdM,
 				file_hdr0->c_filesize, xux->format_codeM);
 	}
+	if (md5buf) {
+		strcpy(md5buf, digs->md5);
+	}
 	if (retval > 0) xux->bytes_writtenM += retval;
+	free(digs);
 	return retval;
 }
 
-int 
+intmax_t
 xformat_copy_pass(XFORMAT * xux, int dst_fd, int src_fd)
 {
-	int retval=0;
-	struct  new_cpio_header *file_hdr0 = 
-		(struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
+	intmax_t ret;
+	ret = xformat_copy_pass_digs(xux, dst_fd, src_fd, (FILE_DIGS *)NULL);
+	return ret;
+}
+
+intmax_t
+xformat_copy_pass_digs(XFORMAT * xux, int dst_fd, int src_fd, FILE_DIGS * digs)
+{
+	intmax_t retval=0;
+	struct  new_cpio_header *file_hdr0;
+
+	file_hdr0 = (struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
 
 	XFORMAT_E_DEBUG("");
 	if ((file_hdr0->c_mode & CP_IFMT) == CP_IFLNK) {
@@ -1317,34 +1360,41 @@ xformat_copy_pass(XFORMAT * xux, int dst_fd, int src_fd)
 		}
 	}
      
+	XFORMAT_E_DEBUG("");
 	if (dst_fd < 0) {
+		XFORMAT_E_DEBUG("");
 		if (taru_read_amount(src_fd, file_hdr0->c_filesize) !=
-					(int)(file_hdr0->c_filesize)) {
+					(intmax_t)(file_hdr0->c_filesize)) {
 			XFORMAT_E_FAIL("");
 			retval = -3;
 		}
 	} else {
+		XFORMAT_E_DEBUG("");
 		retval = taru_write_archive_member_data(xux->taruM,
 				file_hdr0, dst_fd, src_fd,
 				(int(*)(int))NULL,
-				xux->output_format_codeM, -1, NULL); 
+				xux->output_format_codeM, -1, digs); 
 	} 
+	XFORMAT_E_DEBUG("");
 
 	if (uxfio_getfd(xux->ifdM, (int*)(NULL)) == src_fd ||
 						xux->ifdM == src_fd ) { 
+		XFORMAT_E_DEBUG("");
 		taru_tape_skip_padding(xux->ifdM,
 				file_hdr0->c_filesize, xux->format_codeM);
 	}
 	if (retval > 0) xux->bytes_writtenM += retval;
+	XFORMAT_E_DEBUG2("retval=%d", retval);
 	return retval;
 }
 
-int 
+intmax_t 
 xformat_copy_pass_file_data(XFORMAT * xux, int dst_fd, int src_fd)
 {
-	int retval=0;
-	struct  new_cpio_header *file_hdr0 = 
-		(struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
+	intmax_t retval=0;
+	struct  new_cpio_header *file_hdr0;
+
+	file_hdr0 = (struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
 
 	XFORMAT_E_DEBUG("");
 	if ((file_hdr0->c_mode & CP_IFMT) == CP_IFLNK) {
@@ -1355,7 +1405,7 @@ xformat_copy_pass_file_data(XFORMAT * xux, int dst_fd, int src_fd)
      
 	if (dst_fd < 0) {
 		if (taru_read_amount(src_fd, file_hdr0->c_filesize) !=
-					(int)(file_hdr0->c_filesize)) {
+					(intmax_t)(file_hdr0->c_filesize)) {
 			XFORMAT_E_FAIL("");
 			retval = -3;
 		}
@@ -1375,12 +1425,13 @@ xformat_copy_pass_file_data(XFORMAT * xux, int dst_fd, int src_fd)
 	return retval;
 }
 
-int 
+intmax_t 
 xformat_copy_pass2(XFORMAT * xux, int dst_fd, int src_fd, int adjunct_ofd)
 {
-	int retval=0;
-	struct  new_cpio_header *file_hdr0 =
-			(struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
+	intmax_t retval=0;
+	struct  new_cpio_header * file_hdr0;
+
+	file_hdr0 = (struct new_cpio_header *)(xformat_vfile_hdr(xux)); 
 
 	XFORMAT_E_DEBUG("");
 	if ((file_hdr0->c_mode & CP_IFMT) == CP_IFLNK) {
@@ -1473,7 +1524,7 @@ xformat_set_sys_db_g_policy(XFORMAT * xux, int c)
 	ahs_set_sys_db_g_policy(xux->ahsM, c);
 }
 
-size_t
+intmax_t
 xformat_get_filesize (XFORMAT * xux)
 {
 	XFORMAT_E_DEBUG("");
