@@ -7,6 +7,8 @@
 #include "uinfile.h"
 #include "swlib.h"
 #include "taru.h"
+#include "swevents_array.h"
+#include "swcommon_options.h"
 
 int 
 main (int argc, char ** argv ) {
