@@ -30,7 +30,7 @@ echo
 
 (
 	export SWPACKAGEPASSFD=3
-	exec 3<$PASSFILE swign -u "$AUTHOR" -D bin/checksig.sh @-
+	exec 3<$PASSFILE swign -s . -u "$AUTHOR" -D bin/checksig.sh @-
 ) | swcopy -Waudit -s - @- | swverify --checksig
 
 ret=$?
